@@ -30,10 +30,10 @@ FREI0R_REVISION=
 ENABLE_MOVIT=1
 SUBDIRS=
 MOVIT_HEAD=0
-MOVIT_REVISION=origin/shotcut
+MOVIT_REVISION="origin/shotcut"
 ENABLE_SWH_PLUGINS=0
 FFMPEG_HEAD=0
-FFMPEG_REVISION="origin/release/4.2"
+FFMPEG_REVISION="origin/release/4.4"
 FFMPEG_ADDITIONAL_OPTIONS=
 MLT_HEAD=1
 MLT_REVISION=
@@ -47,10 +47,13 @@ BIGSH0T_REVISION=
 ENABLE_ZIMG=1
 ZIMG_HEAD=1
 ZIMG_REVISION=
-DAV1D_HEAD=1
-DAV1D_REVISION=
-AOM_HEAD=1
-AOM_REVISION=
+DAV1D_HEAD=0
+DAV1D_REVISION="0.9.2"
+AOM_HEAD=0
+AOM_REVISION="v3.2.0"
+ENABLE_VMAF=1
+VMAF_HEAD=0
+VMAF_REVISION="v2.2.0"
 
 
 # QT_INCLUDE_DIR="$(pkg-config --variable=prefix QtCore)/include"
@@ -173,6 +176,9 @@ function to_key {
     aom)
       echo 11
     ;;
+    vmaf)
+      echo 12
+    ;;
     *)
       echo UNKNOWN
     ;;
@@ -344,6 +350,9 @@ function set_globals {
     if test "$ENABLE_ZIMG" = 1 ; then
         SUBDIRS="zimg $SUBDIRS"
     fi
+    if test "$ENABLE_VMAF" = 1 ; then
+        SUBDIRS="vmaf $SUBDIRS"
+    fi
     SUBDIRS="$SUBDIRS mlt shotcut"
   fi
 
@@ -354,6 +363,7 @@ function set_globals {
   else
     CONFIGURE_DEBUG_FLAG=
     QMAKE_DEBUG_FLAG=
+    CMAKE_DEBUG_FLAG="-DCMAKE_BUILD_TYPE=Release"
   fi
 
   if [ "$ASAN_BUILD" = "1" ]; then
@@ -369,19 +379,20 @@ function set_globals {
   debug "SUBDIRS = $SUBDIRS"
 
   # REPOLOCS Array holds the repo urls
-  REPOLOCS[0]="git://github.com/FFmpeg/FFmpeg.git"
-  REPOLOCS[1]="git://github.com/mltframework/mlt.git"
-  REPOLOCS[2]="git://github.com/dyne/frei0r.git"
-  REPOLOCS[3]="git://github.com/ddennedy/movit.git"
-  REPOLOCS[4]="git://github.com/mltframework/shotcut.git"
+  REPOLOCS[0]="https://github.com/FFmpeg/FFmpeg.git"
+  REPOLOCS[1]="https://github.com/mltframework/mlt.git"
+  REPOLOCS[2]="https://github.com/dyne/frei0r.git"
+  REPOLOCS[3]="https://github.com/ddennedy/movit.git"
+  REPOLOCS[4]="https://github.com/mltframework/shotcut.git"
   REPOLOCS[5]="http://ftp.us.debian.org/debian/pool/main/s/swh-plugins/swh-plugins_0.4.15+1.orig.tar.gz"
-  REPOLOCS[6]="git://github.com/FFmpeg/nv-codec-headers.git"
-  REPOLOCS[7]="git://github.com/GPUOpen-LibrariesAndSDKs/AMF.git"
+  REPOLOCS[6]="https://github.com/FFmpeg/nv-codec-headers.git"
+  REPOLOCS[7]="https://github.com/GPUOpen-LibrariesAndSDKs/AMF.git"
 #  REPOLOCS[8]="https://bitbucket.org/dandennedy/bigsh0t.git"
   REPOLOCS[8]="https://bitbucket.org/leo_sutic/bigsh0t.git"
-  REPOLOCS[9]="git://github.com/sekrit-twc/zimg.git"
+  REPOLOCS[9]="https://github.com/sekrit-twc/zimg.git"
   REPOLOCS[10]="https://code.videolan.org/videolan/dav1d.git"
   REPOLOCS[11]="https://aomedia.googlesource.com/aom"
+  REPOLOCS[12]="https://github.com/Netflix/vmaf.git"
 
   # REPOTYPE Array holds the repo types. (Yes, this might be redundant, but easy for me)
   REPOTYPES[0]="git"
@@ -396,6 +407,7 @@ function set_globals {
   REPOTYPES[9]="git"
   REPOTYPES[10]="git"
   REPOTYPES[11]="git"
+  REPOTYPES[12]="git"
 
   # And, set up the revisions
   REVISIONS[0]=""
@@ -437,6 +449,10 @@ function set_globals {
   if test 0 = "$AOM_HEAD" -a "$AOM_REVISION" ; then
     REVISIONS[11]="$AOM_REVISION"
   fi
+  REVISIONS[12]=""
+  if test 0 = "$VMAF_HEAD" -a "$VMAF_REVISION" ; then
+    REVISIONS[12]="$VMAF_REVISION"
+  fi
 
   # Figure out the number of cores in the system. Used both by make and startup script
   CPUS=$(nproc)
@@ -471,20 +487,20 @@ function set_globals {
   #####
   # ffmpeg
   CONFIG[0]="./configure --prefix=$FINAL_INSTALL_DIR --disable-static --disable-doc --enable-gpl --enable-version3 --enable-shared --enable-runtime-cpudetect $CONFIGURE_DEBUG_FLAG"
-  CONFIG[0]="${CONFIG[0]} --enable-libtheora --enable-libvorbis --enable-libmp3lame --enable-libx264 --enable-libx265 --enable-libvpx --enable-libopus --enable-libmfx --enable-libdav1d --enable-libaom --disable-decoder=libaom_av1"
+  CONFIG[0]="${CONFIG[0]} --enable-libtheora --enable-libvorbis --enable-libmp3lame --enable-libx264 --enable-libx265 --enable-libvpx --enable-libopus --enable-libmfx --enable-libdav1d --enable-libaom --disable-decoder=libaom_av1 --enable-libwebp"
   # Add optional parameters
   if [ "$ENABLE_ZIMG" = "1" ]; then
     CONFIG[0]="${CONFIG[0]} --enable-libzimg"
   fi
+  if [ "$ENABLE_VMAF" = "1" ]; then
+    CONFIG[0]="${CONFIG[0]} --enable-libvmaf --disable-w32threads"
+  fi
   CONFIG[0]="${CONFIG[0]} $FFMPEG_ADDITIONAL_OPTIONS"
   CFLAGS_[0]="-I$FINAL_INSTALL_DIR/include $CFLAGS"
 
   #####
   # mlt
-  CONFIG[1]="./configure --prefix=$FINAL_INSTALL_DIR --enable-gpl --enable-gpl3 --without-kde --disable-sdl --disable-gdk --disable-gtk2 --disable-kino --disable-vorbis $CONFIGURE_DEBUG_FLAG"
-  # Remember, if adding more of these, to update the post-configure check.
-  [ "$QT_INCLUDE_DIR" ] && CONFIG[1]="${CONFIG[1]} --qt-includedir=$QT_INCLUDE_DIR"
-  [ "$QT_LIB_DIR" ] && CONFIG[1]="${CONFIG[1]} --qt-libdir=$QT_LIB_DIR"
+  CONFIG[1]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR -DCMAKE_PREFIX_PATH=$QTDIR -DGPL=ON -DGPL3=ON -DMOD_GDK=OFF -DMOD_SDL1=OFF $CMAKE_DEBUG_FLAG"
   CFLAGS_[1]="-I$FINAL_INSTALL_DIR/include $ASAN_CFLAGS $CFLAGS"
   LDFLAGS_[1]="${LDFLAGS_[1]} -L$FINAL_INSTALL_DIR/lib $ASAN_LDFLAGS $LDFLAGS"
 
@@ -551,6 +567,17 @@ function set_globals {
   CONFIG[11]="cmake -GNinja -DCMAKE_INSTALL_PREFIX=$FINAL_INSTALL_DIR $CMAKE_DEBUG_FLAG -DBUILD_SHARED_LIBS=1 -DCONFIG_AV1_DECODER=0 -DENABLE_EXAMPLES=0 -DENABLE_TESTS=0 ../aom"
   CFLAGS_[11]=$CFLAGS
   LDFLAGS_[11]=$LDFLAGS
+
+  #####
+  # vmaf
+  CONFIG[12]="meson setup libvmaf/build libvmaf --prefix=$FINAL_INSTALL_DIR --libdir=$FINAL_INSTALL_DIR/lib"
+  if [ "$DEBUG_BUILD" = "1" ]; then
+    CONFIG[12]="${CONFIG[12]} --buildtype=debug"
+  else
+    CONFIG[12]="${CONFIG[12]} --buildtype=release"
+  fi
+  CFLAGS_[12]=$CFLAGS
+  LDFLAGS_[12]=$LDFLAGS
 }
 
 ######################################################################
@@ -871,10 +898,12 @@ function configure_compile_install_subproject {
   feedback_status Building $1 - this could take some time
   if test "movit" = "$1" ; then
     cmd make -j$MAKEJ libmovit.la || die "Unable to build $1"
-  elif test "frei0r" = "$1" -o "bigsh0t" = "$1" -o "aom" = "$1"; then
+  elif test "frei0r" = "$1" -o "bigsh0t" = "$1" -o "aom" = "$1" -o "mlt" = "$1"; then
     cmd ninja -j $MAKEJ || die "Unable to build $1"
   elif test "dav1d" = "$1"; then
     cmd ninja -C builddir -j $MAKEJ || die "Unable to build $1"
+  elif test "vmaf" = "$1"; then
+    cmd ninja -C libvmaf/build -j $MAKEJ || die "Unable to build $1"
   elif test "$MYCONFIG" != ""; then
     cmd make -j$MAKEJ || die "Unable to build $1"
   fi
@@ -894,12 +923,16 @@ function configure_compile_install_subproject {
     cmd install -p -c translations/*.qm "$FINAL_INSTALL_DIR"/share/translations
     cmd install -d "$FINAL_INSTALL_DIR"/share/shotcut
     cmd cp -a src/qml "$FINAL_INSTALL_DIR"/share/shotcut
-  elif test "frei0r" = "$1" -o "aom" = "$1"; then
+  elif test "frei0r" = "$1" -o "aom" = "$1" -o "mlt" = "$1"; then
     cmd ninja install || die "Unable to install $1"
   elif test "bigsh0t" = "$1" ; then
     cmd install -p -c *.dll "$FINAL_INSTALL_DIR"/lib/frei0r-1  || die "Unable to install $1"
   elif test "dav1d" = "$1"; then
     cmd meson install -C builddir || die "Unable to install $1"
+  elif test "vmaf" = "$1"; then
+    cmd ninja install -C libvmaf/build || die "Unable to install $1"
+    cmd install -d "$FINAL_INSTALL_DIR"/share/vmaf
+    cmd install -p -c model/*.json "$FINAL_INSTALL_DIR"/share/vmaf || die "Unable to install $1"
   elif test "$MYCONFIG" != "" ; then
     cmd make install || die "Unable to install $1"
   fi
@@ -1020,7 +1053,7 @@ function deploy
 
   log Reorganizing installed files
   cmd mv bin/*.dll .
-  cmd mv lib/*.dll .
+  cmd mv lib/libaom.dll .
   if [ "$SDK" = "1" ]; then
     cmd mv bin/*.exe .
   else
@@ -1032,9 +1065,6 @@ function deploy
     cmd rm -rf lib/cmake lib/pkgconfig lib/gdk-pixbuf-2.0 lib/glib-2.0 lib/gtk-2.0
     cmd rm -rf share/doc share/man share/ffmpeg/examples share/aclocal share/glib-2.0 share/gtk-2.0 share/gtk-doc share/themes share/locale
   fi
-  for exe in *.exe; do
-    cmd touch "${exe}.local"
-  done
   cmd mv COPYING COPYING.txt
 
   log Copying some libs from Qt
