@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2020 Meltytech, LLC
+ * Copyright (c) 2013-2021 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,11 +17,14 @@
 
 #include "dialogs/longuitask.h"
 #include "timelinecommands.h"
+#include "mainwindow.h"
 #include "mltcontroller.h"
 #include "shotcut_mlt_properties.h"
 #include "settings.h"
 #include "proxymanager.h"
 #include "dialogs/longuitask.h"
+#include "controllers/filtercontroller.h"
+#include "qmltypes/qmlmetadata.h"
 #include <Logger.h>
 
 
@@ -51,7 +54,7 @@ void AppendCommand::redo()
     LongUiTask longTask(QObject::tr("Append to Timeline"));
     m_undoHelper.recordBeforeState();
     Mlt::Producer* producer = longTask.runAsync<Mlt::Producer*>(QObject::tr("Preparing"), deserializeProducer, m_xml);
-    if (producer->type() == playlist_type) {
+    if (producer->type() == mlt_service_playlist_type) {
         Mlt::Playlist playlist(*producer);
         int count = playlist.count();
         for (int i = 0; i < count; i++) {
@@ -77,26 +80,31 @@ void AppendCommand::undo()
     m_undoHelper.undoChanges();
 }
 
-InsertCommand::InsertCommand(MultitrackModel &model, int trackIndex,
+InsertCommand::InsertCommand(MultitrackModel &model, MarkersModel& markersModel, int trackIndex,
     int position, const QString &xml, bool seek, QUndoCommand *parent)
     : QUndoCommand(parent)
     , m_model(model)
+    , m_markersModel(markersModel)
     , m_trackIndex(qBound(0, trackIndex, qMax(model.rowCount() - 1, 0)))
     , m_position(position)
     , m_xml(xml)
     , m_undoHelper(m_model)
     , m_seek(seek)
     , m_rippleAllTracks(Settings.timelineRippleAllTracks())
+    , m_rippleMarkers(Settings.timelineRippleMarkers())
+    , m_markersShift(0)
 {
     setText(QObject::tr("Insert into track"));
+    m_undoHelper.setHints(UndoHelper::RestoreTracks);
 }
 
 void InsertCommand::redo()
 {
     LOG_DEBUG() << "trackIndex" << m_trackIndex << "position" << m_position;
+    int shift = 0;
     m_undoHelper.recordBeforeState();
     Mlt::Producer clip(MLT.profile(), "xml-string", m_xml.toUtf8().constData());
-    if (clip.type() == playlist_type) {
+    if (clip.type() == mlt_service_playlist_type) {
         LongUiTask longTask(QObject::tr("Add Files"));
         Mlt::Playlist playlist(clip);
         int n = playlist.count();
@@ -107,19 +115,29 @@ void InsertCommand::redo()
             longTask.reportProgress(QFileInfo(ProxyManager::resource(clip)).fileName(), n - i - 1, n);
             ProxyManager::generateIfNotExists(clip);
             clip.set_in_and_out(info->frame_in, info->frame_out);
-            m_model.insertClip(m_trackIndex, clip, m_position, m_rippleAllTracks, false);
+            bool lastClip = i == 0;
+            m_model.insertClip(m_trackIndex, clip, m_position, m_rippleAllTracks, false, lastClip);
+            shift += clip.get_playtime();
         }
     } else {
         ProxyManager::generateIfNotExists(clip);
         m_model.insertClip(m_trackIndex, clip, m_position, m_rippleAllTracks, m_seek);
+        shift = clip.get_playtime();
     }
     m_undoHelper.recordAfterState();
+    if (m_rippleMarkers && shift > 0) {
+        m_markersShift = shift;
+        m_markersModel.doShift(m_position, m_markersShift);
+    }
 }
 
 void InsertCommand::undo()
 {
     LOG_DEBUG() << "trackIndex" << m_trackIndex << "position" << m_position;
     m_undoHelper.undoChanges();
+    if (m_rippleMarkers && m_markersShift > 0) {
+        m_markersModel.doShift(m_position + m_markersShift, -m_markersShift);
+    }
 }
 
 OverwriteCommand::OverwriteCommand(MultitrackModel &model, int trackIndex,
@@ -133,6 +151,7 @@ OverwriteCommand::OverwriteCommand(MultitrackModel &model, int trackIndex,
     , m_seek(seek)
 {
     setText(QObject::tr("Overwrite onto track"));
+    m_undoHelper.setHints(UndoHelper::RestoreTracks);
 }
 
 void OverwriteCommand::redo()
@@ -140,7 +159,7 @@ void OverwriteCommand::redo()
     LOG_DEBUG() << "trackIndex" << m_trackIndex << "position" << m_position;
     m_undoHelper.recordBeforeState();
     Mlt::Producer clip(MLT.profile(), "xml-string", m_xml.toUtf8().constData());
-    if (clip.type() == playlist_type) {
+    if (clip.type() == mlt_service_playlist_type) {
         LongUiTask longTask(QObject::tr("Add Files"));
         Mlt::Playlist playlist(clip);
         int position = m_position;
@@ -151,7 +170,8 @@ void OverwriteCommand::redo()
             longTask.reportProgress(QFileInfo(ProxyManager::resource(clip)).fileName(), i, n);
             ProxyManager::generateIfNotExists(clip);
             clip.set_in_and_out(info->frame_in, info->frame_out);
-            m_model.overwrite(m_trackIndex, clip, position, false);
+            bool lastClip = i == (n - 1);
+            m_model.overwrite(m_trackIndex, clip, position, false, lastClip);
             position += info->frame_count;
         }
     } else {
@@ -193,14 +213,18 @@ void LiftCommand::undo()
     m_undoHelper.undoChanges();
 }
 
-RemoveCommand::RemoveCommand(MultitrackModel &model, int trackIndex,
+RemoveCommand::RemoveCommand(MultitrackModel &model, MarkersModel& markersModel, int trackIndex,
     int clipIndex, QUndoCommand *parent)
     : QUndoCommand(parent)
     , m_model(model)
+    , m_markersModel(markersModel)
     , m_trackIndex(qBound(0, trackIndex, qMax(model.rowCount() - 1, 0)))
     , m_clipIndex(clipIndex)
     , m_undoHelper(m_model)
     , m_rippleAllTracks(Settings.timelineRippleAllTracks())
+    , m_rippleMarkers(Settings.timelineRippleMarkers())
+    , m_markerRemoveStart(-1)
+    , m_markerRemoveEnd(-1)
 {
     setText(QObject::tr("Remove from track"));
     m_undoHelper.setHints(UndoHelper::RestoreTracks);
@@ -209,6 +233,47 @@ RemoveCommand::RemoveCommand(MultitrackModel &model, int trackIndex,
 void RemoveCommand::redo()
 {
     LOG_DEBUG() << "trackIndex" << m_trackIndex << "clipIndex" << m_clipIndex;
+
+    if (m_rippleMarkers) {
+        // Remove and shift markers as appropriate
+        bool markersModified = false;
+        m_markers = m_markersModel.getMarkers();
+        if (m_markers.size() > 0) {
+            auto mlt_index = m_model.trackList().at(m_trackIndex).mlt_index;
+            QScopedPointer<Mlt::Producer> track(m_model.tractor()->track(mlt_index));
+            if (track && track->is_valid()) {
+                Mlt::Playlist playlist(*track);
+                m_markerRemoveStart = playlist.clip_start(m_clipIndex);
+                m_markerRemoveEnd = m_markerRemoveStart + playlist.clip_length(m_clipIndex);
+            }
+        }
+        if (m_markers.size() > 0 && m_markerRemoveStart >= 0) {
+            QList<Markers::Marker> newMarkers = m_markers;
+            for (int i = 0; i < newMarkers.size(); i++) {
+                Markers::Marker& marker = newMarkers[i];
+                if (marker.start >= m_markerRemoveStart &&
+                    marker.start <= m_markerRemoveEnd) {
+                    // This marker is in the removed segment. Remove it
+                    newMarkers.removeAt(i);
+                    i--;
+                    markersModified = true;
+                } else if (marker.start > m_markerRemoveEnd) {
+                    // This marker is after the removed segment. Shift it left
+                    marker.start -= m_markerRemoveEnd - m_markerRemoveStart;
+                    marker.end -= m_markerRemoveEnd - m_markerRemoveStart;
+                    markersModified = true;
+                }
+            }
+            if (markersModified) {
+                m_markersModel.doReplace(newMarkers);
+            }
+        }
+        if (!markersModified) {
+            m_markerRemoveStart = -1;
+            m_markers.clear();
+        }
+    }
+
     m_undoHelper.recordBeforeState();
     m_model.removeClip(m_trackIndex, m_clipIndex, m_rippleAllTracks);
     m_undoHelper.recordAfterState();
@@ -218,9 +283,11 @@ void RemoveCommand::undo()
 {
     LOG_DEBUG() << "trackIndex" << m_trackIndex << "clipIndex" << m_clipIndex;
     m_undoHelper.undoChanges();
+    if (m_rippleMarkers && m_markerRemoveStart >= 0) {
+        m_markersModel.doReplace(m_markers);
+    }
 }
 
-
 NameTrackCommand::NameTrackCommand(MultitrackModel &model, int trackIndex,
     const QString &name, QUndoCommand *parent)
     : QUndoCommand(parent)
@@ -355,15 +422,19 @@ void LockTrackCommand::undo()
     m_model.setTrackLock(m_trackIndex, m_oldValue);
 }
 
-MoveClipCommand::MoveClipCommand(MultitrackModel &model, int trackDelta, bool ripple, QUndoCommand *parent)
+MoveClipCommand::MoveClipCommand(MultitrackModel &model, MarkersModel &markersModel, int trackDelta, bool ripple, QUndoCommand *parent)
     : QUndoCommand(parent)
     , m_model(model)
+    , m_markersModel(markersModel)
     , m_trackDelta(trackDelta)
     , m_ripple(ripple)
     , m_rippleAllTracks(Settings.timelineRippleAllTracks())
+    , m_rippleMarkers(Settings.timelineRippleMarkers())
     , m_undoHelper(m_model)
     , m_redo(false)
     , m_start(-1)
+    , m_markerOldStart(-1)
+    , m_markerNewStart(-1)
 {
     m_undoHelper.setHints(UndoHelper::RestoreTracks);
     m_undoHelper.recordBeforeState();
@@ -377,7 +448,7 @@ void MoveClipCommand::redo()
 
     if (!m_redo) {
         if (m_selection.size() > 1)
-            setText(QObject::tr("Move %1 timelime clips").arg(m_selection.size()));
+            setText(QObject::tr("Move %n timelime clips", nullptr, m_selection.size()));
         else
             setText(QObject::tr("Move timelime clip"));
     }
@@ -399,6 +470,8 @@ void MoveClipCommand::redo()
                     m_start = newStart;
                     m_trackIndex = trackIndex;
                     m_clipIndex = clipIndex;
+                    m_markerOldStart = info->start;
+                    m_markerNewStart = newStart;
                 }
             }
         }
@@ -406,6 +479,7 @@ void MoveClipCommand::redo()
             // Use old behavior to push or pull clips on the same track.
             m_model.moveClip(m_trackIndex, m_trackIndex, m_clipIndex, m_start, m_ripple, m_rippleAllTracks);
             m_undoHelper.recordAfterState();
+            redoMarkers();
             m_redo = true;
             return;
         }
@@ -415,19 +489,24 @@ void MoveClipCommand::redo()
         if (!m_redo) {
             // On the initial pass, remove clips while recording info about them.
             QScopedPointer<Mlt::ClipInfo> info(m_model.findClipByUuid(MLT.uuid(clip), trackIndex, clipIndex));
-            if (info->producer && info->producer->is_valid()) {
-                info->producer->set(kPlaylistIndexProperty, qBound(0, trackIndex + m_trackDelta, m_model.trackList().size() - 1));
+            if (info && info->producer && info->producer->is_valid()) {
+                info->producer->set(kNewTrackIndexProperty, qBound(0, trackIndex + m_trackDelta, m_model.trackList().size() - 1));
                 info->producer->pass_property(*info->cut, kPlaylistStartProperty);
-                info->producer->set("_shotcut:trackIndex", trackIndex);
-                info->producer->set("_shotcut:clipIndex", clipIndex);
-                info->producer->set("_shotcut:in", info->frame_in);
-                info->producer->set("_shotcut:out", info->frame_out);
+                info->producer->set(kTrackIndexProperty, trackIndex);
+                info->producer->set(kClipIndexProperty, clipIndex);
+                info->producer->set(kShotcutInProperty, info->frame_in);
+                info->producer->set(kShotcutOutProperty, info->frame_out);
                 newSelection.insert(info->cut->get_int(kPlaylistStartProperty), info->producer);
+                if (m_markerOldStart < 0 || m_markerOldStart > info->start) {
+                    // Record the left most clip position being moved
+                    m_markerOldStart = info->start;
+                    m_markerNewStart = info->cut->get_int(kPlaylistStartProperty);
+                }
             }
         } else {
             // On redo, use the recorded indices to remove them.
-            trackIndex = clip.get_int("_shotcut:trackIndex");
-            clipIndex = clip.get_int("_shotcut:clipIndex");
+            trackIndex = clip.get_int(kTrackIndexProperty);
+            clipIndex = clip.get_int(kClipIndexProperty);
         }
         if (m_ripple)
             m_model.removeClip(trackIndex, clipIndex, m_rippleAllTracks);
@@ -439,11 +518,11 @@ void MoveClipCommand::redo()
     }
     // Next, add the save clips to their new locations.
     for (auto& clip : m_selection) {
-        auto toTrack = clip.get_int(kPlaylistIndexProperty);
+        auto toTrack = clip.get_int(kNewTrackIndexProperty);
         auto start = clip.get_int(kPlaylistStartProperty);
-        clip.set_in_and_out(clip.get_int("_shotcut:in"), clip.get_int("_shotcut:out"));
+        clip.set_in_and_out(clip.get_int(kShotcutInProperty), clip.get_int(kShotcutOutProperty));
         if (start + clip.get_playtime() >= 0) {
-//            LOG_DEBUG() << "adding clip" << clip.get_int("_shotcut:clipIndex") << "to track" << toTrack << "start" << start;
+//            LOG_DEBUG() << "adding clip" << clip.get_int(kClipIndexProperty) << "to track" << toTrack << "start" << start;
             if (m_ripple)
                 m_model.insertClip(toTrack, clip, start, m_rippleAllTracks);
             else
@@ -455,12 +534,48 @@ void MoveClipCommand::redo()
         m_redo = true;
         m_undoHelper.recordAfterState();
     }
+
+    redoMarkers();
 }
 
 void MoveClipCommand::undo()
 {
     LOG_DEBUG() << "track delta" << m_trackDelta;
     m_undoHelper.undoChanges();
+    if (m_rippleMarkers && m_markerOldStart >= 0) {
+        m_markersModel.doReplace(m_markers);
+    }
+}
+
+void MoveClipCommand::redoMarkers()
+{
+    if (m_rippleMarkers && m_markerOldStart >= 0) {
+        m_markers = m_markersModel.getMarkers();
+        QList<Markers::Marker> newMarkers = m_markers;
+        bool markersModified = false;
+        int startDelta = m_markerNewStart - m_markerOldStart;
+        for (int i = 0; i < newMarkers.size(); i++) {
+            Markers::Marker& marker = newMarkers[i];
+            if (marker.start < m_markerOldStart &&
+                marker.start > m_markerNewStart) {
+                // This marker is in the overwritten segment. Remove it
+                newMarkers.removeAt(i);
+                i--;
+                markersModified = true;
+            } else if (marker.start >= m_markerOldStart) {
+                // This marker is after the start of the moved segment. Shift it with the move
+                marker.start += startDelta;
+                marker.end += startDelta;
+                markersModified = true;
+            }
+        }
+        if (markersModified) {
+            m_markersModel.doReplace(newMarkers);
+        } else {
+            m_markerOldStart = -1;
+            m_markers.clear();
+        }
+    }
 }
 
 TrimClipInCommand::TrimClipInCommand(MultitrackModel &model, int trackIndex, int clipIndex, int delta, bool ripple, bool redo, QUndoCommand* parent)
@@ -481,7 +596,11 @@ void TrimClipInCommand::redo()
     if (m_redo) {
         LOG_DEBUG() << "trackIndex" << m_trackIndex << "clipIndex" << m_clipIndex << "delta" << m_delta;
         m_undoHelper.reset(new UndoHelper(m_model));
-        if (!m_ripple) m_undoHelper->setHints(UndoHelper::SkipXML);
+        if (m_ripple) {
+            m_undoHelper->setHints(UndoHelper::SkipXML);
+        } else {
+            m_undoHelper->setHints(UndoHelper::RestoreTracks);
+        }
         m_undoHelper->recordBeforeState();
         m_model.trimClipIn(m_trackIndex, m_clipIndex, m_delta, m_ripple, m_rippleAllTracks);
         m_undoHelper->recordAfterState();
@@ -568,6 +687,7 @@ SplitCommand::SplitCommand(MultitrackModel &model, int trackIndex,
     , m_undoHelper(m_model)
 {
     setText(QObject::tr("Split clip"));
+    m_undoHelper.setHints(UndoHelper::RestoreTracks);
 }
 
 void SplitCommand::redo()
@@ -653,6 +773,8 @@ bool FadeOutCommand::mergeWith(const QUndoCommand *other)
 AddTransitionCommand::AddTransitionCommand(TimelineDock& timeline, int trackIndex, int clipIndex, int position, bool ripple, QUndoCommand *parent)
     : QUndoCommand(parent)
     , m_timeline(timeline)
+    , m_model(*m_timeline.model())
+    , m_markersModel(*m_timeline.markersModel())
     , m_trackIndex(trackIndex)
     , m_clipIndex(clipIndex)
     , m_position(position)
@@ -660,6 +782,9 @@ AddTransitionCommand::AddTransitionCommand(TimelineDock& timeline, int trackInde
     , m_ripple(ripple)
     , m_undoHelper(*m_timeline.model())
     , m_rippleAllTracks(Settings.timelineRippleAllTracks())
+    , m_rippleMarkers(Settings.timelineRippleMarkers())
+    , m_markerOldStart(-1)
+    , m_markerNewStart(-1)
 {
     setText(QObject::tr("Add transition"));
 }
@@ -667,10 +792,52 @@ AddTransitionCommand::AddTransitionCommand(TimelineDock& timeline, int trackInde
 void AddTransitionCommand::redo()
 {
     LOG_DEBUG() << "trackIndex" << m_trackIndex << "clipIndex" << m_clipIndex << "position" << m_position;
+
+    if (m_rippleMarkers) {
+        // Calculate the marker delta before moving anything
+        auto mlt_index = m_model.trackList().at(m_trackIndex).mlt_index;
+        QScopedPointer<Mlt::Producer> track(m_model.tractor()->track(mlt_index));
+        if (track && track->is_valid()) {
+            Mlt::Playlist playlist(*track);
+            m_markerOldStart = playlist.clip_start(m_clipIndex);
+            m_markerNewStart = m_position;
+        }
+    }
+
     m_undoHelper.recordBeforeState();
-    m_transitionIndex = m_timeline.model()->addTransition(m_trackIndex, m_clipIndex, m_position, m_ripple, m_rippleAllTracks);
+    m_transitionIndex = m_model.addTransition(m_trackIndex, m_clipIndex, m_position, m_ripple, m_rippleAllTracks);
     LOG_DEBUG() << "m_transitionIndex" << m_transitionIndex;
     m_undoHelper.recordAfterState();
+
+    // Remove and shift markers as appropriate
+    bool markersModified = false;
+    if (m_transitionIndex >= 0 && m_rippleMarkers && m_markerOldStart >= 0) {
+        m_markers = m_markersModel.getMarkers();
+        QList<Markers::Marker> newMarkers = m_markers;
+        int startDelta = m_markerNewStart - m_markerOldStart;
+        for (int i = 0; i < newMarkers.size(); i++) {
+            Markers::Marker& marker = newMarkers[i];
+            if (marker.start <= m_markerOldStart &&
+                marker.start > m_markerNewStart) {
+                // This marker is in the overwritten segment. Remove it
+                newMarkers.removeAt(i);
+                i--;
+                markersModified = true;
+            } else if (marker.start >= m_markerOldStart) {
+                // This marker is after the start of the moved segment. Shift it with the move
+                marker.start += startDelta;
+                marker.end += startDelta;
+                markersModified = true;
+            }
+        }
+        if (markersModified) {
+            m_markersModel.doReplace(newMarkers);
+        }
+    }
+    if (!markersModified) {
+        m_markerOldStart = -1;
+        m_markers.clear();
+    }
 }
 
 void AddTransitionCommand::undo()
@@ -681,6 +848,10 @@ void AddTransitionCommand::undo()
         m_timeline.setSelection();
         m_undoHelper.undoChanges();
         m_timeline.setSelection(QList<QPoint>() << QPoint(m_clipIndex, m_trackIndex));
+
+        if (m_rippleMarkers && m_markerOldStart >= 0) {
+            m_markersModel.doReplace(m_markers);
+        }
     }
 }
 
@@ -958,12 +1129,15 @@ void AddTrackCommand::undo()
     m_model.removeTrack(m_trackIndex);
 }
 
-InsertTrackCommand::InsertTrackCommand(MultitrackModel& model, int trackIndex, QUndoCommand* parent)
+InsertTrackCommand::InsertTrackCommand(MultitrackModel& model, int trackIndex, TrackType trackType, QUndoCommand* parent)
     : QUndoCommand(parent)
     , m_model(model)
     , m_trackIndex(trackIndex)
-    , m_trackType(model.trackList().size() > 0 ? model.trackList().at(trackIndex).type : VideoTrackType)
+    , m_trackType(trackType)
 {
+    if (trackType != AudioTrackType && trackType != VideoTrackType) {
+        m_trackType = model.trackList().size() > 0 ? model.trackList().at(trackIndex).type : VideoTrackType;
+    }
     if (m_trackType == AudioTrackType)
         setText(QObject::tr("Insert audio track"));
     else if (m_trackType == VideoTrackType)
@@ -1138,22 +1312,38 @@ DetachAudioCommand::DetachAudioCommand(MultitrackModel& model, int trackIndex, i
 void DetachAudioCommand::redo()
 {
     LOG_DEBUG() << "trackIndex" << m_trackIndex << "clipIndex" << m_clipIndex << "position" << m_position;
-    Mlt::Producer clip(MLT.profile(), "xml-string", m_xml.toUtf8().constData());
-    if (clip.is_valid()) {
-        // Save the original clip's audio_index.
-        m_audioIndex = QString::fromLatin1(clip.get("audio_index"));
+    Mlt::Producer audioClip(MLT.profile(), "xml-string", m_xml.toUtf8().constData());
+    Mlt::Producer videoClip(MLT.profile(), "xml-string", m_xml.toUtf8().constData());
+    if (audioClip.is_valid() && videoClip.is_valid()) {
 
-        // Disable audio on the original clip.
-        int i = m_model.trackList().at(m_trackIndex).mlt_index;
-        Mlt::Producer track(m_model.tractor()->track(i));
-        if (track.is_valid()) {
-            Mlt::Playlist playlist(track);
-            Mlt::Producer originalClip(playlist.get_clip(m_clipIndex));
-            if (originalClip.is_valid()) {
-                originalClip.parent().set("audio_index", -1);
-                QModelIndex modelIndex = m_model.makeIndex(m_trackIndex, m_clipIndex);
-                emit m_model.dataChanged(modelIndex, modelIndex, QVector<int>() << MultitrackModel::AudioIndexRole);
+        // Disable audio on the video clip.
+        videoClip.set("audio_index", -1);
+        // Remove audio filters from the video clip.
+        for (int i = 0; i < videoClip.filter_count(); i++) {
+            Mlt::Filter* filter = videoClip.filter(i);
+            if (filter && filter->is_valid() && !filter->get_int("_loader")) {
+                QmlMetadata* newMeta = MAIN.filterController()->metadataForService(filter);
+                if (newMeta && newMeta->isAudio()) {
+                    videoClip.detach(*filter);
+                    i--;
+                }
             }
+            delete filter;
+        }
+
+        // Disable video on the audio clip.
+        audioClip.set("video_index", -1);
+        // Remove video filters from the audio clip.
+        for (int i = 0; i < audioClip.filter_count(); i++) {
+            Mlt::Filter* filter = audioClip.filter(i);
+            if (filter && filter->is_valid() && !filter->get_int("_loader")) {
+                QmlMetadata* newMeta = MAIN.filterController()->metadataForService(filter);
+                if (newMeta && !newMeta->isAudio()) {
+                    audioClip.detach(*filter);
+                    i--;
+                }
+            }
+            delete filter;
         }
 
         // Add an audio track if needed.
@@ -1162,11 +1352,9 @@ void DetachAudioCommand::redo()
             Mlt::Producer track(m_model.tractor()->track(m_model.trackList()[i].mlt_index));
             if (!track.is_valid())
                 continue;
-            int hide = track.get_int("hide");
-            // hide: 1 = audio only track, 3 = muted audio-only track
-            if (track.get(kAudioTrackProperty) || hide == 1 || hide == 3) {
+            if (track.get(kAudioTrackProperty)) {
                 Mlt::Playlist playlist(track);
-                int out = clip.get_playtime() - 1;
+                int out = videoClip.get_playtime() - 1;
                 // If the audio track is blank in the target region.
                 if (playlist.is_blank_at(m_position) && playlist.is_blank_at(m_position + out)
                         && playlist.get_clip_index_at(m_position) == playlist.get_clip_index_at(m_position + out)) {
@@ -1185,8 +1373,11 @@ void DetachAudioCommand::redo()
         if (m_targetTrackIndex > -1) {
             // Add the clip to the new audio track.
             m_undoHelper.recordBeforeState();
-            m_model.overwrite(m_targetTrackIndex, clip, m_position, false);
+            m_model.overwrite(m_targetTrackIndex, audioClip, m_position, false);
+            m_model.overwrite(m_trackIndex, videoClip, m_position, false);
             m_undoHelper.recordAfterState();
+            QModelIndex modelIndex = m_model.makeIndex(m_trackIndex, m_clipIndex);
+            emit m_model.dataChanged(modelIndex, modelIndex, QVector<int>() << MultitrackModel::AudioIndexRole);
         }
     }
 }
@@ -1194,27 +1385,16 @@ void DetachAudioCommand::redo()
 void DetachAudioCommand::undo()
 {
     LOG_DEBUG() << "trackIndex" << m_trackIndex << "clipIndex" << m_clipIndex << "position" << m_position;
+    m_undoHelper.undoChanges();
     if (m_trackAdded) {
         // Remove the new audio track.
         m_model.removeTrack(m_targetTrackIndex);
         m_targetTrackIndex = -1;
-    } else {
-        // Remove the clip from the audio track.
-        m_undoHelper.undoChanges();
-    }
-
-    // Restore the audio stream index on the original clip.
-    int i = m_model.trackList().at(m_trackIndex).mlt_index;
-    Mlt::Producer track(m_model.tractor()->track(i));
-    if (track.is_valid()) {
-        Mlt::Playlist playlist(track);
-        Mlt::Producer clip(playlist.get_clip(m_clipIndex));
-        if (clip.is_valid()) {
-            clip.parent().set("audio_index", m_audioIndex.toLatin1().constData());
-            QModelIndex modelIndex = m_model.makeIndex(m_trackIndex, m_clipIndex);
-            emit m_model.dataChanged(modelIndex, modelIndex, QVector<int>() << MultitrackModel::AudioIndexRole << MultitrackModel::AudioLevelsRole);
-        }
     }
+    Mlt::Producer originalClip(MLT.profile(), "xml-string", m_xml.toUtf8().constData());
+    m_model.overwrite(m_trackIndex, originalClip, m_position, true);
+    QModelIndex modelIndex = m_model.makeIndex(m_trackIndex, m_clipIndex);
+    emit m_model.dataChanged(modelIndex, modelIndex, QVector<int>() << MultitrackModel::AudioIndexRole << MultitrackModel::AudioLevelsRole);
 }
 
 ReplaceCommand::ReplaceCommand(MultitrackModel& model, int trackIndex, int clipIndex,
