@@ -146,7 +146,9 @@ void UndoHelper::undoChanges()
     debugPrintState();
 #endif
     if (m_hints & RestoreTracks) {
-        return restoreAffectedTracks();
+        restoreAffectedTracks();
+        emit m_model.modified();
+        return;
     }
     int indexAdjustment = 0;
 
@@ -202,7 +204,7 @@ void UndoHelper::undoChanges()
                 Q_ASSERT(!(m_hints & SkipXML) && "Cannot restore clip without stored XML");
                 Q_ASSERT(!info.xml.isEmpty());
                 Mlt::Producer restoredClip(MLT.profile(), "xml-string", info.xml.toUtf8().constData());
-                if (restoredClip.type() == tractor_type) { // transition
+                if (restoredClip.type() == mlt_service_tractor_type) { // transition
                     restoredClip.set("mlt_type", "mlt_producer");
                 } else {
                     fixTransitions(playlist, currentIndex, restoredClip);
@@ -234,7 +236,7 @@ void UndoHelper::undoChanges()
                 if (clip->get_data("mix_out"))
                     clip->set("mix_out", nullptr, 0);
                 playlist.resize_clip(currentIndex, info.frame_in, info.frame_out);
-                m_model.adjustClipFilters(clip->parent(), filterIn, filterOut, info.in_delta, info.out_delta);
+                MLT.adjustClipFilters(clip->parent(), filterIn, filterOut, info.in_delta, info.out_delta);
             }
 
             QModelIndex modelIndex = m_model.createIndex(currentIndex, 0, info.oldTrackIndex);
@@ -341,7 +343,7 @@ void UndoHelper::restoreAffectedTracks()
                 Q_ASSERT(!(m_hints & SkipXML) && "Cannot restore clip without stored XML");
                 Q_ASSERT(!info.xml.isEmpty());
                 Mlt::Producer restoredClip(MLT.profile(), "xml-string", info.xml.toUtf8().constData());
-                if (restoredClip.type() == tractor_type) { // transition
+                if (restoredClip.type() == mlt_service_tractor_type) { // transition
                     restoredClip.set("mlt_type", "mlt_producer");
                 }
                 playlist.append(restoredClip, info.frame_in, info.frame_out);
