@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2020 Meltytech, LLC
+ * Copyright (c) 2014-2021 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,11 +28,13 @@
 #include "qmltypes/qmlutilities.h"
 #include "qmltypes/qmlfilter.h"
 
+#include <MltLink.h>
+
 FilterController::FilterController(QObject* parent) : QObject(parent),
- m_mltFilter(0),
+ m_mltService(0),
  m_metadataModel(this),
  m_attachedModel(this),
- m_currentFilterIndex(-1)
+ m_currentFilterIndex(QmlFilter::NoCurrentFilter)
 {
     startTimer(0);
     connect(&m_attachedModel, SIGNAL(changed()), this, SLOT(handleAttachedModelChange()));
@@ -44,6 +46,7 @@ FilterController::FilterController(QObject* parent) : QObject(parent),
 
 void FilterController::loadFilterMetadata() {
     QScopedPointer<Mlt::Properties> mltFilters(MLT.repository()->filters());
+    QScopedPointer<Mlt::Properties> mltLinks(MLT.repository()->links());
     QDir dir = QmlUtilities::qmlDir();
     dir.cd("filters");
     foreach (QString dirName, dir.entryList(QDir::AllDirs | QDir::NoDotAndDotDot | QDir::Executable)) {
@@ -56,7 +59,7 @@ void FilterController::loadFilterMetadata() {
             QQmlComponent component(QmlUtilities::sharedEngine(), subdir.absoluteFilePath(fileName));
             QmlMetadata *meta = qobject_cast<QmlMetadata*>(component.create());
             if (meta) {
-                QScopedPointer<Mlt::Properties> mltMetadata(MLT.repository()->metadata(filter_type, meta->mlt_service().toLatin1().constData()));
+                QScopedPointer<Mlt::Properties> mltMetadata(MLT.repository()->metadata(mlt_service_filter_type, meta->mlt_service().toLatin1().constData()));
                 QString version;
                 if (mltMetadata && mltMetadata->is_valid() && mltMetadata->get("version")) {
                     version = QString::fromLatin1(mltMetadata->get("version"));
@@ -78,10 +81,17 @@ void FilterController::loadFilterMetadata() {
                         meta->setProperty("version", version);
                         meta->keyframes()->checkVersion(version);
                     }
-
-                    if (meta->isDeprecated())
-                        meta->setName(meta->name() + " " + tr("(DEPRECATED)"));
                 }
+                else if (meta->type() == QmlMetadata::Link && mltLinks->get_data(meta->mlt_service().toLatin1().constData())) {
+                    LOG_DEBUG() << "added link" << meta->name();
+                    meta->loadSettings();
+                    meta->setPath(subdir);
+                    meta->setParent(0);
+                    addMetadata(meta);
+                }
+
+                if (meta->isDeprecated())
+                    meta->setName(meta->name() + " " + tr("(DEPRECATED)"));
             } else if (!meta) {
                 LOG_WARNING() << component.errorString();
             }
@@ -132,8 +142,9 @@ void FilterController::setProducer(Mlt::Producer *producer)
     m_attachedModel.setProducer(producer);
     if (producer && producer->is_valid()) {
         mlt_service_type service_type = producer->type();
-        m_metadataModel.setIsClipProducer(service_type != playlist_type &&
-            (service_type != tractor_type || !producer->get_int(kShotcutXmlProperty)));
+        m_metadataModel.setIsClipProducer(service_type != mlt_service_playlist_type &&
+            (service_type != mlt_service_tractor_type || !producer->get_int(kShotcutXmlProperty)));
+        m_metadataModel.setIsChainProducer(service_type == mlt_service_chain_type);
     }
 }
 
@@ -146,9 +157,9 @@ void FilterController::setCurrentFilter(int attachedIndex, bool isNew)
 
     // VUIs may instruct MLT filters to not render if they are doing the rendering
     // theirself, for example, Text: Rich. Component.onDestruction is not firing.
-    if (m_mltFilter) {
-        if (m_mltFilter->get_int("_hide")) {
-            m_mltFilter->clear("_hide");
+    if (m_mltService) {
+        if (m_mltService->get_int("_hide")) {
+            m_mltService->clear("_hide");
             MLT.refreshConsumer();
         }
     }
@@ -156,9 +167,10 @@ void FilterController::setCurrentFilter(int attachedIndex, bool isNew)
     QmlMetadata* meta = m_attachedModel.getMetadata(m_currentFilterIndex);
     QmlFilter* filter = 0;
     if (meta) {
-        emit currentFilterChanged(nullptr, nullptr, -1);
-        m_mltFilter = m_attachedModel.getFilter(m_currentFilterIndex);
-        filter = new QmlFilter(*m_mltFilter, meta);
+        emit currentFilterChanged(nullptr, nullptr, QmlFilter::NoCurrentFilter);
+        m_mltService = m_attachedModel.getService(m_currentFilterIndex);
+        if (!m_mltService) return;
+        filter = new QmlFilter(*m_mltService, meta);
         filter->setIsNew(isNew);
         connect(filter, SIGNAL(changed()), SLOT(onQmlFilterChanged()));
         connect(filter, SIGNAL(changed(QString)), SLOT(onQmlFilterChanged(const QString&)));
@@ -184,16 +196,16 @@ void FilterController::onFadeOutChanged()
     }
 }
 
-void FilterController::onFilterInChanged(int delta, Mlt::Filter* filter)
+void FilterController::onServiceInChanged(int delta, Mlt::Service* service)
 {
-    if (delta && m_currentFilter && (!filter || m_currentFilter->filter().get_filter() == filter->get_filter())) {
+    if (delta && m_currentFilter && (!service || m_currentFilter->service().get_service() == service->get_service())) {
         emit m_currentFilter->inChanged(delta);
     }
 }
 
-void FilterController::onFilterOutChanged(int delta, Mlt::Filter* filter)
+void FilterController::onServiceOutChanged(int delta, Mlt::Service* service)
 {
-    if (delta && m_currentFilter && (!filter || m_currentFilter->filter().get_filter() == filter->get_filter())) {
+    if (delta && m_currentFilter && (!service || m_currentFilter->service().get_service() == service->get_service())) {
         emit m_currentFilter->outChanged(delta);
     }
 }
@@ -207,18 +219,18 @@ void FilterController::handleAttachedModelChange()
 
 void FilterController::handleAttachedModelAboutToReset()
 {
-    setCurrentFilter(-1);
+    setCurrentFilter(QmlFilter::NoCurrentFilter);
 }
 
 void FilterController::handleAttachedRowsRemoved(const QModelIndex&, int first, int)
 {
-    m_currentFilterIndex = -2; // Force update
+    m_currentFilterIndex = QmlFilter::DeselectCurrentFilter; // Force update
     setCurrentFilter(qBound(0, first, m_attachedModel.rowCount() - 1));
 }
 
 void FilterController::handleAttachedRowsInserted(const QModelIndex&, int first, int)
 {
-    m_currentFilterIndex = -2; // Force update
+    m_currentFilterIndex = QmlFilter::DeselectCurrentFilter; // Force update
     setCurrentFilter(qBound(0, first, m_attachedModel.rowCount() - 1), true);
 }
 
@@ -231,7 +243,7 @@ void FilterController::handleAttachDuplicateFailed(int index)
 
 void FilterController::onQmlFilterChanged()
 {
-    emit filterChanged(m_mltFilter);
+    emit filterChanged(m_mltService);
 }
 
 void FilterController::onQmlFilterChanged(const QString &name)
@@ -244,7 +256,7 @@ void FilterController::onQmlFilterChanged(const QString &name)
 
 void FilterController::removeCurrent()
 {
-    if (m_currentFilterIndex > -1)
+    if (m_currentFilterIndex > QmlFilter::NoCurrentFilter)
         m_attachedModel.remove(m_currentFilterIndex);
 }
 
