@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2021 Meltytech, LLC
+ * Copyright (c) 2012-2022 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 #include "encodedock.h"
 #include "ui_encodedock.h"
 #include "dialogs/addencodepresetdialog.h"
+#include "dialogs/multifileexportdialog.h"
 #include "jobqueue.h"
 #include "mltcontroller.h"
 #include "mainwindow.h"
@@ -28,6 +29,7 @@
 #include "util.h"
 #include "dialogs/listselectiondialog.h"
 #include "qmltypes/qmlfilter.h"
+#include "models/markersmodel.h"
 
 #include <Logger.h>
 #include <QtWidgets>
@@ -153,9 +155,16 @@ void EncodeDock::loadPresetFromProperties(Mlt::Properties& preset)
 
     // Default the HEVC crf to 28 per libx265 default.
     QString vcodec = QString::fromLatin1(preset.get("vcodec"));
-    if (vcodec == "libx265" || vcodec.contains("hevc"))
+    if (vcodec == "libx265" || vcodec.contains("hevc")) {
         ui->videoQualitySpinner->setValue(45);
 
+        // Apple needs codec ID hvc1 even though technically that is not what we are writing.
+        QString f = preset.get("f");
+        if (!preset.property_exists("vtag") && (f == "mp4" || f == "mov")) {
+            other.append("vtag=hvc1");
+        }
+    }
+
     for (int i = 0; i < preset.count(); i++) {
         QString name(preset.get_name(i));
 
@@ -343,6 +352,7 @@ void EncodeDock::loadPresetFromProperties(Mlt::Properties& preset)
                 ui->interpolationCombo->setCurrentIndex(3);
         }
         else if (name != "an" && name != "vn" && name != "threads"
+                 && !(name == "frame_rate_den" && preset.property_exists("frame_rate_num"))
                  && !name.startsWith('_') && !name.startsWith("meta.preset.")) {
             other.append(QString("%1=%2").arg(name).arg(preset.get(i)));
         }
@@ -365,12 +375,17 @@ void EncodeDock::loadPresetFromProperties(Mlt::Properties& preset)
     if (ui->videoRateControlCombo->currentIndex() == RateControlQuality && videoQuality > -1) {
         const QString& vcodec = ui->videoCodecCombo->currentText();
         //val = min + (max - min) * paramval;
-        if (vcodec == "libx264" || vcodec == "libx265" || vcodec.contains("nvenc") || vcodec.endsWith("_amf")
-                || vcodec.endsWith("_qsv") || vcodec.endsWith("_videotoolbox") || vcodec.endsWith("_vaapi"))
+        if (vcodec.startsWith("libx264") || vcodec == "libx265" || vcodec.contains("nvenc") || vcodec.endsWith("_amf")
+                || vcodec.endsWith("_videotoolbox") || vcodec.endsWith("_vaapi"))
             // 0 (best, 100%) - 51 (worst)
             ui->videoQualitySpinner->setValue(TO_RELATIVE(51, 0, videoQuality));
+        else if (vcodec.endsWith("_qsv"))
+            // 1 (best, 100%) - 51 (worst)
+            ui->videoQualitySpinner->setValue(TO_RELATIVE(51, 1, videoQuality));
         else if (vcodec.startsWith("libvpx") || vcodec.startsWith("libaom-")) // 0 (best, 100%) - 63 (worst)
             ui->videoQualitySpinner->setValue(TO_RELATIVE(63, 0, videoQuality));
+        else if (vcodec.startsWith("libwebp")) // 100 (best) - 0 (worst)
+            ui->videoQualitySpinner->setValue(TO_RELATIVE(0, 100, videoQuality));
         else // 1 (best, NOT 100%) - 31 (worst)
             ui->videoQualitySpinner->setValue(TO_RELATIVE(31, 1, videoQuality));
     }
@@ -391,12 +406,11 @@ void EncodeDock::onProducerOpened()
     ui->fromCombo->clear();
     if (MAIN.isMultitrackValid())
         ui->fromCombo->addItem(tr("Timeline"), "timeline");
-    if (MAIN.playlist() && MAIN.playlist()->count() > 0)
+    if (MAIN.playlist() && MAIN.playlist()->count() > 0) {
         ui->fromCombo->addItem(tr("Playlist"), "playlist");
-    if (MAIN.playlist() && MAIN.playlist()->count() > 1)
         ui->fromCombo->addItem(tr("Each Playlist Item"), "batch");
-    if (MLT.isClip() && MLT.producer() && MLT.producer()->is_valid()
-            && qstrcmp("_hide", MLT.producer()->get("resource"))) {
+    }
+    if (MLT.isClip() && qstrcmp("_hide", MLT.producer()->get("resource"))) {
         ui->fromCombo->addItem(tr("Source"), "clip");
         if (MLT.producer()->get_int(kBackgroundCaptureProperty) || MLT.producer()->get_int(kExportFromProperty))
             index = ui->fromCombo->count() - 1;
@@ -404,6 +418,22 @@ void EncodeDock::onProducerOpened()
                && qstrcmp("_hide", MLT.savedProducer()->get("resource"))) {
         ui->fromCombo->addItem(tr("Source"), "clip");
     }
+    if (MAIN.isMultitrackValid()) {
+        MarkersModel markersModel;
+        markersModel.load(MAIN.multitrack());
+        const auto markerStr = tr("Marker");
+        auto ranges = markersModel.ranges();
+        for (auto i = ranges.constBegin(); i != ranges.constEnd(); ++i) {
+            QString comboText;
+            if (i.value().startsWith(markerStr)) {
+                comboText = i.value();
+            } else {
+                comboText = QString("%1: %2").arg(tr("Marker"), i.value());
+            }
+            comboText= ui->fromCombo->fontMetrics().elidedText(comboText, Qt::ElideRight, 400);
+            ui->fromCombo->addItem(comboText, QString("marker:%1").arg(i.key()));
+        }
+    }
     ui->fromCombo->blockSignals(false);
     if (!m_immediateJob) {
         ui->fromCombo->setCurrentIndex(index);
@@ -520,7 +550,7 @@ static void setIfNotSet(Mlt::Properties* properties, const char* name, T value)
          properties->set(name, value);
 }
 
-Mlt::Properties* EncodeDock::collectProperties(int realtime)
+Mlt::Properties* EncodeDock::collectProperties(int realtime, bool includeProfile)
 {
     Mlt::Properties* p = new Mlt::Properties;
     if (p && p->is_valid()) {
@@ -728,7 +758,7 @@ Mlt::Properties* EncodeDock::collectProperties(int realtime)
                     break;
                     }
                 case RateControlQuality: {
-                    if (vcodec == "libx264") {
+                    if (vcodec.startsWith("libx264")) {
                         setIfNotSet(p, "crf", TO_ABSOLUTE(51, 0, vq));
                     } else if (vcodec.startsWith("libvpx") || vcodec.startsWith("libaom-")) {
                         setIfNotSet(p, "crf", TO_ABSOLUTE(63, 0, vq));
@@ -738,13 +768,15 @@ Mlt::Properties* EncodeDock::collectProperties(int realtime)
                         setIfNotSet(p, "vq", TO_ABSOLUTE(51, 0, vq));
                     } else if (vcodec.endsWith("_qsv")) {
                         setIfNotSet(p, "qscale", TO_ABSOLUTE(51, 1, vq));
+                    } else if (vcodec.startsWith("libwebp")) {
+                        setIfNotSet(p, "qscale", TO_ABSOLUTE(0, 100, vq));
                     } else {
                         setIfNotSet(p, "qscale", TO_ABSOLUTE(31, 1, vq));
                     }
                     break;
                     }
                 case RateControlConstrained: {
-                    if (vcodec == "libx264") {
+                    if (vcodec.startsWith("libx264")) {
                         setIfNotSet(p, "crf", TO_ABSOLUTE(51, 0, vq));
                     } else if (vcodec.startsWith("libvpx") || vcodec.startsWith("libaom-")) {
                         setIfNotSet(p, "crf", TO_ABSOLUTE(63, 0, vq));
@@ -754,6 +786,8 @@ Mlt::Properties* EncodeDock::collectProperties(int realtime)
                         setIfNotSet(p, "vb", vbitrate.toLatin1().constData());
                         setIfNotSet(p, "vglobal_quality", TO_ABSOLUTE(51, 0, vq));
                         setIfNotSet(p, "vq", TO_ABSOLUTE(51, 0, vq));
+                    } else if (vcodec.startsWith("libwebp")) {
+                        setIfNotSet(p, "qscale", TO_ABSOLUTE(0, 100, vq));
                     } else {
                         setIfNotSet(p, "qscale", TO_ABSOLUTE(31, 1, vq));
                     }
@@ -774,23 +808,23 @@ Mlt::Properties* EncodeDock::collectProperties(int realtime)
                     setIfNotSet(p, "pix_fmt", "nv12");
                 } else if (vcodec.endsWith("_vaapi")) {
                     setIfNotSet(p, "vprofile", "main");
-                    setIfNotSet(p, "connection_type", "x11");
                 }
             }
-            if (ui->widthSpinner->value() != MLT.profile().width()) {
+            if (includeProfile || ui->widthSpinner->value() != MLT.profile().width()) {
                 setIfNotSet(p, "width", ui->widthSpinner->value());
             }
-            if (ui->heightSpinner->value() != MLT.profile().height()) {
+            if (includeProfile || ui->heightSpinner->value() != MLT.profile().height()) {
                 setIfNotSet(p, "height", ui->heightSpinner->value());
             }
             if (ui->previewScaleCheckBox->isChecked() && !p->get("scale") && Settings.playerPreviewScale() > 0) {
                 p->set("scale", double(Settings.playerPreviewScale()) / MLT.profile().height());
             }
-            if (ui->aspectNumSpinner->value() != MLT.profile().display_aspect_num() ||
+            if (includeProfile ||
+                ui->aspectNumSpinner->value() != MLT.profile().display_aspect_num() ||
                 ui->aspectDenSpinner->value() != MLT.profile().display_aspect_den()) {
                 setIfNotSet(p, "aspect", double(ui->aspectNumSpinner->value()) / double(ui->aspectDenSpinner->value()));
             }
-            if (ui->scanModeCombo->currentIndex() != MLT.profile().progressive()) {
+            if (includeProfile || ui->scanModeCombo->currentIndex() != MLT.profile().progressive()) {
                 setIfNotSet(p, "progressive", ui->scanModeCombo->currentIndex());
             }
             setIfNotSet(p, "top_field_first", ui->fieldOrderCombo->currentIndex());
@@ -825,31 +859,17 @@ Mlt::Properties* EncodeDock::collectProperties(int realtime)
             // If the frame rate is not specified in Other.
             if (!p->get("r") && !(p->get("frame_rate_num") && p->get("frame_rate_den"))
                 // Only if the frame rate spinner does not match the profile.
-                && qRound(ui->fpsSpinner->value() * 1000000.0) != qRound(MLT.profile().fps() * 1000000.0))
+                && (includeProfile || qRound(ui->fpsSpinner->value() * 1000000.0) != qRound(MLT.profile().fps() * 1000000.0)))
             {
-                // Convert some common non-integer frame rates to fractions.
-                if (qRound(ui->fpsSpinner->value() * 1000000.0) == 23976024) {
-                    p->set("frame_rate_num", 24000);
-                    p->set("frame_rate_den", 1001);
-                } else if (qRound(ui->fpsSpinner->value() * 100000.0) == 2997003) {
-                    p->set("frame_rate_num", 30000);
-                    p->set("frame_rate_den", 1001);
-                } else if (qRound(ui->fpsSpinner->value() * 1000000.0) == 47952048) {
-                    p->set("frame_rate_num", 48000);
-                    p->set("frame_rate_den", 1001);
-                } else if (qRound(ui->fpsSpinner->value() * 100000.0) == 5994006) {
-                    p->set("frame_rate_num", 60000);
-                    p->set("frame_rate_den", 1001);
-                } else {
-                    // Workaround storing QDoubleSpinBox::value() loses precision.
-                    p->set("frame_rate_num", qRound(ui->fpsSpinner->value() * 1000000.0));
-                    p->set("frame_rate_den", 1000000);
-                }
+                int numerator, denominator;
+                Util::normalizeFrameRate(ui->fpsSpinner->value(), numerator, denominator);
+                p->set("frame_rate_num", numerator);
+                p->set("frame_rate_den", denominator);
             }
             if (ui->formatCombo->currentText() == "image2")
                 setIfNotSet(p, "threads", 1);
             else if (ui->videoCodecThreadsSpinner->value() == 0
-                     && ui->videoCodecCombo->currentText() != "libx264"
+                     && !ui->videoCodecCombo->currentText().startsWith("libx264")
                      && ui->videoCodecCombo->currentText() != "libx265")
                 setIfNotSet(p, "threads", ui->videoCodecThreadsSpinner->maximum() - 1);
             else
@@ -859,7 +879,7 @@ Mlt::Properties* EncodeDock::collectProperties(int realtime)
                     && !vcodec.endsWith("_videotoolbox") && !vcodec.endsWith("_vaapi") &&
                 ui->dualPassCheckbox->isEnabled() && ui->dualPassCheckbox->isChecked())
                 setIfNotSet(p, "pass", 1);
-            if (ui->scanModeCombo->currentIndex() == 0 && ui->fieldOrderCombo->currentIndex() == 0 && vcodec == "libx264") {
+            if (ui->scanModeCombo->currentIndex() == 0 && ui->fieldOrderCombo->currentIndex() == 0 && vcodec.startsWith("libx264")) {
                 QString x264params = QString::fromUtf8(p->get("x264-params"));
                 if (!x264params.contains("bff=") && !x264params.contains("tff=")) {
                     x264params.prepend("bff=1:");
@@ -892,7 +912,7 @@ MeltJob* EncodeDock::createMeltJob(Mlt::Producer* service, const QString& target
     QString mytarget = target;
     if (!ui->disableVideoCheckbox->isChecked()) {
         const QString& codec = ui->videoCodecCombo->currentText();
-        if (codec == "bmp" || codec == "dpx" || codec == "png" || codec == "ppm" ||
+        if (codec == "bmp" || codec == "dpx" || codec == "png" || codec == "ppm" || (codec == "libwebp" && ui->formatCombo->currentText() == "image2") ||
                 codec == "targa" || codec == "tiff" || (codec == "mjpeg" && ui->formatCombo->currentText() == "image2")) {
             QFileInfo fi(mytarget);
             mytarget = QString("%1/%2-%05d.%3").arg(fi.path()).arg(fi.baseName()).arg(fi.completeSuffix());
@@ -901,7 +921,7 @@ MeltJob* EncodeDock::createMeltJob(Mlt::Producer* service, const QString& target
 
     // Fix in/out points of filters on clip-only project.
     QScopedPointer<Mlt::Producer> tempProducer;
-    if (MLT.isSeekable(service))
+    if (MLT.isSeekable(service) && service->type() != mlt_service_chain_type)
     if (ui->fromCombo->currentData().toString() == "clip" || ui->fromCombo->currentData().toString() == "batch") {
         QString xml = MLT.XML(service);
         tempProducer.reset(new Mlt::Producer(MLT.profile(), "xml-string", xml.toUtf8().constData()));
@@ -990,6 +1010,20 @@ MeltJob* EncodeDock::createMeltJob(Mlt::Producer* service, const QString& target
             ui->heightSpinner->value() != MLT.profile().height() ||
             double(ui->aspectNumSpinner->value()) / double(ui->aspectDenSpinner->value()) != MLT.profile().dar() ||
             (ui->fromCombo->currentData().toString() != "clip" && qFloor(ui->fpsSpinner->value() * 10000.0) != qFloor(MLT.profile().fps() * 10000.0)));
+
+    const auto& from = ui->fromCombo->currentData().toString();
+    if (MAIN.isMultitrackValid() && from.startsWith("marker:")) {
+        bool ok = false;
+        int index = from.midRef(7).toInt(&ok);
+        if (ok) {
+            MarkersModel markersModel;
+            markersModel.load(MAIN.multitrack());
+            auto marker = markersModel.getMarker(index);
+            if (marker.end > marker.start) {
+                job->setInAndOut(marker.start, marker.end);
+            }
+        }
+    }
     return job;
 }
 
@@ -1073,6 +1107,10 @@ public:
     int on_end_filter(Mlt::Filter*) { return 0; }
     int on_start_transition(Mlt::Transition*) { return 0; }
     int on_end_transition(Mlt::Transition*) { return 0; }
+    int on_start_chain(Mlt::Chain*) { return 0; }
+    int on_end_chain(Mlt::Chain*) { return 0; }
+    int on_start_link(Mlt::Link*) { return 0; }
+    int on_end_link(Mlt::Link*) { return 0; }
 };
 
 void EncodeDock::enqueueAnalysis()
@@ -1106,7 +1144,7 @@ void EncodeDock::enqueueAnalysis()
     }
 }
 
-void EncodeDock::enqueueMelt(const QString& target, int realtime)
+void EncodeDock::enqueueMelt(const QStringList& targets, int realtime)
 {
     Mlt::Producer* service = fromProducer();
     int pass = (ui->videoRateControlCombo->currentIndex() != RateControlQuality
@@ -1118,10 +1156,8 @@ void EncodeDock::enqueueMelt(const QString& target, int realtime)
             &&  ui->dualPassCheckbox->isEnabled() && ui->dualPassCheckbox->isChecked())? 1 : 0;
     if (!service) {
         // For each playlist item.
-        if (MAIN.playlist() && MAIN.playlist()->count() > 1) {
-            QFileInfo fi(target);
+        if (MAIN.playlist() && MAIN.playlist()->count() > 0) {
             int n = MAIN.playlist()->count();
-            int digits = QString::number(n).size();
             for (int i = 0; i < n; i++) {
                 QScopedPointer<Mlt::ClipInfo> info(MAIN.playlist()->clip_info(i));
                 if (!info) continue;
@@ -1129,13 +1165,11 @@ void EncodeDock::enqueueMelt(const QString& target, int realtime)
                 QScopedPointer<Mlt::Producer> producer(
                     new Mlt::Producer(MLT.profile(), "xml-string", xml.toUtf8().constData()));
                 producer->set_in_and_out(info->frame_in, info->frame_out);
-                QString filename = QString("%1/%2-%3.%4").arg(fi.path()).arg(fi.baseName())
-                                                         .arg(i + 1, digits, 10, QChar('0')).arg(fi.completeSuffix());
-                MeltJob* job = createMeltJob(producer.data(), filename, realtime, pass);
+                MeltJob* job = createMeltJob(producer.data(), targets[i], realtime, pass);
                 if (job) {
                     JOBS.add(job);
                     if (pass) {
-                        job = createMeltJob(producer.data(), filename, realtime, 2);
+                        job = createMeltJob(producer.data(), targets[i], realtime, 2);
                         if (job)
                             JOBS.add(job);
                     }
@@ -1143,11 +1177,11 @@ void EncodeDock::enqueueMelt(const QString& target, int realtime)
             }
         }
     } else {
-        MeltJob* job = createMeltJob(service, target, realtime, pass);
+        MeltJob* job = createMeltJob(service, targets[0], realtime, pass);
         if (job) {
             JOBS.add(job);
             if (pass) {
-                job = createMeltJob(service, target, realtime, 2);
+                job = createMeltJob(service, targets[0], realtime, 2);
                 if (job)
                     JOBS.add(job);
             }
@@ -1229,7 +1263,7 @@ Mlt::Producer *EncodeDock::fromProducer() const
         return MLT.isClip()? MLT.producer() : MLT.savedProducer();
     else if (from == "playlist")
         return MAIN.playlist();
-    else if (from == "timeline")
+    else if (from == "timeline" || from.startsWith("marker:"))
         return MAIN.multitrack();
     else
         return 0;
@@ -1405,93 +1439,108 @@ void EncodeDock::on_encodeButton_clicked()
         }
     }
 
-    QString caption = seekable? tr("Export File") : tr("Capture File");
-    QString nameFilter;
-    if (!m_extension.isEmpty())
-        nameFilter = tr("%1 (*.%2);;All Files (*)").arg(ui->formatCombo->currentText()).arg(m_extension);
-    else
-        tr("Determined by Export (*)");
-    m_outputFilename = QFileDialog::getSaveFileName(this, caption, directory, nameFilter,
-        nullptr, Util::getFileDialogOptions());
-    if (!m_outputFilename.isEmpty()) {
-        QFileInfo fi(m_outputFilename);
-        MLT.pause();
-        Settings.setEncodePath(fi.path());
-        if (!m_extension.isEmpty()) {
+    QString caption = seekable ? tr("Export File") : tr("Capture File");
+    if (ui->fromCombo->currentData().toString() == "batch") {
+        caption = tr("Export Files");
+        MultiFileExportDialog dialog(tr("Export Each Playlist Item"), MAIN.playlist(),
+            QDir(directory).absolutePath(), QFileInfo(MAIN.fileName()).completeBaseName(), m_extension, this);
+        if (dialog.exec() != QDialog::Accepted) {
+            return;
+        }
+        m_outputFilenames = dialog.getExportFiles();
+    } else {
+        QString nameFilter;
+        if (!m_extension.isEmpty())
+            nameFilter = tr("%1 (*.%2);;All Files (*)").arg(ui->formatCombo->currentText()).arg(m_extension);
+        else
+            nameFilter = tr("Determined by Export (*)");
+        QString newName = QFileDialog::getSaveFileName(this, caption, directory, nameFilter,
+            nullptr, Util::getFileDialogOptions());
+        if (!newName.isEmpty() && !m_extension.isEmpty()) {
+            QFileInfo fi(newName);
             if (fi.suffix().isEmpty()) {
-                m_outputFilename += '.';
-                m_outputFilename += m_extension;
+                newName += '.';
+                newName += m_extension;
             }
         }
+        m_outputFilenames = QStringList(newName);
+    }
 
-        // Check if the drive this file will be on is getting low on space.
-        if (Settings.encodeFreeSpaceCheck()) {
-            QStorageInfo si(fi.path());
-            LOG_DEBUG() << si.bytesAvailable() << "bytes available on" << si.displayName();
-            if (si.isValid() && si.bytesAvailable() < kFreeSpaceThesholdGB) {
-                QMessageBox dialog(QMessageBox::Question, caption,
-                   tr("The drive you chose only has %1 MiB of free space.\n"
-                      "Do you still want to continue?")
-                   .arg(si.bytesAvailable() / 1024 / 1024),
-                   QMessageBox::No | QMessageBox::Yes, this);
-                dialog.setWindowModality(QmlApplication::dialogModality());
-                dialog.setDefaultButton(QMessageBox::Yes);
-                dialog.setEscapeButton(QMessageBox::No);
-                dialog.setCheckBox(new QCheckBox(tr("Do not show this anymore.", "Export free disk space warning dialog")));
-                int result = dialog.exec();
-                if (dialog.checkBox()->isChecked())
-                    Settings.setEncodeFreeSpaceCheck(false);
-                if (result == QMessageBox::No) {
-                    MAIN.showStatusMessage(tr("Export canceled."));
-                    return;
-                }
+    if (m_outputFilenames.isEmpty() || m_outputFilenames[0].isEmpty()) {
+        return;
+    }
+
+    QFileInfo fi(m_outputFilenames[0]);
+    MLT.pause();
+    Settings.setEncodePath(fi.path());
+
+    // Check if the drive this file will be on is getting low on space.
+    if (Settings.encodeFreeSpaceCheck()) {
+        QStorageInfo si(fi.path());
+        LOG_DEBUG() << si.bytesAvailable() << "bytes available on" << si.displayName();
+        if (si.isValid() && si.bytesAvailable() < kFreeSpaceThesholdGB) {
+            QMessageBox dialog(QMessageBox::Question, caption,
+               tr("The drive you chose only has %1 MiB of free space.\n"
+                  "Do you still want to continue?")
+               .arg(si.bytesAvailable() / 1024 / 1024),
+               QMessageBox::No | QMessageBox::Yes, this);
+            dialog.setWindowModality(QmlApplication::dialogModality());
+            dialog.setDefaultButton(QMessageBox::Yes);
+            dialog.setEscapeButton(QMessageBox::No);
+            dialog.setCheckBox(new QCheckBox(tr("Do not show this anymore.", "Export free disk space warning dialog")));
+            int result = dialog.exec();
+            if (dialog.checkBox()->isChecked())
+                Settings.setEncodeFreeSpaceCheck(false);
+            if (result == QMessageBox::No) {
+                MAIN.showStatusMessage(tr("Export canceled."));
+                return;
             }
         }
+    }
 
-        if (seekable) {
-            MLT.purgeMemoryPool();
-            // Batch encode
-            int threadCount = QThread::idealThreadCount();
-            if (threadCount > 2 && ui->parallelCheckbox->isChecked())
-                threadCount = qMin(threadCount - 1, 4);
-            else
-                threadCount = 1;
-            enqueueAnalysis();
-            enqueueMelt(m_outputFilename, Settings.playerGPU()? -1 : -threadCount);
-        }
-        else if (MLT.producer()->get_int(kBackgroundCaptureProperty)) {
-            // Capture in background
-            ui->dualPassCheckbox->setChecked(false);
-            m_immediateJob.reset(createMeltJob(fromProducer(), m_outputFilename, -1));
-            if (m_immediateJob) {
-                // Close the player's producer to prevent resource contention.
-                MAIN.hideProducer();
+    if (seekable) {
+        MLT.purgeMemoryPool();
+        // Batch encode
+        int threadCount = QThread::idealThreadCount();
+        if (threadCount > 2 && ui->parallelCheckbox->isChecked())
+            threadCount = qMin(threadCount - 1, 4);
+        else
+            threadCount = 1;
+        enqueueAnalysis();
+        enqueueMelt(m_outputFilenames, Settings.playerGPU()? -1 : -threadCount);
+    }
+    else if (MLT.producer()->get_int(kBackgroundCaptureProperty)) {
+        // Capture in background
+        ui->dualPassCheckbox->setChecked(false);
+        m_immediateJob.reset(createMeltJob(fromProducer(), m_outputFilenames[0], -1));
+        if (m_immediateJob) {
+            // Close the player's producer to prevent resource contention.
+            MAIN.hideProducer();
 
-                m_immediateJob->setIsStreaming(true);
-                connect(m_immediateJob.data(), SIGNAL(finished(AbstractJob*,bool,QString)), this, SLOT(onFinished(AbstractJob*,bool)));
+            m_immediateJob->setIsStreaming(true);
+            connect(m_immediateJob.data(), SIGNAL(finished(AbstractJob*,bool,QString)), this, SLOT(onFinished(AbstractJob*,bool)));
 
-                if (MLT.resource().startsWith("gdigrab:") || MLT.resource().startsWith("x11grab:")) {
-                    ui->stopCaptureButton->show();
-                } else {
-                    ui->encodeButton->setText(tr("Stop Capture"));
-                    ui->fromCombo->setDisabled(true);
-                }
-                if (MLT.resource().startsWith("gdigrab:"))
-                    MAIN.showMinimized();
-
-                int msec = MLT.producer()->get_int(kBackgroundCaptureProperty) * 1000;
-                QTimer::singleShot(msec, m_immediateJob.data(), SLOT(start()));
+            if (MLT.resource().startsWith("gdigrab:") || MLT.resource().startsWith("x11grab:")) {
+                ui->stopCaptureButton->show();
+            } else {
+                ui->encodeButton->setText(tr("Stop Capture"));
+                ui->fromCombo->setDisabled(true);
             }
-        }
-        else {
-            // Capture to file
-            // use multi consumer to encode and preview simultaneously
-            ui->dualPassCheckbox->setChecked(false);
-            ui->encodeButton->setText(tr("Stop Capture"));
-            encode(m_outputFilename);
-            emit captureStateChanged(true);
-            ui->streamButton->setDisabled(true);
-        }
+            if (MLT.resource().startsWith("gdigrab:"))
+                MAIN.showMinimized();
+
+            int msec = MLT.producer()->get_int(kBackgroundCaptureProperty) * 1000;
+            QTimer::singleShot(msec, m_immediateJob.data(), SLOT(start()));
+       }
+    }
+    else {
+        // Capture to file
+        // use multi consumer to encode and preview simultaneously
+        ui->dualPassCheckbox->setChecked(false);
+        ui->encodeButton->setText(tr("Stop Capture"));
+        encode(m_outputFilenames[0]);
+        emit captureStateChanged(true);
+        ui->streamButton->setDisabled(true);
     }
 }
 
@@ -1582,13 +1631,13 @@ void EncodeDock::on_streamButton_clicked()
             emit captureStateChanged(true);
             emit ui->encodeButton->setDisabled(true);
         }
-        m_outputFilename.clear();
+        m_outputFilenames.clear();
     }
 }
 
 void EncodeDock::on_addPresetButton_clicked()
 {
-    QScopedPointer<Mlt::Properties> data(collectProperties(0));
+    QScopedPointer<Mlt::Properties> data(collectProperties(0, true));
     AddEncodePresetDialog dialog(this);
     QStringList ls;
 
@@ -1621,9 +1670,11 @@ void EncodeDock::on_addPresetButton_clicked()
                 if (dir.mkdir(subdir))
                     dir.cd(subdir);
             }
-            QFile f(dir.filePath(preset));
-            if (f.open(QIODevice::WriteOnly | QIODevice::Text))
-                f.write(dialog.properties().toUtf8());
+            {
+                QFile f(dir.filePath(preset));
+                if (f.open(QIODevice::WriteOnly | QIODevice::Text))
+                    f.write(dialog.properties().toUtf8());
+            }
 
             // add the preset and select it
             loadPresets();
@@ -1679,7 +1730,7 @@ void EncodeDock::on_stopCaptureButton_clicked()
     ui->stopCaptureButton->hide();
     if (m_immediateJob)
         m_immediateJob->stop();
-    if (!m_outputFilename.isEmpty())
+    if (!m_outputFilenames.isEmpty())
         QTimer::singleShot(kOpenCaptureFileDelayMs, this, SLOT(openCaptureFile()));
 }
 
@@ -1803,11 +1854,12 @@ void EncodeDock::on_resetButton_clicked()
     m_isDefaultSettings = true;
     resetOptions();
     onProfileChanged();
+    ui->presetsTree->setCurrentIndex(QModelIndex());
 }
 
 void EncodeDock::openCaptureFile()
 {
-    MAIN.open(m_outputFilename);
+    MAIN.open(m_outputFilenames[0]);
 }
 
 void EncodeDock::on_formatCombo_currentIndexChanged(int index)
@@ -1965,7 +2017,7 @@ void EncodeDock::on_videoQualitySpinner_valueChanged(int vq)
 {
     const QString& vcodec = ui->videoCodecCombo->currentText();
     QString s;
-    if (vcodec == "libx264" || vcodec == "libx265") {
+    if (vcodec.startsWith("libx264") || vcodec == "libx265") {
         s = QString("crf=%1").arg(TO_ABSOLUTE(51, 0, vq));
     } else if (vcodec.startsWith("libvpx") || vcodec.startsWith("libaom-")) {
         s = QString("crf=%1").arg(TO_ABSOLUTE(63, 0, vq));
@@ -1980,6 +2032,8 @@ void EncodeDock::on_videoQualitySpinner_valueChanged(int vq)
         s = QString("vglobal_quality=%1").arg(TO_ABSOLUTE(51, 0, vq));
     } else if (vcodec.endsWith("_qsv")) {
         s = QString("qscale=%1").arg(TO_ABSOLUTE(51, 1, vq));
+    } else if (vcodec.startsWith("libwebp")) {
+        s = QString("qscale=%1").arg(TO_ABSOLUTE(0, 100, vq));
     } else {
         s = QString("qscale=%1").arg(TO_ABSOLUTE(31, 1, vq));
     }
@@ -2015,7 +2069,7 @@ bool EncodeDock::detectHardwareEncoders()
         QStringList args;
         args << "-hide_banner" << "-f" << "lavfi" << "-i" << "color=s=640x360" << "-frames" << "1" << "-an";
         if (codec.endsWith("_vaapi"))
-            args << "-init_hw_device" << "vaapi=vaapi0:,connection_type=x11" << "-filter_hw_device" << "vaapi0" << "-vf" << "format=nv12,hwupload";
+            args << "-init_hw_device" << "vaapi=vaapi0:" << "-filter_hw_device" << "vaapi0" << "-vf" << "format=nv12,hwupload";
         else if (codec == "hevc_qsv")
             args << "-load_plugin" << "hevc_hw";
         args << "-c:v" << codec << "-f" << "rawvideo" << "pipe:";
@@ -2071,7 +2125,7 @@ bool EncodeDock::checkForMissingFiles()
     MLT.saveXML(fileName, service, false /* without relative paths */, tmp.get(), isProxy);
     tmp->close();
     MltXmlChecker checker;
-    if (!checker.check(fileName)) {
+    if (checker.check(fileName) != QXmlStreamReader::NoError) {
         LOG_ERROR() << "Encode: Unable to check XML - skipping check";
     } else if (checker.unlinkedFilesModel().rowCount() > 0) {
         QMessageBox dialog(QMessageBox::Critical,
@@ -2079,12 +2133,23 @@ bool EncodeDock::checkForMissingFiles()
            tr("Your project is missing some files.\n\n"
               "Save your project, close it, and reopen it.\n"
               "Shotcut will attempt to repair your project."),
-           QMessageBox::Ok,
+           QMessageBox::Ok | QMessageBox::Ignore,
            this);
         dialog.setWindowModality(QmlApplication::dialogModality());
         dialog.setDefaultButton(QMessageBox::Ok);
-        dialog.exec();
+        if (QMessageBox::Ignore == dialog.exec()) {
+            return false;
+        }
         return true;
     }
     return false;
 }
+
+void EncodeDock::on_resolutionComboBox_activated(const QString &arg1)
+{
+    if (arg1.isEmpty()) return;
+    auto parts = arg1.splitRef(' ');
+    ui->widthSpinner->setValue(parts[0].toInt());
+    ui->heightSpinner->setValue(parts[2].toInt());
+}
+
