@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2020 Meltytech, LLC
+ * Copyright (c) 2012-2021 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -125,6 +125,8 @@ private slots:
 
     void on_parallelCheckbox_clicked(bool checked);
 
+    void on_resolutionComboBox_activated(const QString &arg1);
+
 private:
     enum {
         RateControlAverage = 0,
@@ -143,17 +145,17 @@ private:
     QString m_extension;
     Mlt::Properties *m_profiles;
     PresetsProxyModel m_presetsModel;
-    QString m_outputFilename;
+    QStringList m_outputFilenames;
     bool m_isDefaultSettings;
     double m_fps;
 
     void loadPresets();
-    Mlt::Properties* collectProperties(int realtime);
+    Mlt::Properties* collectProperties(int realtime, bool includeProfile = false);
     void collectProperties(QDomElement& node, int realtime);
     MeltJob* createMeltJob(Mlt::Producer* service, const QString& target, int realtime, int pass = 0);
     void runMelt(const QString& target, int realtime = -1);
     void enqueueAnalysis();
-    void enqueueMelt(const QString& target, int realtime);
+    void enqueueMelt(const QStringList& targets, int realtime);
     void encode(const QString& target);
     void resetOptions();
     Mlt::Producer* fromProducer() const;
