@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2020 Meltytech, LLC
+ * Copyright (c) 2013-2021 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ FiltersDock::FiltersDock(MetadataModel* metadataModel, AttachedFiltersModel* att
     connect(&m_producer, SIGNAL(seeked(int)), SIGNAL(seeked(int)));
     connect(this, SIGNAL(producerInChanged(int)), &m_producer, SIGNAL(inChanged(int)));
     connect(this, SIGNAL(producerOutChanged(int)), &m_producer, SIGNAL(outChanged(int)));
-    setCurrentFilter(0, 0, -1);
+    setCurrentFilter(0, 0, QmlFilter::NoCurrentFilter);
     connect(m_qview.quickWindow(), SIGNAL(sceneGraphInitialized()), SLOT(resetQview()));
 
     LOG_DEBUG() << "end";
@@ -111,7 +111,7 @@ void FiltersDock::onSeeked(int position)
             // Make the position relative to the clip's in point.
             position -= m_producer.in();
         }
-        m_producer.seek(qBound(0, position, m_producer.duration()));
+        m_producer.seek(position);
     }
 }
 
@@ -119,15 +119,7 @@ void FiltersDock::onShowFrame(const SharedFrame& frame)
 {
     if (m_producer.producer().is_valid()) {
         int position = frame.get_position();
-        if (MLT.isMultitrack()) {
-            // Make the position relative to clip's position on a timeline track.
-            position -= m_producer.producer().get_int(kPlaylistStartProperty);
-        } else {
-            // Make the position relative to the clip's in point.
-            position -= m_producer.in();
-        }
-        if (position >= 0 && position <= m_producer.duration())
-            m_producer.seek(position);
+        onSeeked(position);
     }
 }
 
@@ -163,5 +155,5 @@ void FiltersDock::resetQview()
 
     QObject::connect(m_qview.rootObject(), SIGNAL(currentFilterRequested(int)),
         SIGNAL(currentFilterRequested(int)));
-    emit currentFilterRequested(-1);
+    emit currentFilterRequested(QmlFilter::NoCurrentFilter);
 }
