@@ -40,6 +40,8 @@
 #include <QHeaderView>
 #include <QKeyEvent>
 #include <QDir>
+#include <QGuiApplication>
+#include <QClipboard>
 
 static const int kInOutChangedTimeoutMs = 100;
 
@@ -148,6 +150,8 @@ PlaylistDock::PlaylistDock(QWidget *parent) :
     toggleViewAction()->setIcon(windowIcon());
     ui->actionPlayAfterOpen->setChecked(Settings.playlistAutoplay());
 
+    ui->stackedWidget->setCurrentIndex(0);
+
     m_iconsView = new PlaylistIconView(this);
     ui->listView->parentWidget()->layout()->addWidget(m_iconsView);
     m_iconsView->setSelectionMode(QAbstractItemView::SingleSelection);
@@ -376,24 +380,24 @@ void PlaylistDock::on_actionInsertCut_triggered()
 
 void PlaylistDock::on_actionAppendCut_triggered()
 {
-    if (MLT.producer() && MLT.producer()->is_valid() && !MAIN.isSourceClipMyProject()) {
+    Mlt::Producer producer(MLT.isClip()? MLT.producer() : MLT.savedProducer());
+    if (producer.is_valid() && !MAIN.isSourceClipMyProject()) {
         if (MLT.isSeekableClip()
             || (MLT.savedProducer() && MLT.isSeekable(MLT.savedProducer()))) {
-            Mlt::Producer producer(MLT.isClip()? MLT.producer() : MLT.savedProducer());
             ProxyManager::generateIfNotExists(producer);
             MAIN.undoStack()->push(
                 new Playlist::AppendCommand(m_model, MLT.XML(&producer)));
-            setPlaylistIndex(MLT.producer(), m_model.playlist()->count() - 1);
+            setPlaylistIndex(&producer, m_model.playlist()->count() - 1);
             setUpdateButtonEnabled(true);
         } else {
             DurationDialog dialog(this);
             dialog.setDuration(MLT.profile().fps() * 5);
             if (dialog.exec() == QDialog::Accepted) {
-                MLT.producer()->set_in_and_out(0, dialog.duration() - 1);
-                if (MLT.producer()->get("mlt_service") && !strcmp(MLT.producer()->get("mlt_service"), "avformat"))
-                    MLT.producer()->set("mlt_service", "avformat-novalidate");
+                producer.set_in_and_out(0, dialog.duration() - 1);
+                if (producer.get("mlt_service") && !strcmp(producer.get("mlt_service"), "avformat"))
+                    producer.set("mlt_service", "avformat-novalidate");
                 MAIN.undoStack()->push(new Playlist::AppendCommand(m_model, MLT.XML()));
-                setPlaylistIndex(MLT.producer(), m_model.playlist()->count() - 1);
+                setPlaylistIndex(&producer, m_model.playlist()->count() - 1);
                 setUpdateButtonEnabled(true);
             }
         }
@@ -424,14 +428,16 @@ void PlaylistDock::on_actionAppendBlank_triggered()
 void PlaylistDock::on_actionUpdate_triggered()
 {
     QModelIndex index = m_view->currentIndex();
-    if (!index.isValid() || !m_model.playlist()) return;
-    Mlt::ClipInfo* info = m_model.playlist()->clip_info(index.row());
-    if (!info || MAIN.isSourceClipMyProject()) return;
-    if (MLT.producer()->type() != playlist_type) {
-        if (MLT.isSeekable()) {
-            ProxyManager::generateIfNotExists(*MLT.producer());
-            MAIN.undoStack()->push(new Playlist::UpdateCommand(m_model, MLT.XML(), index.row()));
-            setPlaylistIndex(MLT.producer(), index.row());
+    if (!index.isValid() || !m_model.playlist() || MAIN.isSourceClipMyProject()) return;
+    QScopedPointer<Mlt::ClipInfo> info(m_model.playlist()->clip_info(index.row()));
+    Mlt::Producer producer(MLT.isClip()? MLT.producer() : MLT.savedProducer());
+    if (!info || !producer.is_valid()) return;
+    if (producer.type() != mlt_service_playlist_type) {
+        if (MLT.isSeekableClip()
+                || (MLT.savedProducer() && MLT.isSeekable(MLT.savedProducer()))) {
+            ProxyManager::generateIfNotExists(producer);
+            MAIN.undoStack()->push(new Playlist::UpdateCommand(m_model, MLT.XML(&producer), index.row()));
+            setPlaylistIndex(&producer, index.row());
             setUpdateButtonEnabled(true);
         }
         else {
@@ -439,11 +445,11 @@ void PlaylistDock::on_actionUpdate_triggered()
             DurationDialog dialog(this);
             dialog.setDuration(info->frame_count);
             if (dialog.exec() == QDialog::Accepted) {
-                MLT.producer()->set_in_and_out(0, dialog.duration() - 1);
-                if (MLT.producer()->get("mlt_service") && !strcmp(MLT.producer()->get("mlt_service"), "avformat"))
-                    MLT.producer()->set("mlt_service", "avformat-novalidate");
+                producer.set_in_and_out(0, dialog.duration() - 1);
+                if (producer.get("mlt_service") && !strcmp(producer.get("mlt_service"), "avformat"))
+                    producer.set("mlt_service", "avformat-novalidate");
                 MAIN.undoStack()->push(new Playlist::UpdateCommand(m_model, MLT.XML(), index.row()));
-                setPlaylistIndex(MLT.producer(), index.row());
+                setPlaylistIndex(&producer, index.row());
                 setUpdateButtonEnabled(true);
             }
         }
@@ -452,7 +458,6 @@ void PlaylistDock::on_actionUpdate_triggered()
         emit showStatusMessage(tr("You cannot insert a playlist into a playlist!"));
         setUpdateButtonEnabled(false);
     }
-    delete info;
 }
 
 void PlaylistDock::on_removeButton_clicked()
@@ -673,14 +678,19 @@ void PlaylistDock::onDropped(const QMimeData *data, int row)
         for (const auto& path : fileNames) {
             if (MAIN.isSourceClipMyProject(path)) continue;
             longTask.reportProgress(Util::baseName(path), i++, count);
-            Mlt::Producer p(MLT.profile(), path.toUtf8().constData());
-            if (p.is_valid()) {
-                // Convert MLT XML to a virtual clip.
-                if (!qstrcmp(p.get("mlt_service"), "xml")) {
+            Mlt::Producer p;
+            if (path.endsWith(".mlt") || path.endsWith(".xml")) {
+                p = Mlt::Producer(MLT.profile(), "xml", path.toUtf8().constData());
+                if (p.is_valid()) {
+                    // Convert MLT XML to a virtual clip.
                     p.set(kShotcutVirtualClip, 1);
                     p.set("resource", path.toUtf8().constData());
                     first = false;
                 }
+            } else {
+                p = Mlt::Producer(MLT.profile(), path.toUtf8().constData());
+            }
+            if (p.is_valid()) {
                 Mlt::Producer* producer = &p;
                 if (first) {
                     first = false;
@@ -692,15 +702,8 @@ void PlaylistDock::onDropped(const QMimeData *data, int row)
                         }
                     }
                 }
-                // Convert avformat to avformat-novalidate so that XML loads faster.
-                if (!qstrcmp(producer->get("mlt_service"), "avformat")) {
-                    producer->set("mlt_service", "avformat-novalidate");
-                    producer->set("mute_on_pause", 0);
-                }
-                MLT.setImageDurationFromDefault(producer);
-                MLT.lockCreationTime(producer);
-                producer->get_length_time(mlt_time_clock);
-                if (MLT.isSeekable(producer)) {
+                producer = MLT.setupNewProducer(producer);
+                if (MLT.isSeekable(producer) || producer->get_int(kShotcutVirtualClip)) {
                     ProxyManager::generateIfNotExists(*producer);
                     if (row == -1)
                         MAIN.undoStack()->push(new Playlist::AppendCommand(m_model, MLT.XML(producer)));
@@ -722,12 +725,13 @@ void PlaylistDock::onDropped(const QMimeData *data, int row)
                     setIndex(0);
                     resetIndex = false;
                 }
+                delete producer;
             }
         }
     }
     else if (data && data->hasFormat(Mlt::XmlMimeType)) {
         if (MLT.producer() && MLT.producer()->is_valid()) {
-            if (MLT.producer()->type() == playlist_type) {
+            if (MLT.producer()->type() == mlt_service_playlist_type) {
                 emit showStatusMessage(tr("You cannot insert a playlist into a playlist!"));
             } else if (MAIN.isSourceClipMyProject()) {
                 return;
@@ -893,6 +897,8 @@ void PlaylistDock::on_updateButton_clicked()
 
 void PlaylistDock::onProducerChanged(Mlt::Producer* producer)
 {
+    if (!producer || !producer->is_valid())
+        return;
     int index = producer->get_int(kPlaylistIndexProperty) - 1;
     if (index < 0 || !m_model.playlist() || !m_model.playlist()->is_valid() || index >= m_model.playlist()->count())
         return;
@@ -1078,6 +1084,7 @@ void PlaylistDock::on_actionCopy_triggered()
         QString xml = MLT.XML(i->producer);
         Mlt::Producer* p = new Mlt::Producer(MLT.profile(), "xml-string", xml.toUtf8().constData());
         p->set_in_and_out(i->frame_in, i->frame_out);
+        QGuiApplication::clipboard()->setText(MLT.XML(p));
         emit clipOpened(p);
         delete i;
         m_iconsView->resetMultiSelect();
@@ -1119,3 +1126,26 @@ void PlaylistDock::onProducerModified()
     // The clip name may have changed.
     emitDataChanged(QVector<int>() << PlaylistModel::FIELD_RESOURCE);
 }
+
+void PlaylistDock::on_addFilesButton_clicked()
+{
+    QMimeData mimeData;
+    QList<QUrl> urls;
+
+    QString path = Settings.openPath();
+#ifdef Q_OS_MAC
+    path.append("/*");
+#endif
+    LOG_DEBUG() << Util::getFileDialogOptions();
+    QStringList filenames = QFileDialog::getOpenFileNames(this, tr("Open File"), path,
+                            tr("All Files (*);;MLT XML (*.mlt)"), nullptr, Util::getFileDialogOptions());
+
+    if (filenames.length() > 0) {
+        Settings.setOpenPath(QFileInfo(filenames.first()).path());
+        foreach(const QString& s, filenames) {
+            urls << s;
+        }
+        mimeData.setUrls(urls);
+        onDropped(&mimeData, m_view->currentIndex().row() + 1);
+    }
+}
