@@ -22,17 +22,21 @@
 #include "qmltypes/thumbnailprovider.h"
 #include "mainwindow.h"
 #include "commands/timelinecommands.h"
+#include "qmltypes/qmlapplication.h"
 #include "qmltypes/qmlutilities.h"
 #include "qmltypes/qmlview.h"
 #include "shotcut_mlt_properties.h"
 #include "settings.h"
 #include "util.h"
 #include "proxymanager.h"
+#include "dialogs/editmarkerdialog.h"
 #include "dialogs/longuitask.h"
 
 #include <QAction>
 #include <QtQml>
 #include <QtQuick>
+#include <QGuiApplication>
+#include <QClipboard>
 #include <Logger.h>
 
 static const char* kFileUrlProtocol = "file://";
@@ -56,6 +60,7 @@ TimelineDock::TimelineDock(QWidget *parent) :
     toggleViewAction()->setIcon(windowIcon());
 
     qmlRegisterType<MultitrackModel>("Shotcut.Models", 1, 0, "MultitrackModel");
+    qmlRegisterType<MarkersModel>("Shotcut.Models", 1, 0, "MarkersModel");
 
     QDir importPath = QmlUtilities::qmlDir();
     importPath.cd("modules");
@@ -65,6 +70,7 @@ TimelineDock::TimelineDock(QWidget *parent) :
     m_quickView.rootContext()->setContextProperty("view", new QmlView(&m_quickView));
     m_quickView.rootContext()->setContextProperty("timeline", this);
     m_quickView.rootContext()->setContextProperty("multitrack", &m_model);
+    m_quickView.rootContext()->setContextProperty("markers", &m_markersModel);
     m_quickView.setResizeMode(QQuickWidget::SizeRootObjectToView);
     m_quickView.setClearColor(palette().window().color());
     m_quickView.quickWindow()->setPersistentSceneGraph(false);
@@ -73,10 +79,15 @@ TimelineDock::TimelineDock(QWidget *parent) :
 #endif
 
     connect(&m_model, SIGNAL(modified()), this, SLOT(clearSelectionIfInvalid()));
-    connect(&m_model, &MultitrackModel::inserted, this, &TimelineDock::onInserted, Qt::QueuedConnection);
-    connect(&m_model, &MultitrackModel::overWritten, this, &TimelineDock::onOverWritten, Qt::QueuedConnection);
+    connect(&m_model, &MultitrackModel::appended, this, &TimelineDock::selectClip, Qt::QueuedConnection);
+    connect(&m_model, &MultitrackModel::inserted, this, &TimelineDock::selectClip, Qt::QueuedConnection);
+    connect(&m_model, &MultitrackModel::overWritten, this, &TimelineDock::selectClip, Qt::QueuedConnection);
     connect(&m_model, SIGNAL(rowsInserted(QModelIndex,int,int)), SLOT(onRowsInserted(QModelIndex,int,int)));
     connect(&m_model, SIGNAL(rowsRemoved(QModelIndex,int,int)), SLOT(onRowsRemoved(QModelIndex,int,int)));
+    connect(&m_model, SIGNAL(closed()), SLOT(onMultitrackClosed()));
+    connect(&m_model, SIGNAL(created()), SLOT(reloadTimelineMarkers()));
+    connect(&m_model, SIGNAL(loaded()), SLOT(reloadTimelineMarkers()));
+    connect(&m_model, SIGNAL(closed()), SLOT(reloadTimelineMarkers()));
 
     setWidget(&m_quickView);
 
@@ -85,6 +96,7 @@ TimelineDock::TimelineDock(QWidget *parent) :
     connect(MLT.videoWidget(), SIGNAL(frameDisplayed(const SharedFrame&)), this, SLOT(onShowFrame(const SharedFrame&)));
     connect(this, SIGNAL(visibilityChanged(bool)), this, SLOT(load()));
     connect(this, SIGNAL(topLevelChanged(bool)), this, SLOT(onTopLevelChanged(bool)));
+    connect(&m_markersModel, SIGNAL(rangesChanged()), this, SIGNAL(markerRangesChanged()));
     LOG_DEBUG() << "end";
 }
 
@@ -171,6 +183,22 @@ void TimelineDock::emitNonSeekableWarning()
     emit showStatusMessage(tr("You cannot add a non-seekable source."));
 }
 
+void TimelineDock::addTrackIfNeeded(int trackIndex, Mlt::Producer* srcTrack)
+{
+    const auto n = m_model.trackList().size();
+    if (trackIndex >= n) {
+        if (m_selection.selectedTrack != -1)
+            setSelection();
+        if (srcTrack->get_int(kAudioTrackProperty) || (n > 0 && m_model.trackList()[n - 1].type == AudioTrackType)) {
+            MAIN.undoStack()->push(
+                new Timeline::InsertTrackCommand(m_model, trackIndex, AudioTrackType));
+        } else {
+            MAIN.undoStack()->push(
+                new Timeline::InsertTrackCommand(m_model, trackIndex, VideoTrackType));
+        }
+    }
+}
+
 void TimelineDock::chooseClipAtPosition(int position, int& trackIndex, int& clipIndex)
 {
     QScopedPointer<Mlt::Producer> clip;
@@ -392,7 +420,7 @@ bool TimelineDock::isRipple() const
 void TimelineDock::copyToSource()
 {
     if (model()->tractor() && model()->tractor()->is_valid()) {
-        if (MAIN.on_actionSave_triggered()) {
+        if (!MAIN.isWindowModified() || MAIN.on_actionSave_triggered()) {
             if (!MLT.openXML(MAIN.fileName())) {
                 MLT.producer()->set(kExportFromProperty, 1);
                 MAIN.open(MLT.producer());
@@ -439,6 +467,22 @@ void TimelineDock::insertTrack()
                 new Timeline::InsertTrackCommand(m_model, currentTrack()));
 }
 
+void TimelineDock::insertAudioTrack()
+{
+    if (m_selection.selectedTrack != -1)
+        setSelection();
+    MAIN.undoStack()->push(
+                new Timeline::InsertTrackCommand(m_model, currentTrack(), AudioTrackType));
+}
+
+void TimelineDock::insertVideoTrack()
+{
+    if (m_selection.selectedTrack != -1)
+        setSelection();
+    MAIN.undoStack()->push(
+                new Timeline::InsertTrackCommand(m_model, currentTrack(), VideoTrackType));
+}
+
 void TimelineDock::removeTrack()
 {
     if (m_model.trackList().size() > 0) {
@@ -486,7 +530,6 @@ void TimelineDock::onProducerChanged(Mlt::Producer* after)
             int length = qRound(info->length * speedRatio);
             int in = qMin(qRound(info->frame_in * speedRatio), length - 1);
             int out = qMin(qRound(info->frame_out * speedRatio), length - 1);
-            after->set("length", after->frames_to_time(length, mlt_time_clock));
             after->set_in_and_out(in, out);
 
             // Adjust filters.
@@ -529,7 +572,7 @@ void TimelineDock::onShowFrame(const SharedFrame& frame)
     if (m_ignoreNextPositionChange) {
         m_ignoreNextPositionChange = false;
     } else if (MLT.isMultitrack() && m_position != frame.get_position()) {
-        m_position = frame.get_position();
+        m_position = qMin(frame.get_position(), m_model.tractor()->get_length());
         emit positionChanged();
     }
 }
@@ -537,11 +580,17 @@ void TimelineDock::onShowFrame(const SharedFrame& frame)
 void TimelineDock::onSeeked(int position)
 {
     if (MLT.isMultitrack() && m_position != position) {
-        m_position = position;
+        m_position = qMin(position, m_model.tractor()->get_length());
         emit positionChanged();
     }
 }
 
+
+static bool isSystemClipboardValid(const QString& xml)
+{
+    return MLT.isMltXml(xml) && MAIN.isClipboardNewer() && !xml.contains(kShotcutFiltersClipboard);
+}
+
 void TimelineDock::append(int trackIndex)
 {
     if (trackIndex < 0)
@@ -551,12 +600,73 @@ void TimelineDock::append(int trackIndex)
         return;
     }
     if (MAIN.isSourceClipMyProject()) return;
-    if (MLT.isSeekableClip() || MLT.savedProducer()) {
-        Mlt::Producer producer(MLT.isClip()? MLT.producer() : MLT.savedProducer());
-        ProxyManager::generateIfNotExists(producer);
-        MAIN.undoStack()->push(
-            new Timeline::AppendCommand(m_model, trackIndex, MLT.XML(&producer)));
-        selectClipUnderPlayhead();
+
+    // Use MLT XML on the clipboard if it exists and is newer than source clip.
+    QString xmlToUse = QGuiApplication::clipboard()->text();
+    if (isSystemClipboardValid(xmlToUse)) {
+        if (!Settings.proxyEnabled()) {
+            ProxyManager::filterXML(xmlToUse, "");
+        }
+    } else {
+        xmlToUse.clear();
+    }
+
+    if (MLT.isSeekableClip() || MLT.savedProducer() || !xmlToUse.isEmpty()) {
+        if (xmlToUse.isEmpty()) {
+            Mlt::Producer producer(MLT.isClip()? MLT.producer() : MLT.savedProducer());
+            ProxyManager::generateIfNotExists(producer);
+            xmlToUse = MLT.XML(&producer);
+        }
+        if (xmlToUse.isEmpty()) {
+            return;
+        }
+
+        // Insert multiple if the XML is a <tractor> with child <property name="shotcut">1</property>
+        // No need to create a track in an empty timeline.
+        // This can be a macro of QUndoCommands.
+        Mlt::Producer producer(MLT.profile(), "xml-string", xmlToUse.toUtf8().constData());
+        if (producer.is_valid() && producer.type() == mlt_service_tractor_type && producer.get_int(kShotcutXmlProperty)) {
+            Mlt::Tractor tractor(producer);
+            Mlt::ClipInfo info;
+            MAIN.undoStack()->beginMacro(tr("Append multiple to timeline"));
+            Mlt::Controller::RefreshBlocker blocker;
+
+            // Loop over each source track
+            for (int mltTrackIndex = 0; mltTrackIndex < tractor.count(); mltTrackIndex++) {
+                QScopedPointer<Mlt::Producer> srcTrack(tractor.track(mltTrackIndex));
+                if (srcTrack) {
+                    const auto trackIndex = currentTrack() + mltTrackIndex;
+                    addTrackIfNeeded(trackIndex, srcTrack.get());
+
+                    // Insert the clips for this track
+                    Mlt::Playlist playlist(*srcTrack);
+                    for (int mltClipIndex = 0; mltClipIndex < playlist.count(); mltClipIndex++) {
+                        if (!playlist.is_blank(mltClipIndex)) {
+                            playlist.clip_info(mltClipIndex, &info);
+                            Mlt::Producer clip(info.producer);
+                            clip.set_in_and_out(info.frame_in, info.frame_out);
+                            MAIN.undoStack()->push(
+                                new Timeline::AppendCommand(m_model, trackIndex, MLT.XML(&clip)));
+                        }
+                    }
+                }
+            }
+            MAIN.undoStack()->endMacro();
+            MLT.refreshConsumer();
+
+        } else {
+            if (m_model.trackList().size() == 0) {
+                addVideoTrack();
+            }
+
+            MAIN.undoStack()->push(
+                new Timeline::AppendCommand(m_model, trackIndex, xmlToUse));
+        }
+
+        if (m_position < 0) {
+            // This happens when pasting in a new session
+            MAIN.openCut(new Mlt::Producer(m_model.tractor()));
+        }
     } else if (!MLT.isSeekableClip()) {
         emitNonSeekableWarning();
     }
@@ -574,7 +684,7 @@ void TimelineDock::remove(int trackIndex, int clipIndex)
     Mlt::Producer clip = producerForClip(trackIndex, clipIndex);
     if (clip.is_valid()) {
         MAIN.undoStack()->push(
-            new Timeline::RemoveCommand(m_model, trackIndex, clipIndex));
+            new Timeline::RemoveCommand(m_model, m_markersModel, trackIndex, clipIndex));
     }
 }
 
@@ -611,15 +721,21 @@ void TimelineDock::removeSelection(bool withCopy)
     // Cut
     if (withCopy) {
         auto clip = selection().first();
-        copyClip(clip.y(), clip.x());
-        remove(clip.y(), clip.x());
-        return;
+        copy(clip.y(), clip.x());
+        if (selection().size() < 2) {
+            remove(clip.y(), clip.x());
+            return;
+        }
     }
 
     // Ripple delete
     int n = selection().size();
-    if (n > 1)
-        MAIN.undoStack()->beginMacro(tr("Remove %1 from timeline").arg(n));
+    if (n > 1) {
+        if (withCopy)
+            MAIN.undoStack()->beginMacro(tr("Cut %1 from timeline").arg(n));
+        else
+            MAIN.undoStack()->beginMacro(tr("Remove %1 from timeline").arg(n));
+    }
     int trackIndex, clipIndex;
     for (const auto& uuid : selectionUuids()) {
         delete m_model.findClipByUuid(uuid, trackIndex, clipIndex);
@@ -680,22 +796,85 @@ void TimelineDock::selectMultitrack()
     emit selected(m_model.tractor());
 }
 
-void TimelineDock::copyClip(int trackIndex, int clipIndex)
+
+template<typename T>
+static void insertSorted(std::vector<T> & vec, T const& item)
 {
-    if (trackIndex < 0)
-        trackIndex = currentTrack();
-    if (clipIndex < 0)
-        clipIndex = clipIndexAtPlayhead(trackIndex);
-    Q_ASSERT(trackIndex >= 0 && clipIndex >= 0);
-    QScopedPointer<Mlt::ClipInfo> info(getClipInfo(trackIndex, clipIndex));
-    if (info) {
-        QString xml = MLT.XML(info->producer);
-        Mlt::Producer p(MLT.profile(), "xml-string", xml.toUtf8().constData());
-        p.set_speed(0);
-        p.seek(info->frame_in);
-        p.set_in_and_out(info->frame_in, info->frame_out);
-        MLT.setSavedProducer(&p);
-        emit clipCopied();
+    vec.insert(std::upper_bound(vec.begin(), vec.end(), item), item);
+}
+
+void TimelineDock::copy(int trackIndex, int clipIndex)
+{
+    auto selected = selection();
+    if (selected.size() < 2) {
+        if (trackIndex < 0)
+            trackIndex = currentTrack();
+        if (clipIndex < 0)
+            clipIndex = clipIndexAtPlayhead(trackIndex);
+        Q_ASSERT(trackIndex >= 0 && clipIndex >= 0);
+        QScopedPointer<Mlt::ClipInfo> info(getClipInfo(trackIndex, clipIndex));
+        if (info) {
+            QString xml = MLT.XML(info->producer);
+            Mlt::Producer p(MLT.profile(), "xml-string", xml.toUtf8().constData());
+            p.set_speed(0);
+            p.seek(info->frame_in);
+            p.set_in_and_out(info->frame_in, info->frame_out);
+            MLT.setSavedProducer(&p);
+            QGuiApplication::clipboard()->setText(MLT.XML(&p));
+            emit clipCopied();
+        }
+    } else {
+        // Determine the track indices
+        auto minY = std::numeric_limits<int>::max();
+        auto maxY = -1;
+        auto minStart = std::numeric_limits<int>::max();
+        for (auto& a : selected) {
+            minY = std::min(minY, a.y());
+            maxY = std::max(maxY, a.y());
+            auto info = getClipInfo(a.y(), a.x());
+            if (info) minStart = std::min(minStart, info->start);
+            delete info;
+        }
+        // Create the tracks
+        Mlt::Tractor tractor(MLT.profile());
+        tractor.set(kShotcutXmlProperty, 1);
+        for (int trackIndex = minY, i = 0; trackIndex <= maxY; trackIndex++, i++) {
+            Mlt::Playlist playlist(MLT.profile());
+            if (m_model.trackList()[trackIndex].type == AudioTrackType) {
+                playlist.set("hide", 1);
+                playlist.set(kAudioTrackProperty, 1);
+            } else {
+                playlist.set(kVideoTrackProperty, 1);
+            }
+            tractor.set_track(playlist, i);
+
+            // Sort all the clips on this track
+            std::vector<int> clipIndices;
+            for (auto& a : selected) {
+                if (a.y() == trackIndex) {
+                    clipIndices.insert(std::upper_bound(clipIndices.begin(), clipIndices.end(), a.x()), a.x());
+                }
+            }
+
+            // Add the clips to the tracks
+            if (clipIndices.size() > 0) {
+                int prevEnd = minStart;
+                auto mlt_index = m_model.trackList()[trackIndex].mlt_index;
+                QScopedPointer<Mlt::Producer> sourceTrack(m_model.tractor()->track(mlt_index));
+                if (sourceTrack) {
+                    Mlt::Playlist sourcePlaylist(*sourceTrack);
+                    Mlt::ClipInfo info;
+                    for (auto clipIndex : clipIndices) {
+                        sourcePlaylist.clip_info(clipIndex, &info);
+                        playlist.blank(info.start - prevEnd - 1);
+                        playlist.append(*info.producer, info.frame_in, info.frame_out);
+                        prevEnd = info.start + info.frame_count;
+                    }
+                }
+            }
+        }
+        // Put XML in clipboard
+        QGuiApplication::clipboard()->setText(MLT.XML(&tractor));
     }
 }
 
@@ -826,6 +1005,9 @@ void TimelineDock::detachAudio(int trackIndex, int clipIndex)
     QScopedPointer<Mlt::ClipInfo> info(getClipInfo(trackIndex, clipIndex));
     if (info && info->producer && info->producer->is_valid() && !info->producer->is_blank()
              && info->producer->get("audio_index") && info->producer->get_int("audio_index") >= 0) {
+        if (!info->producer->property_exists(kDefaultAudioIndexProperty)) {
+            info->producer->set(kDefaultAudioIndexProperty, info->producer->get_int("audio_index"));
+        }
         Mlt::Producer clip(MLT.profile(), "xml-string", MLT.XML(info->producer).toUtf8().constData());
         clip.set_in_and_out(info->frame_in, info->frame_out);
         MAIN.undoStack()->push(
@@ -874,7 +1056,7 @@ void TimelineDock::replace(int trackIndex, int clipIndex, const QString& xml)
     if (clipIndex < 0)
         clipIndex = clipIndexAtPlayhead(trackIndex);
     Mlt::Producer producer(producerForClip(trackIndex, clipIndex));
-    if (producer.is_valid() && producer.type() == tractor_type) {
+    if (producer.is_valid() && producer.type() == mlt_service_tractor_type) {
         emit showStatusMessage(tr("You cannot replace a transition."));
         return;
     }
@@ -889,6 +1071,78 @@ void TimelineDock::replace(int trackIndex, int clipIndex, const QString& xml)
     }
 }
 
+void TimelineDock::createOrEditMarker()
+{
+    if (!m_model.trackList().count() || MLT.producer()->get_length() <= 1)
+        return;
+    int index = m_markersModel.markerIndexForPosition(m_position);
+    if (index >= 0) {
+        editMarker(index);
+        return;
+    }
+    createMarker();
+}
+
+void TimelineDock::createMarker()
+{
+    if (!m_model.trackList().count() || MLT.producer()->get_length() <= 1)
+        return;
+    int index = m_markersModel.markerIndexForPosition(m_position);
+    if (index >= 0) {
+        return;
+    }
+    Markers::Marker marker;
+    marker.text = QString("Marker %1").arg(m_markersModel.uniqueKey() + 1);
+    marker.color = Settings.markerColor();
+    marker.start = position();
+    marker.end = position();
+    m_markersModel.append(marker);
+    emit showStatusMessage(tr("Added marker: \"%1\". Hold %2 and drag to create a range")
+        .arg(marker.text, QmlApplication::OS() == "OS X"? "⌘" : "Ctrl"));
+}
+
+void TimelineDock::editMarker(int markerIndex)
+{
+    Markers::Marker marker = m_markersModel.getMarker(markerIndex);
+    EditMarkerDialog dialog(this, marker.text, marker.color, marker.start, marker.end, m_model.tractor()->get_length() - 1);
+    dialog.setWindowModality(QmlApplication::dialogModality());
+    if (dialog.exec() == QDialog::Accepted) {
+        marker.text = dialog.getText();
+        marker.color = dialog.getColor();
+        marker.start = dialog.getStart();
+        marker.end = dialog.getEnd();
+        m_markersModel.update(markerIndex, marker);
+    }
+}
+
+void TimelineDock::deleteMarker(int markerIndex)
+{
+    if (markerIndex < 0) {
+        markerIndex = m_markersModel.markerIndexForPosition(m_position);
+    }
+    if (markerIndex >= 0) {
+        m_markersModel.remove(markerIndex);
+    }
+}
+
+void TimelineDock::seekNextMarker()
+{
+    int nextPos = m_markersModel.nextMarkerPosition(m_position);
+    if (nextPos >= 0) {
+        setPosition(nextPos);
+        markerSeeked(m_markersModel.markerIndexForPosition(nextPos));
+    }
+}
+
+void TimelineDock::seekPrevMarker()
+{
+    int prevPos = m_markersModel.prevMarkerPosition(m_position);
+    if (prevPos >= 0) {
+        setPosition(prevPos);
+        markerSeeked(m_markersModel.markerIndexForPosition(prevPos));
+    }
+}
+
 void TimelineDock::setTrackName(int trackIndex, const QString &value)
 {
     MAIN.undoStack()->push(
@@ -981,7 +1235,7 @@ void TimelineDock::onClipMoved(int fromTrack, int toTrack, int clipIndex, int po
                 }
             }
         }
-        auto command = new Timeline::MoveClipCommand(m_model, toTrack - fromTrack, ripple);
+        auto command = new Timeline::MoveClipCommand(m_model, m_markersModel, toTrack - fromTrack, ripple);
 
         // Copy selected
         for (const auto& clip : selection()) {
@@ -1001,7 +1255,7 @@ void TimelineDock::onClipMoved(int fromTrack, int toTrack, int clipIndex, int po
 bool TimelineDock::trimClipIn(int trackIndex, int clipIndex, int oldClipIndex, int delta, bool ripple)
 {
     if (!ripple && m_model.addTransitionByTrimInValid(trackIndex, clipIndex, delta)) {
-        m_model.addTransitionByTrimIn(trackIndex, clipIndex, delta);
+        clipIndex = m_model.addTransitionByTrimIn(trackIndex, clipIndex, delta);
         m_transitionDelta += delta;
         m_trimCommand.reset(new Timeline::AddTransitionByTrimInCommand(m_model, trackIndex, clipIndex - 1, m_transitionDelta, m_trimDelta, false));
         if (m_updateCommand && m_updateCommand->trackIndex() == trackIndex && m_updateCommand->clipIndex() == clipIndex)
@@ -1026,7 +1280,11 @@ bool TimelineDock::trimClipIn(int trackIndex, int clipIndex, int oldClipIndex, i
     else if (m_model.trimClipInValid(trackIndex, clipIndex, delta, ripple)) {
         if (!m_undoHelper) {
             m_undoHelper.reset(new UndoHelper(m_model));
-            if (ripple) m_undoHelper->setHints(UndoHelper::SkipXML);
+            if (ripple) {
+                m_undoHelper->setHints(UndoHelper::SkipXML);
+            } else {
+                m_undoHelper->setHints(UndoHelper::RestoreTracks);
+            }
             m_undoHelper->recordBeforeState();
         }
         clipIndex = m_model.trimClipIn(trackIndex, clipIndex, delta, ripple, Settings.timelineRippleAllTracks());
@@ -1125,20 +1383,14 @@ static QString convertUrlsToXML(const QString& xml)
                 p = Mlt::Producer(MLT.profile(), path.toUtf8().constData());
             }
             if (p.is_valid()) {
-                // Convert avformat to avformat-novalidate so that XML loads faster.
-                if (!qstrcmp(p.get("mlt_service"), "avformat")) {
-                    if (!p.get_int("seekable")) {
-                        MAIN.showStatusMessage(QObject::tr("Not adding non-seekable file: ") + Util::baseName(path));
-                        continue;
-                    }
-                    p.set("mlt_service", "avformat-novalidate");
-                    p.set("mute_on_pause", 0);
+                if (!qstrcmp(p.get("mlt_service"), "avformat") && !p.get_int("seekable")) {
+                    MAIN.showStatusMessage(QObject::tr("Not adding non-seekable file: ") + Util::baseName(path));
+                    continue;
                 }
-                ProxyManager::generateIfNotExists(p);
-                MLT.setImageDurationFromDefault(&p);
-                MLT.lockCreationTime(&p);
-                p.get_length_time(mlt_time_clock);
-                playlist.append(p);
+                Mlt::Producer* producer = MLT.setupNewProducer(&p);
+                ProxyManager::generateIfNotExists(*producer);
+                playlist.append(*producer);
+                delete producer;
             }
         }
         return MLT.XML(&playlist);
@@ -1165,36 +1417,106 @@ void TimelineDock::insert(int trackIndex, int position, const QString &xml, bool
         MAIN.open(Util::removeFileScheme(url), &properties, false /* play */ );
     }
 
-    if (MLT.isSeekableClip() || MLT.savedProducer() || !xml.isEmpty()) {
-        QString xmlToUse;
+    // Use MLT XML on the clipboard if it exists and is newer than source clip.
+    QString xmlToUse = QGuiApplication::clipboard()->text();
+    if (isSystemClipboardValid(xmlToUse)) {
+        if (!Settings.proxyEnabled()) {
+            ProxyManager::filterXML(xmlToUse, "");
+        }
+    } else {
+        xmlToUse.clear();
+    }
+
+    if (MLT.isSeekableClip() || MLT.savedProducer() || !xml.isEmpty() || !xmlToUse.isEmpty()) {
         QScopedPointer<TimelineSelectionBlocker> selectBlocker;
-        if (xml.isEmpty()) {
+        if (xmlToUse.isEmpty() && xml.isEmpty()) {
             Mlt::Producer producer(MLT.isClip()? MLT.producer() : MLT.savedProducer());
             ProxyManager::generateIfNotExists(producer);
             xmlToUse = MLT.XML(&producer);
-        } else {
+        } else if (!xml.isEmpty()) {
+            // Convert a list of file URLs from the xml arg to MLT XML
             xmlToUse = convertUrlsToXML(xml);
             if (xml.startsWith(kFileUrlProtocol) && xml.split(kFilesUrlDelimiter).size() > 1) {
                 selectBlocker.reset(new TimelineSelectionBlocker(*this));
             }
         }
-        if (position < 0)
-            position = m_position;
-        if (m_model.trackList().size() == 0)
-            position = 0;
-        MAIN.undoStack()->push(
-            new Timeline::InsertCommand(m_model, trackIndex, position, xmlToUse, seek));
+        if (xmlToUse.isEmpty()) {
+            return;
+        }
+        if (position < 0) {
+            position = qMax(m_position, 0);
+        }
+
+        // Insert multiple if the XML is a <tractor> with child <property name="shotcut">1</property>
+        // No need to create a track in an empty timeline.
+        // This can be a macro of QUndoCommands.
+        Mlt::Producer producer(MLT.profile(), "xml-string", xmlToUse.toUtf8().constData());
+        if (producer.is_valid() && producer.type() == mlt_service_tractor_type && producer.get_int(kShotcutXmlProperty)) {
+            Mlt::Tractor tractor(producer);
+            Mlt::ClipInfo info;
+            MAIN.undoStack()->beginMacro(tr("Insert multiple into timeline"));
+            Mlt::Controller::RefreshBlocker blocker;
+
+            // Loop over each source track
+            for (int mltTrackIndex = 0; mltTrackIndex < tractor.count(); mltTrackIndex++) {
+                QScopedPointer<Mlt::Producer> srcTrack(tractor.track(mltTrackIndex));
+                if (srcTrack) {
+                    const auto trackIndex = currentTrack() + mltTrackIndex;
+                    addTrackIfNeeded(trackIndex, srcTrack.get());
+
+                    // Insert the clips for this track
+                    Mlt::Playlist playlist(*srcTrack);
+                    for (int mltClipIndex = 0; mltClipIndex < playlist.count(); mltClipIndex++) {
+                        if (!playlist.is_blank(mltClipIndex)) {
+                            playlist.clip_info(mltClipIndex, &info);
+                            Mlt::Producer clip(info.producer);
+                            clip.set_in_and_out(info.frame_in, info.frame_out);
+                            MAIN.undoStack()->push(
+                                new Timeline::InsertCommand(m_model, m_markersModel, trackIndex, position + info.start, MLT.XML(&clip), seek));
+                        }
+                    }
+                }
+            }
+            MAIN.undoStack()->endMacro();
+
+        } else {
+            if (m_model.trackList().size() == 0) {
+                position = 0;
+                addVideoTrack();
+            }
+            MAIN.undoStack()->push(
+                new Timeline::InsertCommand(m_model, m_markersModel, trackIndex, position, xmlToUse, seek));
+        }
+        if (m_position < 0) {
+            // This happens when pasting in a new session
+            MAIN.openCut(new Mlt::Producer(m_model.tractor()));
+        }
     } else if (!MLT.isSeekableClip()) {
         emitNonSeekableWarning();
     }
 }
 
-void TimelineDock::onInserted(int trackIndex, int clipIndex)
+void TimelineDock::selectClip(int trackIndex, int clipIndex)
 {
-    Q_UNUSED(trackIndex)
     setSelection(QList<QPoint>() << QPoint(clipIndex, trackIndex));
 }
 
+void TimelineDock::onMultitrackClosed()
+{
+    m_position = -1;
+    m_ignoreNextPositionChange = false;
+    m_trimDelta = 0;
+    m_transitionDelta = 0;
+    m_blockSetSelection = false;
+    setSelection();
+    emit resetZoom();
+}
+
+void TimelineDock::reloadTimelineMarkers()
+{
+    m_markersModel.load(m_model.tractor());
+}
+
 void TimelineDock::overwrite(int trackIndex, int position, const QString &xml, bool seek)
 {
     // Validations
@@ -1214,36 +1536,82 @@ void TimelineDock::overwrite(int trackIndex, int position, const QString &xml, b
         MAIN.open(Util::removeFileScheme(url), &properties, false /* play */ );
     }
 
-    if (MLT.isSeekableClip() || MLT.savedProducer() || !xml.isEmpty()) {
-        QString xmlToUse;
+    // Use MLT XML on the clipboard if it exists and is newer than source clip.
+    QString xmlToUse = QGuiApplication::clipboard()->text();
+    if (isSystemClipboardValid(xmlToUse)) {
+        if (!Settings.proxyEnabled()) {
+            ProxyManager::filterXML(xmlToUse, "");
+        }
+    } else {
+        xmlToUse.clear();
+    }
+
+    if (MLT.isSeekableClip() || MLT.savedProducer() || !xml.isEmpty() || !xmlToUse.isEmpty()) {
         QScopedPointer<TimelineSelectionBlocker> selectBlocker;
-        if (xml.isEmpty()) {
+        if (xmlToUse.isEmpty() && xml.isEmpty()) {
             Mlt::Producer producer(MLT.isClip()? MLT.producer() : MLT.savedProducer());
             ProxyManager::generateIfNotExists(producer);
             xmlToUse = MLT.XML(&producer);
-        } else {
+        } else if (!xml.isEmpty()) {
             xmlToUse = convertUrlsToXML(xml);
             if (xml.startsWith(kFileUrlProtocol) && xml.split(kFilesUrlDelimiter).size() > 1) {
                 selectBlocker.reset(new TimelineSelectionBlocker(*this));
             }
         }
-        if (position < 0)
-            position = m_position;
-        if (m_model.trackList().size() == 0)
-            position = 0;
-        MAIN.undoStack()->push(
-            new Timeline::OverwriteCommand(m_model, trackIndex, position, xmlToUse, seek));
+        if (position < 0) {
+            position = qMax(m_position, 0);
+        }
+
+        // Overwrite multiple if the XML is a <tractor> with child <property name="shotcut">1</property>
+        // No need to create a track in an empty timeline.
+        // This can be a macro of QUndoCommands.
+        Mlt::Producer producer(MLT.profile(), "xml-string", xmlToUse.toUtf8().constData());
+        if (producer.is_valid() && producer.type() == mlt_service_tractor_type && producer.get_int(kShotcutXmlProperty)) {
+            Mlt::Tractor tractor(producer);
+            Mlt::ClipInfo info;
+            MAIN.undoStack()->beginMacro(tr("Overwrite multiple onto timeline"));
+            Mlt::Controller::RefreshBlocker blocker;
+
+            // Loop over each source track
+            for (int mltTrackIndex = 0; mltTrackIndex < tractor.count(); mltTrackIndex++) {
+                QScopedPointer<Mlt::Producer> srcTrack(tractor.track(mltTrackIndex));
+                if (srcTrack) {
+                    const auto trackIndex = currentTrack() + mltTrackIndex;
+                    addTrackIfNeeded(trackIndex, srcTrack.get());
+
+                    // Insert the clips for this track
+                    Mlt::Playlist playlist(*srcTrack);
+                    for (int mltClipIndex = 0; mltClipIndex < playlist.count(); mltClipIndex++) {
+                        if (!playlist.is_blank(mltClipIndex)) {
+                            playlist.clip_info(mltClipIndex, &info);
+                            Mlt::Producer clip(info.producer);
+                            clip.set_in_and_out(info.frame_in, info.frame_out);
+                            MAIN.undoStack()->push(
+                                new Timeline::OverwriteCommand(m_model, trackIndex, position + info.start, MLT.XML(&clip), seek));
+                        }
+                    }
+                }
+            }
+            MAIN.undoStack()->endMacro();
+
+        } else {
+            if (m_model.trackList().size() == 0) {
+                position = 0;
+                addVideoTrack();
+            }
+
+            MAIN.undoStack()->push(
+                new Timeline::OverwriteCommand(m_model, trackIndex, position, xmlToUse, seek));
+        }
+        if (m_position < 0) {
+            // This happens when pasting in a new session
+            MAIN.openCut(new Mlt::Producer(m_model.tractor()));
+        }
     } else if (!MLT.isSeekableClip()) {
         emitNonSeekableWarning();
     }
 }
 
-void TimelineDock::onOverWritten(int trackIndex, int clipIndex)
-{
-    Q_UNUSED(trackIndex)
-    setSelection(QList<QPoint>() << QPoint(clipIndex, trackIndex));
-}
-
 void TimelineDock::appendFromPlaylist(Mlt::Playlist *playlist, bool skipProxy)
 {
     int trackIndex = currentTrack();
@@ -1259,9 +1627,10 @@ void TimelineDock::appendFromPlaylist(Mlt::Playlist *playlist, bool skipProxy)
         if (clipIndex >= 0)
             m_model.removeClip(trackIndex, clipIndex, Settings.timelineRippleAllTracks());
     }
+    disconnect(&m_model, &MultitrackModel::appended, this, &TimelineDock::selectClip);
     MAIN.undoStack()->push(
         new Timeline::AppendCommand(m_model, trackIndex, MLT.XML(playlist), skipProxy));
-    selectClipUnderPlayhead();
+    connect(&m_model, &MultitrackModel::appended, this, &TimelineDock::selectClip, Qt::QueuedConnection);
 }
 
 void TimelineDock::splitClip(int trackIndex, int clipIndex)
@@ -1279,6 +1648,7 @@ void TimelineDock::splitClip(int trackIndex, int clipIndex)
             if (!m_model.isTransition(playlist, clipIndex)) {
                 QScopedPointer<Mlt::ClipInfo> info(getClipInfo(trackIndex, clipIndex));
                 if (info && m_position > info->start && m_position < info->start + info->frame_count) {
+                    setSelection(); // Avoid filter views becoming out of sync
                     MAIN.undoStack()->push(
                         new Timeline::SplitCommand(m_model, trackIndex, clipIndex, m_position));
                 }
@@ -1492,6 +1862,10 @@ public:
     int on_end_filter(Mlt::Filter*) { return 0; }
     int on_start_transition(Mlt::Transition*) { return 0; }
     int on_end_transition(Mlt::Transition*) { return 0; }
+    int on_start_chain(Mlt::Chain*) { return 0; }
+    int on_end_chain(Mlt::Chain*) { return 0; }
+    int on_start_link(Mlt::Link*) { return 0; }
+    int on_end_link(Mlt::Link*) { return 0; }
 };
 
 void TimelineDock::replaceClipsWithHash(const QString& hash, Mlt::Producer& producer)
@@ -1507,11 +1881,11 @@ void TimelineDock::replaceClipsWithHash(const QString& hash, Mlt::Producer& prod
         // lookup the current track and clip index by UUID
         QScopedPointer<Mlt::ClipInfo> info(MAIN.timelineClipInfoByUuid(clip.get(kUuidProperty), trackIndex, clipIndex));
 
-        if (info && info->producer->is_valid() && trackIndex >= 0 && clipIndex >= 0 && info->producer->type() != tractor_type) {
+        if (info && info->producer->is_valid() && trackIndex >= 0 && clipIndex >= 0 && info->producer->type() != mlt_service_tractor_type) {
             if (producer.get_int(kIsProxyProperty) && info->producer->get_int(kIsProxyProperty)) {
                 // Not much to do on a proxy clip but change its resource
                 info->producer->set(kOriginalResourceProperty, producer.get("resource"));
-                auto caption = Util::baseName(ProxyManager::resource(*info->producer));
+                auto caption = Util::baseName(ProxyManager::resource(*info->producer), true);
                 if (!::qstrcmp(info->producer->get("mlt_service"), "timewarp")) {
                     caption = QString("%1 (%2x)").arg(caption, info->producer->get("warp_speed"));
                 }
