@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2020 Meltytech, LLC
+ * Copyright (c) 2011-2022 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,11 +49,12 @@ static const char* kMltXmlPropertyName = "string";
 Controller::Controller()
     : m_profile(kDefaultMltProfile)
     , m_previewProfile(kDefaultMltProfile)
+    , m_blockRefresh(false)
 {
     LOG_DEBUG() << "begin";
     m_repo = Mlt::Factory::init();
     resetLocale();
-    m_filtersClipboard.reset(new Mlt::Producer(profile(), "color", "black"));
+    initFiltersClipboard();
     updateAvformatCaching(0);
     LOG_DEBUG() << "end";
 }
@@ -106,19 +107,20 @@ static bool isFpsDifferent(double a, double b)
 int Controller::open(const QString &url, const QString& urlToSave)
 {
     int error = 0;
+    Mlt::Producer* newProducer = nullptr;
 
     close();
 
     if (Settings.playerGPU() && !profile().is_explicit())
         // Prevent loading normalizing filters, which might be Movit ones that
         // may not have a proper OpenGL context when requesting a sample frame.
-        m_producer.reset(new Mlt::Producer(profile(), "abnormal", url.toUtf8().constData()));
+        newProducer = new Mlt::Producer(profile(), "abnormal", url.toUtf8().constData());
     else
-        m_producer.reset(new Mlt::Producer(profile(), url.toUtf8().constData()));
-    if (m_producer->is_valid()) {
+        newProducer = new Mlt::Producer(profile(), url.toUtf8().constData());
+    if (newProducer && newProducer->is_valid()) {
         double fps = profile().fps();
         if (!profile().is_explicit()) {
-            profile().from_producer(*m_producer);
+            profile().from_producer(*newProducer);
             profile().set_width(Util::coerceMultiple(profile().width()));
             profile().set_height(Util::coerceMultiple(profile().height()));
         }
@@ -126,11 +128,11 @@ int Controller::open(const QString &url, const QString& urlToSave)
         setPreviewScale(Settings.playerPreviewScale());
         if ( url.endsWith(".mlt") ) {
             // Load the number of audio channels being used when this project was created.
-            int channels = m_producer->get_int(kShotcutProjectAudioChannels);
+            int channels = newProducer->get_int(kShotcutProjectAudioChannels);
             if (!channels)
                 channels = 2;
             m_audioChannels = channels;
-            if (m_producer->get_int(kShotcutProjectFolder)) {
+            if (newProducer->get_int(kShotcutProjectFolder)) {
                 QFileInfo info(url);
                 setProjectFolder(info.absolutePath());
                 ProxyManager::removePending();
@@ -140,25 +142,24 @@ int Controller::open(const QString &url, const QString& urlToSave)
         }
         if (isFpsDifferent(profile().fps(), fps) || (Settings.playerGPU() && !profile().is_explicit())) {
             // Reload with correct FPS or with Movit normalizing filters attached.
-            m_producer.reset(new Mlt::Producer(profile(), url.toUtf8().constData()));
+            delete newProducer;
+            newProducer = new Mlt::Producer(profile(), url.toUtf8().constData());
         }
-        // Convert avformat to avformat-novalidate so that XML loads faster.
-        if (!qstrcmp(m_producer->get("mlt_service"), "avformat")) {
-            m_producer->set("mlt_service", "avformat-novalidate");
-            m_producer->set("mute_on_pause", 0);
-        }
-        if (m_url.isEmpty() && QString(m_producer->get("xml")) == "was here") {
-            if (m_producer->get_int("_original_type") != tractor_type ||
-               (m_producer->get_int("_original_type") == tractor_type && m_producer->get(kShotcutXmlProperty)))
+        if (m_url.isEmpty() && QString(newProducer->get("xml")) == "was here") {
+            if (newProducer->get_int("_original_type") != mlt_service_tractor_type ||
+               (newProducer->get_int("_original_type") == mlt_service_tractor_type && newProducer->get(kShotcutXmlProperty)))
                 m_url = urlToSave;
         }
-        setImageDurationFromDefault(m_producer.data());
-        lockCreationTime(m_producer.data());
+        Producer* producer = setupNewProducer(newProducer);
+        delete newProducer;
+        newProducer = producer;
     }
     else {
-        m_producer.reset();
+        delete newProducer;
+        newProducer = nullptr;
         error = 1;
     }
+    m_producer.reset(newProducer);
     return error;
 }
 
@@ -233,6 +234,11 @@ bool Controller::isPaused() const
     return m_producer && qAbs(m_producer->get_speed()) < 0.1;
 }
 
+static void fire_jack_seek_event(mlt_properties jack, int position)
+{
+    mlt_events_fire(jack, "jack-seek", mlt_event_data_from_int(position));
+}
+
 void Controller::pause()
 {
     if (m_producer && !isPaused()) {
@@ -255,7 +261,7 @@ void Controller::pause()
         stopJack();
         int position = (m_producer && m_producer->is_valid())? m_producer->position() : 0;
         ++m_skipJackEvents;
-        mlt_events_fire(m_jackFilter->get_properties(), "jack-seek", &position, NULL);
+        fire_jack_seek_event(m_jackFilter->get_properties(), position);
     }
     setVolume(m_volume);
 }
@@ -269,10 +275,10 @@ void Controller::stop()
     stopJack();
 }
 
-void Controller::on_jack_started(mlt_properties, void* object, const mlt_position *position)
+void Controller::on_jack_started(mlt_properties, void* object, mlt_event_data data)
 {
-    if (object && position)
-        (static_cast<Controller*>(object))->onJackStarted(*position);
+    if (object)
+        (static_cast<Controller*>(object))->onJackStarted(Mlt::EventData(data).to_int());
 }
 
 void Controller::onJackStarted(int position)
@@ -284,10 +290,10 @@ void Controller::onJackStarted(int position)
     }
 }
 
-void Controller::on_jack_stopped(mlt_properties, void* object, const mlt_position *position)
+void Controller::on_jack_stopped(mlt_properties, void* object, mlt_event_data data)
 {
-    if (object && position)
-        (static_cast<Controller*>(object))->onJackStopped(*position);
+    if (object)
+        (static_cast<Controller*>(object))->onJackStopped(EventData(data).to_int());
 }
 
 void Controller::onJackStopped(int position)
@@ -319,6 +325,14 @@ void Controller::stopJack()
     }
 }
 
+void Controller::initFiltersClipboard()
+{
+    m_filtersClipboard.reset(new Mlt::Producer(profile(), "color", "black"));
+    if (m_filtersClipboard->is_valid()) {
+        m_filtersClipboard->set(kShotcutFiltersClipboard, 1);
+    }
+}
+
 bool Controller::enableJack(bool enable)
 {
 	if (!m_consumer)
@@ -412,7 +426,8 @@ double Controller::volume() const
 
 void Controller::onWindowResize()
 {
-    refreshConsumer();
+    bool scrub = isPaused() ? false : Settings.playerScrubAudio();
+    refreshConsumer(scrub);
 }
 
 void Controller::seek(int position)
@@ -434,13 +449,13 @@ void Controller::seek(int position)
     if (m_jackFilter) {
         stopJack();
         ++m_skipJackEvents;
-        mlt_events_fire(m_jackFilter->get_properties(), "jack-seek", &position, NULL);
+        fire_jack_seek_event(m_jackFilter->get_properties(), position);
     }
 }
 
 void Controller::refreshConsumer(bool scrubAudio)
 {
-    if (m_consumer) {
+    if (!m_blockRefresh && m_consumer) {
         // need to refresh consumer when paused
         m_consumer->set("scrub_audio", scrubAudio);
         m_consumer->set("refresh", 1);
@@ -455,7 +470,8 @@ bool Controller::saveXML(const QString& filename, Service* service, bool withRel
     Consumer c(profile(), "xml", proxy? filename.toUtf8().constData() : kMltXmlPropertyName);
     Service s(service? service->get_service() : m_producer->get_service());
     if (s.is_valid()) {
-        QString root = withRelativePaths? fi.absolutePath() : "";
+        // The Shotcut rule for paths in MLT XML is forward slashes as created by QFileDialog and QmlFile.
+        QString root = withRelativePaths? QDir::fromNativeSeparators(fi.absolutePath()) : "";
         s.set(kShotcutProjectAudioChannels, m_audioChannels);
         s.set(kShotcutProjectFolder, m_projectFolder.isEmpty()? 0 : 1);
         int ignore = s.get_int("ignore_points");
@@ -620,7 +636,7 @@ bool Controller::isSeekable(Producer* p) const
 
 bool Controller::isClip() const
 {
-    return !isPlaylist() && !isMultitrack();
+    return producer() && producer()->is_valid() && !isPlaylist() && !isMultitrack();
 }
 
 bool Controller::isSeekableClip()
@@ -632,14 +648,14 @@ bool Controller::isPlaylist() const
 {
     return m_producer && m_producer->is_valid() &&
           !m_producer->get_int(kShotcutVirtualClip) &&
-            (m_producer->get_int("_original_type") == playlist_type || resource() == "<playlist>");
+            (m_producer->get_int("_original_type") == mlt_service_playlist_type || resource() == "<playlist>");
 }
 
 bool Controller::isMultitrack() const
 {
     return m_producer && m_producer->is_valid()
         && !m_producer->get_int(kShotcutVirtualClip)
-        && (m_producer->get_int("_original_type") == tractor_type || resource() == "<tractor>")
+        && (m_producer->get_int("_original_type") == mlt_service_tractor_type || resource() == "<tractor>")
             && (m_producer->get(kShotcutXmlProperty));
 }
 
@@ -733,94 +749,26 @@ void Controller::next(int currentPosition)
 void Controller::setIn(int in)
 {
     if (m_producer && m_producer->is_valid()) {
-        // Adjust filters.
-        bool changed = false;
-        int n = m_producer->filter_count();
-        for (int i = 0; i < n; i++) {
-            QScopedPointer<Filter> filter(m_producer->filter(i));
-            if (filter && filter->is_valid()) {
-                if (QString(filter->get(kShotcutFilterProperty)).startsWith("fadeIn")) {
-                    if (!filter->get(kShotcutAnimInProperty)) {
-                        // Convert legacy fadeIn filters.
-                        filter->set(kShotcutAnimInProperty, filter->get_length());
-                    }
-                    filter->set_in_and_out(in, filter->get_out());
-                    changed = true;
-                } else if (!filter->get_int("_loader") && filter->get_in() == m_producer->get_in()) {
-                    filter->set_in_and_out(in, filter->get_out());
-                    changed = true;
-                }
-            }
+        int delta = in - m_producer->get_in();
+        if (!delta) {
+            return;
         }
-        if (changed)
-            Controller::refreshConsumer();
+        adjustClipFilters(*m_producer, m_producer->get_in(), m_producer->get_out(), delta, 0);
         m_producer->set("in", in);
+        Controller::refreshConsumer();
     }
 }
 
 void Controller::setOut(int out)
 {
     if (m_producer && m_producer->is_valid()) {
-        // Adjust all filters that have an explicit duration.
-        bool changed = false;
-        int n = m_producer->filter_count();
-        for (int i = 0; i < n; i++) {
-            QScopedPointer<Filter> filter(m_producer->filter(i));
-            if (filter && filter->is_valid()) {
-                QString filterName = filter->get(kShotcutFilterProperty);
-                if (filterName.startsWith("fadeOut")) {
-                    if (!filter->get(kShotcutAnimOutProperty)) {
-                        // Convert legacy fadeOut filters.
-                        filter->set(kShotcutAnimOutProperty, filter->get_length());
-                    }
-                    filter->set_in_and_out(filter->get_in(), out);
-                    changed = true;
-                    if (filterName == "fadeOutBrightness") {
-                        const char* key = filter->get_int("alpha") != 1? "alpha" : "level";
-                        filter->clear(key);
-                        filter->anim_set(key, 1, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
-                        filter->anim_set(key, 0, filter->get_length() - 1);
-                    } else if (filterName == "fadeOutMovit") {
-                        filter->clear("opacity");
-                        filter->anim_set("opacity", 1, filter->get_length() - filter->get_int(kShotcutAnimOutProperty), 0, mlt_keyframe_smooth);
-                        filter->anim_set("opacity", 0, filter->get_length() - 1);
-                    } else if (filterName == "fadeOutVolume") {
-                        filter->clear("level");
-                        filter->anim_set("level", 0, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
-                        filter->anim_set("level", -60, filter->get_length() - 1);
-                    }
-                } else if (!filter->get_int("_loader") && filter->get_out() == m_producer->get_out()) {
-                    filter->set_in_and_out(filter->get_in(), out);
-                    changed = true;
-
-                    // Update simple keyframes of non-current filters.
-                    if (filter->get_int(kShotcutAnimOutProperty) > 0
-                        && MAIN.filterController()->currentFilter()
-                        && MAIN.filterController()->currentFilter()->filter().get_filter() != filter.data()->get_filter()) {
-                        QmlMetadata* meta = MAIN.filterController()->metadataForService(filter.data());
-                        if (meta && meta->keyframes()) {
-                            foreach (QString name, meta->keyframes()->simpleProperties()) {
-                                if (!filter->get_animation(name.toUtf8().constData()))
-                                    // Cause a string property to be interpreted as animated value.
-                                    filter->anim_get_double(name.toUtf8().constData(), 0, filter->get_length());
-                                Mlt::Animation animation = filter->get_animation(name.toUtf8().constData());
-                                if (animation.is_valid()) {
-                                    int n = animation.key_count();
-                                    if (n > 1) {
-                                        animation.set_length(filter->get_length());
-                                        animation.key_set_frame(n - 2, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
-                                        animation.key_set_frame(n - 1, filter->get_length() - 1);
-                                    }
-                                }
-                            }
-                        }
-                    }
-                }
-            }
-            if (changed)
-                Controller::refreshConsumer();
+        int delta = out - m_producer->get_out();
+        if (!delta) {
+            return;
         }
+        adjustClipFilters(*m_producer, m_producer->get_in(), m_producer->get_out(), 0, -delta);
         m_producer->set("out", out);
+        Controller::refreshConsumer();
     }
 }
 
@@ -855,7 +803,7 @@ QImage Controller::image(Mlt::Frame* frame, int width, int height)
             frame->set("deinterlace_method", "onefield");
             frame->set("top_field_first", -1);
         }
-        mlt_image_format format = mlt_image_rgb24a;
+        mlt_image_format format = mlt_image_rgba;
         const uchar *image = frame->get_image(format, width, height);
         if (image) {
             QImage temp(width, height, QImage::Format_ARGB32);
@@ -896,7 +844,7 @@ void Controller::updateAvformatCaching(int trackCount)
 
 bool Controller::isAudioFilter(const QString &name)
 {
-    QScopedPointer<Properties> metadata(m_repo->metadata(filter_type, name.toLatin1().constData()));
+    QScopedPointer<Properties> metadata(m_repo->metadata(mlt_service_filter_type, name.toLatin1().constData()));
     if (metadata->is_valid()) {
         Properties tags(metadata->get_data("tags"));
         if (tags.is_valid()) {
@@ -962,6 +910,48 @@ void Controller::lockCreationTime(Producer* producer) const
     }
 }
 
+Producer* Controller::setupNewProducer(Producer* newProducer) const
+{
+    // Call this function before adding a new producer to Shotcut so that
+    // It will be configured correctly. The returned producer must be deleted.
+    QString serviceName = newProducer->get("mlt_service");
+    if (serviceName == "avformat")
+    {
+        // Convert avformat to avformat-novalidate so that XML loads faster.
+        newProducer->set("mlt_service", "avformat-novalidate");
+    }
+    setImageDurationFromDefault(newProducer);
+    lockCreationTime(newProducer);
+    newProducer->get_length_time(mlt_time_clock);
+
+    if (serviceName.startsWith("avformat"))
+    {
+        newProducer->set("mute_on_pause", 0);
+        // Encapsulate in a chain to enable timing effects
+        if (newProducer->type() != mlt_service_chain_type)
+        {
+            Mlt::Chain* chain = new Mlt::Chain(MLT.profile());
+            chain->set_source(*newProducer);
+            chain->get_length_time(mlt_time_clock);
+
+            // Move all non-loader filters to the chain in case this was a clip-only project.
+            int i = 0;
+            QScopedPointer<Mlt::Filter> filter(newProducer->filter(i));
+            while (filter && filter->is_valid()) {
+                if (!filter->get_int("_loader")) {
+                    newProducer->detach(*filter);
+                    chain->Service::attach(*filter);
+                } else {
+                    ++i;
+                }
+                filter.reset(newProducer->filter(i));
+            }
+            return chain;
+        }
+    }
+    return new Mlt::Producer(newProducer);
+}
+
 QUuid Controller::uuid(Mlt::Properties &properties) const
 {
     return {properties.get(kUuidProperty)};
@@ -983,7 +973,7 @@ QUuid Controller::ensureHasUuid(Mlt::Properties& properties) const
     return newUid;
 }
 
-void Controller::copyFilters(Producer& fromProducer, Producer& toProducer, bool fromClipboard)
+void Controller::copyFilters(Producer& fromProducer, Producer& toProducer, bool fromClipboard, bool includeDisabled)
 {
     int in = fromProducer.get(kFilterInProperty)? fromProducer.get_int(kFilterInProperty) : fromProducer.get_in();
     int out = fromProducer.get(kFilterOutProperty)? fromProducer.get_int(kFilterOutProperty) : fromProducer.get_out();
@@ -991,7 +981,8 @@ void Controller::copyFilters(Producer& fromProducer, Producer& toProducer, bool
     
     for (int i = 0; i < count; i++) {
         QScopedPointer<Mlt::Filter> fromFilter(fromProducer.filter(i));
-        if (fromFilter && fromFilter->is_valid() && !fromFilter->get_int("_loader") && fromFilter->get("mlt_service")) {
+        if (fromFilter && fromFilter->is_valid() && !fromFilter->get_int("_loader") && fromFilter->get("mlt_service")
+            && (includeDisabled || !fromFilter->get_int("disable"))) {
             Mlt::Filter toFilter(MLT.profile(), fromFilter->get("mlt_service"));
             if (toFilter.is_valid()) {
                 toFilter.inherit(*fromFilter);
@@ -1006,27 +997,47 @@ void Controller::copyFilters(Producer& fromProducer, Producer& toProducer, bool
             }
         }
     }
+
+    if (fromProducer.type() == mlt_service_chain_type && toProducer.type() == mlt_service_chain_type) {
+        Mlt::Chain fromChain(fromProducer);
+        Mlt::Chain toChain(toProducer);
+        count = fromChain.link_count();
+        for (int i = 0; i < count; i++) {
+            QScopedPointer<Mlt::Link> fromLink(fromChain.link(i));
+            if (fromLink && fromLink->is_valid() && fromLink->get("mlt_service")) {
+                Mlt::Link toLink(fromLink->get("mlt_service"));
+                if (toLink.is_valid()) {
+                    toLink.inherit(*fromLink);
+                    toChain.attach(toLink);
+                }
+            }
+        }
+    }
 }
 
 void Controller::copyFilters(Mlt::Producer* producer)
 {
     if (producer && producer->is_valid()) {
-        m_filtersClipboard.reset(new Mlt::Producer(profile(), "color", "black"));
-        copyFilters(*producer, *m_filtersClipboard);
+        initFiltersClipboard();
+        copyFilters(*producer, *m_filtersClipboard, false, false);
     } else if (m_producer && m_producer->is_valid()) {
-        m_filtersClipboard.reset(new Mlt::Producer(profile(), "color", "black"));
-        copyFilters(*m_producer, *m_filtersClipboard);
+        initFiltersClipboard();
+        copyFilters(*m_producer, *m_filtersClipboard, false, false);
     }
 }
 
-void Controller::pasteFilters(Mlt::Producer* producer)
+void Controller::pasteFilters(Mlt::Producer* producer, Producer* fromProducer)
 {
     Mlt::Producer* targetProducer = (producer && producer->is_valid())? producer
                       :(m_producer && m_producer->is_valid())? m_producer.data()
                       : nullptr;
     if (targetProducer) {
         int j = targetProducer->filter_count();
-        copyFilters(*m_filtersClipboard, *targetProducer, true);
+        if (fromProducer && fromProducer->is_valid()) {
+            copyFilters(*fromProducer, *targetProducer, true);
+        } else if (hasFiltersOnClipboard()) {
+            copyFilters(*m_filtersClipboard, *targetProducer, true);
+        }
         adjustFilters(*targetProducer, j);
     }
 }
@@ -1101,6 +1112,186 @@ void Controller::adjustFilters(Producer& producer, int index)
         MLT.refreshConsumer();
 }
 
+static void shiftKeyframes(Mlt::Service* service, QmlMetadata* meta, int delta)
+{
+    if (!meta || !meta->keyframes() || meta->keyframes()->parameterCount() < 1) {
+        return;
+    }
+    QmlKeyframesMetadata* keyMeta = meta->keyframes();
+    for (int parameterIndex = 0; parameterIndex < keyMeta->parameterCount(); parameterIndex++) {
+        QmlKeyframesParameter* paramMeta = keyMeta->parameter(parameterIndex);
+        QString name = paramMeta->property();
+        Mlt::Animation animation = service->get_animation(qUtf8Printable(name));
+        if (animation.is_valid()) {
+            // Keyframes are not allowed to have negative positions because
+            // there is no way for the user to interact with them.
+            // Handle keyframes that will become negative by deleting them.
+            if (animation.key_get_frame(0) < delta)
+            {
+                // Create a new keyframe at position that will become 0 based on interpolated value
+                int previous = animation.previous_key(delta);
+                mlt_keyframe_type existingType = animation.keyframe_type(previous);
+                if (paramMeta->isRectangle()) {
+                    auto value = service->anim_get_rect(qUtf8Printable(name), delta);
+                    service->anim_set(qUtf8Printable(name), value, delta, 0, existingType);
+                } else {
+                    double value = service->anim_get_double(qUtf8Printable(name), delta);
+                    service->anim_set(qUtf8Printable(name), value, delta, 0, existingType);
+                    foreach (QString gangName, paramMeta->gangedProperties()) {
+                        Mlt::Animation gangAnim = service->get_animation(qUtf8Printable(gangName));
+                        double gangValue = service->anim_get_double(qUtf8Printable(gangName), delta);
+                        service->anim_set(qUtf8Printable(gangName), gangValue, delta, existingType);
+                    }
+                }
+                // Remove all keyframes that will be negative position
+                int count = animation.key_count();
+                for (int keyframeIndex = 0; keyframeIndex < count;) {
+                    int frame = animation.key_get_frame(keyframeIndex);
+                    if (frame < delta) {
+                        animation.remove(frame);
+                        animation.interpolate();
+                        --count;
+                    } else {
+                        break;
+                    }
+                }
+                foreach (QString gangName, paramMeta->gangedProperties()) {
+                    Mlt::Animation gangAnim = service->get_animation(qUtf8Printable(gangName));
+                    int gangKeyCount = gangAnim.key_count();
+                    for (int keyframeIndex = 0; keyframeIndex < gangKeyCount;) {
+                        int frame = gangAnim.key_get_frame(keyframeIndex);
+                        if (frame < delta) {
+                            gangAnim.remove(frame);
+                            gangAnim.interpolate();
+                            gangKeyCount -= 1;
+                        } else {
+                            break;
+                        }
+                    }
+                }
+            }
+            // Shift all the keyframes proportional to the delta
+            animation.shift_frames(-delta);
+            foreach (QString gangName, paramMeta->gangedProperties()) {
+                Mlt::Animation gangAnim = service->get_animation(qUtf8Printable(gangName));
+                gangAnim.shift_frames(-delta);
+            }
+        }
+    }
+}
+
+void Controller::adjustFilter(Mlt::Filter* filter, int in, int out, int inDelta, int outDelta)
+{
+    if (!filter || !filter->is_valid()) {
+        return;
+    }
+
+    QString filterName = filter->get(kShotcutFilterProperty);
+    QmlMetadata* meta = MAIN.filterController()->metadataForService(filter);
+
+    if ((inDelta || outDelta) && meta && meta->mlt_service().startsWith("vidstab")) {
+        filter->clear("results");
+    }
+
+    if (inDelta) {
+        if (in + inDelta < 0) {
+            inDelta = -in;
+        }
+        if (filter->get_int(kShotcutAnimInProperty) == filter->get_int(kShotcutAnimOutProperty)) {
+            // Shift all keyframes proportional to the in delta if they are not simple keyframes
+            shiftKeyframes(filter,  meta, inDelta);
+        }
+        if (filterName.startsWith("fadeIn")) {
+            if (!filter->get(kShotcutAnimInProperty)) {
+                // Convert legacy fadeIn filters.
+                filter->set(kShotcutAnimInProperty, filter->get_length());
+            }
+            filter->set_in_and_out(in + inDelta, filter->get_out());
+            emit MAIN.serviceInChanged(inDelta, filter);
+        } else if (!filter->get_int("_loader") && filter->get_in() <= in) {
+            filter->set_in_and_out(in + inDelta, filter->get_out());
+            emit MAIN.serviceInChanged(inDelta, filter);
+        }
+    }
+
+    if (outDelta) {
+        if (filterName.startsWith("fadeOut")) {
+            if (!filter->get(kShotcutAnimOutProperty)) {
+                // Convert legacy fadeOut filters.
+                filter->set(kShotcutAnimOutProperty, filter->get_length());
+            }
+            filter->set_in_and_out(filter->get_in(), out - outDelta);
+            if (filterName == "fadeOutBrightness") {
+                const char* key = filter->get_int("alpha") != 1? "alpha" : "level";
+                filter->clear(key);
+                filter->anim_set(key, 1, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
+                filter->anim_set(key, 0, filter->get_length() - 1);
+            } else if (filterName == "fadeOutMovit") {
+                filter->clear("opacity");
+                filter->anim_set("opacity", 1, filter->get_length() - filter->get_int(kShotcutAnimOutProperty), 0, mlt_keyframe_smooth);
+                filter->anim_set("opacity", 0, filter->get_length() - 1);
+            } else if (filterName == "fadeOutVolume") {
+                filter->clear("level");
+                filter->anim_set("level", 0, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
+                filter->anim_set("level", -60, filter->get_length() - 1);
+            }
+            emit MAIN.serviceOutChanged(outDelta, filter);
+        } else if (!filter->get_int("_loader")  && filter->get_out() >= out) {
+            filter->set_in_and_out(filter->get_in(), out - outDelta);
+            emit MAIN.serviceOutChanged(outDelta, filter);
+
+            // Update simple keyframes
+            if (filter->get_int(kShotcutAnimOutProperty) && meta && meta->keyframes()) {
+                foreach (QString name, meta->keyframes()->simpleProperties()) {
+                    if (!filter->get_animation(name.toUtf8().constData()))
+                        // Cause a string property to be interpreted as animated value.
+                        filter->anim_get_double(name.toUtf8().constData(), 0, filter->get_length());
+                    Mlt::Animation animation = filter->get_animation(name.toUtf8().constData());
+                    if (animation.is_valid()) {
+                        int n = animation.key_count();
+                        if (n > 1) {
+                            animation.set_length(filter->get_length());
+                            animation.key_set_frame(n - 2, filter->get_length() - filter->get_int(kShotcutAnimOutProperty));
+                            animation.key_set_frame(n - 1, filter->get_length() - 1);
+                        }
+                    }
+                }
+            }
+        }
+    }
+}
+
+void Controller::adjustClipFilters(Mlt::Producer& producer, int in, int out, int inDelta, int outDelta)
+{
+    for (int j = 0; j < producer.filter_count(); j++) {
+        QScopedPointer<Mlt::Filter> filter(producer.filter(j));
+        adjustFilter(filter.data(), in, out, inDelta, outDelta);
+    }
+
+    // Adjust link in/out
+    if (producer.type() == mlt_service_chain_type) {
+        Mlt::Chain chain(producer);
+        int link_count = chain.link_count();
+        for (int j = 0; j < link_count; j++) {
+            QScopedPointer<Mlt::Link> link(chain.link(j));
+            QmlMetadata* meta = MAIN.filterController()->metadataForService(link.data());
+            if (link && link->is_valid()) {
+                if (inDelta) {
+                    shiftKeyframes(link.data(),  meta, inDelta);
+                }
+                if (link->get_out() >= out) {
+                    link->set_in_and_out(link->get_in(), out - outDelta);
+                    emit MAIN.serviceOutChanged(outDelta, link.data());
+                }
+                if (link->get_in() <= in) {
+                    link->set_in_and_out(in + inDelta, link->get_out());
+                    emit MAIN.serviceInChanged(inDelta, link.data());
+                }
+            }
+        }
+    }
+}
+
 void Controller::setSavedProducer(Mlt::Producer* producer)
 {
     m_savedProducer.reset(new Mlt::Producer(producer));
@@ -1237,6 +1428,12 @@ bool Controller::fullRange(Producer& producer)
     return full;
 }
 
+bool Controller::blockRefresh(bool block)
+{
+    m_blockRefresh = block;
+    return m_blockRefresh;
+}
+
 void TransportControl::play(double speed)
 {
     MLT.play(speed);
