@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2020 Meltytech, LLC
+ * Copyright (c) 2011-2021 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,13 +30,8 @@
 // forward declarations
 class QQuickView;
 
-#if defined(Q_OS_UNIX) && !defined(Q_OS_MAC)
-#   define MLT_LC_CATEGORY LC_NUMERIC
-#   define MLT_LC_NAME     "LC_NUMERIC"
-#else
-#   define MLT_LC_CATEGORY LC_ALL
-#   define MLT_LC_NAME     "LC_ALL"
-#endif
+#define MLT_LC_CATEGORY LC_ALL
+#define MLT_LC_NAME     "LC_ALL"
 
 namespace Mlt {
 
@@ -119,16 +114,22 @@ public:
     void setImageDurationFromDefault(Service* service) const;
     void setDurationFromDefault(Producer* service) const;
     void lockCreationTime(Producer* producer) const;
+    Producer* setupNewProducer(Producer* newProducer) const;
     QUuid uuid(Mlt::Properties &properties) const;
     void setUuid(Mlt::Properties &properties, QUuid uid) const;
     QUuid ensureHasUuid(Mlt::Properties& properties) const;
-    static void copyFilters(Mlt::Producer& fromProducer, Mlt::Producer& toProducer, bool fromClipboard = false);
+    static void copyFilters(Mlt::Producer& fromProducer, Mlt::Producer& toProducer, bool fromClipboard = false, bool includeDisabled = true);
     void copyFilters(Mlt::Producer* producer = nullptr);
-    void pasteFilters(Mlt::Producer* producer = nullptr);
+    void pasteFilters(Mlt::Producer* producer = nullptr, Mlt::Producer* fromProducer = nullptr);
     static void adjustFilters(Mlt::Producer& producer, int startIndex = 0);
+    static void adjustFilter(Mlt::Filter* filter, int in, int out, int inDelta, int outDelta);
+    static void adjustClipFilters(Mlt::Producer& producer, int in, int out, int inDelta, int outDelta);
     bool hasFiltersOnClipboard() const {
         return m_filtersClipboard->is_valid() && m_filtersClipboard->filter_count() > 0;
     }
+    QString filtersClipboardXML() {
+        return XML(m_filtersClipboard.get());
+    }
 
     int audioChannels() const {
         return m_audioChannels;
@@ -169,6 +170,23 @@ public:
     void updatePreviewProfile();
     static void purgeMemoryPool();
     static bool fullRange(Mlt::Producer& producer);
+    static bool isMltXml(const QString& s) {
+        return s.contains("<mlt ");
+    }
+    bool blockRefresh(bool block);
+
+    class RefreshBlocker
+    {
+    public:
+        RefreshBlocker()
+        {
+            singleton().blockRefresh(true);
+        }
+        ~RefreshBlocker()
+        {
+            singleton().blockRefresh(false);
+        }
+    };
 
 protected:
     Mlt::Repository* m_repo;
@@ -188,12 +206,14 @@ private:
     unsigned m_skipJackEvents{0};
     QString m_projectFolder;
     QMutex m_saveXmlMutex;
+    bool m_blockRefresh;
 
-    static void on_jack_started(mlt_properties owner, void* object, const mlt_position *position);
+    static void on_jack_started(mlt_properties owner, void* object, mlt_event_data data);
     void onJackStarted(int position);
-    static void on_jack_stopped(mlt_properties owner, void* object, const mlt_position *position);
+    static void on_jack_stopped(mlt_properties owner, void* object, mlt_event_data data);
     void onJackStopped(int position);
     void stopJack();
+    void initFiltersClipboard();
 };
 
 } // namespace
