@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2020 Meltytech, LLC
+ * Copyright (c) 2014-2021 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,7 +38,8 @@ static bool isMltClass(const QStringRef& name)
     return name == "profile" || name == "producer" ||
            name == "filter" || name == "playlist" ||
            name == "tractor" || name == "track" ||
-           name == "transition" || name == "consumer";
+           name == "transition" || name == "consumer" ||
+           name == "chain" || name == "link";
 }
 
 static bool isNetworkResource(const QString& string)
@@ -68,7 +69,7 @@ MltXmlChecker::MltXmlChecker()
     m_unlinkedFilesModel.setColumnCount(ColumnCount);
 }
 
-bool MltXmlChecker::check(const QString& fileName)
+QXmlStreamReader::Error MltXmlChecker::check(const QString& fileName)
 {
     LOG_DEBUG() << "begin";
 
@@ -88,17 +89,29 @@ bool MltXmlChecker::check(const QString& fileName)
                 m_newXml.writeCharacters("\n");
                 m_newXml.writeStartElement("mlt");
                 foreach (QXmlStreamAttribute a, m_xml.attributes()) {
-                    if (a.name().toString().toUpper() != "LC_NUMERIC") {
-                        m_newXml.writeAttribute(a);
-                    } else {
+                    if (a.name().toString().toUpper() == "LC_NUMERIC") {
                         QString value = a.value().toString().toUpper();
                         // Determine whether this document uses a non-POSIX/-generic numeric locale.
                         m_usesLocale = (value != "" && value != "C" && value != "POSIX" && QLocale().decimalPoint() != '.');
                         // Upon correcting the document to conform to current system,
                         // update the declared LC_NUMERIC.
                         m_newXml.writeAttribute("LC_NUMERIC", m_usesLocale? QLocale().name() : "C");
+                    } else if (a.name().toString().toLower() == "version") {
+                        m_mltVersion = QVersionNumber::fromString(a.value());
+                    } else if (a.name().toString().toLower() == "title") {
+                        m_newXml.writeAttribute(a.name().toString(), "Shotcut version " SHOTCUT_VERSION);
+                        auto parts = a.value().split(' ');
+                        LOG_DEBUG() << parts;
+                        if (parts.size() > 2 && parts[1] == "version") {
+                            m_shotcutVersion = parts[2].toString();
+                        }
+                    } else {
+                        m_newXml.writeAttribute(a);
                     }
                 }
+                if (!checkMltVersion()) {
+                    return QXmlStreamReader::CustomError;
+                }
                 // We cannot apply the locale change to the session at this point
                 // because we are merely checking at this point and not loading.
                 // Save the current locale state.
@@ -127,6 +140,7 @@ bool MltXmlChecker::check(const QString& fileName)
             }
         }
     }
+    LOG_DEBUG() << m_tempFile->fileName();
     if (m_tempFile->isOpen()) {
         m_tempFile->close();
 
@@ -135,8 +149,8 @@ bool MltXmlChecker::check(const QString& fileName)
 //        LOG_DEBUG() << m_tempFile->readAll().constData();
 //        m_tempFile->close();
     }
-    LOG_DEBUG() << "end";
-    return m_xml.error() == QXmlStreamReader::NoError;
+    LOG_DEBUG() << "end" << m_xml.errorString();
+    return m_xml.error();
 }
 
 QString MltXmlChecker::errorString() const
@@ -187,12 +201,14 @@ void MltXmlChecker::readMlt()
             break;
         case QXmlStreamReader::StartElement: {
             const QString element = m_xml.name().toString();
+            isPropertyElement = false;
             if (element == "property") {
-                const QString name = m_xml.attributes().value("name").toString();
-                m_properties << MltProperty(name, m_xml.readElementText());
                 isPropertyElement = true;
+                if (isMltClass(&mlt_class)) {
+                    const QString name = m_xml.attributes().value("name").toString();
+                    m_properties << MltProperty(name, m_xml.readElementText());
+                }
             } else {
-                isPropertyElement = false;
                 processProperties();
                 m_newXml.writeStartElement(m_xml.namespaceUri().toString(), element);
                 if (isMltClass(m_xml.name()))
@@ -247,7 +263,8 @@ void MltXmlChecker::processProperties()
         newProperties << MltProperty(p.first, p.second);
     }
 
-    if (mlt_class == "filter" || mlt_class == "transition" || mlt_class == "producer") {
+    if (mlt_class == "filter" || mlt_class == "transition" || mlt_class == "producer"
+            || mlt_class == "chain" || mlt_class == "link") {
         checkGpuEffects(mlt_service);
         checkCpuEffects(mlt_service);
         checkUnlinkedFile(mlt_service);
@@ -381,17 +398,35 @@ static QString getPrefix(const QString& name, const QString& value)
     return QString();
 }
 
+static QString getSuffix(const QString& name, const QString& value)
+{
+    // avformat is only using "resource"
+    if (name == "resource") {
+        const QString queryDelimiter("\\?");
+        const auto i = value.lastIndexOf(queryDelimiter);
+        // webvfx "plain:"
+        if (i > 0) {
+            return value.mid(i);
+        }
+    }
+    return QString();
+}
+
 bool MltXmlChecker::readResourceProperty(const QString& name, QString& value)
 {
-    if (mlt_class == "filter" || mlt_class == "transition" || mlt_class == "producer")
+    if (mlt_class == "filter" || mlt_class == "transition" || mlt_class == "producer"
+            || mlt_class == "chain" || mlt_class == "link")
     if (name == "resource" || name == "src" || name == "filename"
             || name == "luma" || name == "luma.resource" || name == "composite.luma"
             || name == "producer.resource" || name == "av.file" || name == "warp_resource") {
 
         // Handle special prefix such as "plain:" or speed.
         m_resource.prefix = getPrefix(name, value);
+        m_resource.suffix = getSuffix(name, value);
+        auto filePath = value.mid(m_resource.prefix.size());
+        filePath = filePath.left(filePath.size() - m_resource.suffix.size());
         // Save the resource name (minus prefix) for later check for unlinked files.
-        m_resource.info.setFile(value.mid(m_resource.prefix.size()));
+        m_resource.info.setFile(filePath);
         if (!isNetworkResource(value) && m_resource.info.isRelative() && !Util::hasDriveLetter(value))
             m_resource.info.setFile(m_fileInfo.canonicalPath(), m_resource.info.filePath());
         return true;
@@ -427,6 +462,10 @@ void MltXmlChecker::checkUnlinkedFile(const QString& mlt_service)
     if (fileName != "vidstab.trf")
     // not the generic <producer> resource
     if (fileName != "<producer>")
+    // not an invalid <tractor>
+    if (fileName != "<tractor>")
+    // not an invalid blank
+    if (mlt_service != "blank" || fileName != "blank")
     // not a URL
     if (!m_resource.info.filePath().isEmpty() && !isNetworkResource(m_resource.info.filePath()))
     // not an image sequence
@@ -461,6 +500,7 @@ bool MltXmlChecker::fixUnlinkedFile(QString& value)
                 value = value.mid(m_fileInfo.canonicalPath().size() + 1);
             // Restore special prefix such as "plain:" or speed value.
             value.prepend(m_resource.prefix);
+            value.append(m_resource.suffix);
             m_isCorrected = true;
 
             Mlt::Producer producer(MLT.profile(), m_resource.info.filePath().toUtf8().constData());
@@ -597,11 +637,16 @@ void MltXmlChecker::replaceWebVfxChoppyFilter(QString& mlt_service, QVector<MltX
             m_isUpdated = true;
             for (auto& p : properties) {
                 if (p.first == "resource") {
-                    if (QFileInfo(p.second).isRelative()) {
-                        QDir projectDir(QFileInfo(m_tempFile->fileName()).dir());
-                        p.second = projectDir.filePath(p.second);
+                    auto pathName = p.second;
+                    auto plain = QLatin1String("plain:");
+                    if (pathName.startsWith(plain)) {
+                        pathName = pathName.mid(plain.size());
                     }
-                    QFile file(p.second);
+                    if (QFileInfo(pathName).isRelative()) {
+                        QDir projectDir(QFileInfo(m_tempFile->fileName()).dir());
+                        pathName = projectDir.filePath(pathName);
+                    }
+                    QFile file(pathName);
                     if (file.open(QIODevice::ReadOnly)) {
                         p.first = "html";
                         p.second = QString::fromUtf8(file.readAll());
@@ -641,6 +686,23 @@ void MltXmlChecker::checkForProxy(const QString& mlt_service, QVector<MltXmlChec
                 return;
             }
         }
+
+        // Use GoPro LRV if available
+        if (QFile::exists(ProxyManager::GoProProxyFilePath(resource))) {
+            for (auto& p : properties) {
+                if (p.first == "resource") {
+                    p.second = ProxyManager::GoProProxyFilePath(resource);
+                    if (isTimewarp) {
+                        p.second = QString("%1:%2").arg(speed).arg(p.second);
+                    }
+                    properties << MltProperty(kIsProxyProperty, "1");
+                    properties << MltProperty(kOriginalResourceProperty, resource);
+                    m_isUpdated = true;
+                    return;
+                }
+            }
+        }
+
         QDir proxyDir(Settings.proxyFolder());
         QDir projectDir(QFileInfo(m_tempFile->fileName()).dir());
         QString fileName = hash + ProxyManager::videoFilenameExtension();
@@ -703,3 +765,11 @@ void MltXmlChecker::checkForProxy(const QString& mlt_service, QVector<MltXmlChec
         }
     }
 }
+
+bool MltXmlChecker::checkMltVersion()
+{
+    if (m_mltVersion.majorVersion() > 7) {
+        return false;
+    }
+    return true;
+}
