@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2020 Meltytech, LLC
+ * Copyright (c) 2013-2021 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,10 +59,11 @@ void AudioLevelsTask::start(Mlt::Producer& producer, QObject* object, const QMod
     if (Settings.timelineShowWaveforms() && producer.is_valid()) {
 
         QString serviceName = producer.get("mlt_service");
-        if (DB.isShutdown()
-            || serviceName == "pixbuf" || serviceName == "qimage" || serviceName == "webvfx"
-            || serviceName == "color"|| serviceName.startsWith("frei0r"))
+        if (serviceName == "pixbuf" || serviceName == "qimage" || serviceName == "webvfx" ||
+            serviceName == "color"  || serviceName.startsWith("frei0r") ||
+            (serviceName.startsWith("avformat") && producer.get_int("audio_index") == -1)) {
             return;
+        }
 
         AudioLevelsTask* task = new AudioLevelsTask(producer, object, index);
         tasksListMutex.lock();
@@ -167,7 +168,7 @@ void AudioLevelsTask::run()
     // 2 channels interleaved of uchar values
     QVariantList levels;
     QImage image = DB.getThumbnail(cacheKey());
-    if ((image.isNull() || m_isForce) && !DB.isFailing()) {
+    if (image.isNull() || m_isForce) {
         const char* key[2] = { "meta.media.audio_level.0", "meta.media.audio_level.1"};
         QElapsedTimer updateTime;
         updateTime.start();
@@ -188,7 +189,7 @@ void AudioLevelsTask::run()
             if (frame && frame->is_valid() && !frame->get_int("test_audio")) {
                 mlt_audio_format format = mlt_audio_s16;
                 int frequency = 48000;
-                int samples = mlt_sample_calculator(m_producers.first().first->get_fps(), frequency, i);
+                int samples = mlt_audio_calculate_frame_samples(m_producers.first().first->get_fps(), frequency, i);
                 frame->get_audio(format, frequency, channels, samples);
                 // for each channel
                 for (int channel = 0; channel < channels; channel++)
