@@ -22,6 +22,8 @@
 
 #include <Logger.h>
 
+#include <QTimer>
+
 static const quintptr NO_PARENT_ID = quintptr(-1);
 
 KeyframesModel::KeyframesModel(QObject* parent)
@@ -80,7 +82,9 @@ QVariant KeyframesModel::data(const QModelIndex& index, int role) const
                         default:
                             break;
                         }
-                        return QString("%1 - %2").arg(m_filter->timeFromFrames(position)).arg(type);
+                        double value = m_filter->getDouble(name, position);
+                        QString units = m_metadata->keyframes()->parameter(m_metadataIndex[index.internalId()])->units();
+                        return QString("%1 - %2\n%3%4").arg(m_filter->timeFromFrames(position)).arg(type).arg(value).arg(units);
                     }
                     case FrameNumberRole:
                         return position;
@@ -94,11 +98,17 @@ QVariant KeyframesModel::data(const QModelIndex& index, int role) const
                         return result;
                     }
                     case MaximumFrameRole: {
-                        int minimum = animation.previous_key(qMax(0, position - 1)) + 1;
-                        int result = animation.next_key(position + 1) - 1;
-                        result = (result < minimum) ? m_filter->duration() : result;
+                        int result = 0;
+                        if (index.row() >= rowCount(index.parent()) - 1) {
+                            // Last Keyframe
+                            result = m_filter->producer().get_out();
+                        } else {
+                            int minimum = (index.row() > 0 && position > 0) ? (animation.previous_key(position - 1) + 1) : 0;
+                            result = animation.next_key(position + 1) - 1;
+                            result = (result < minimum) ? m_filter->producer().get_out() : result;
+                        }
 //                        LOG_DEBUG() << "keyframeIndex" << index.row() << "maximumFrame" << result;
-                        return result - 1;
+                        return result;
                     }
                     default:
                         break;
@@ -118,9 +128,60 @@ QVariant KeyframesModel::data(const QModelIndex& index, int role) const
         case IsCurveRole:
             return m_metadata->keyframes()->parameter(m_metadataIndex[index.row()])->isCurve();
         case MinimumValueRole:
-            return m_metadata->keyframes()->parameter(m_metadataIndex[index.row()])->minimum();
+        {
+            QmlKeyframesParameter* param = m_metadata->keyframes()->parameter(m_metadataIndex[index.row()]);
+            if (param->rangeType() == QmlKeyframesParameter::MinMax) {
+                return m_metadata->keyframes()->parameter(m_metadataIndex[index.row()])->minimum();
+            } else if (param->rangeType() == QmlKeyframesParameter::ClipLength) {
+                return 0.0;
+            }
+            return 0.0;
+        }
         case MaximumValueRole:
-            return m_metadata->keyframes()->parameter(m_metadataIndex[index.row()])->maximum();
+        {
+            QmlKeyframesParameter* param = m_metadata->keyframes()->parameter(m_metadataIndex[index.row()]);
+            if (param->rangeType() == QmlKeyframesParameter::MinMax) {
+                return m_metadata->keyframes()->parameter(m_metadataIndex[index.row()])->maximum();
+            } else if (param->rangeType() == QmlKeyframesParameter::ClipLength) {
+                int length = m_filter->producer().get_length() - m_filter->in();
+                return (double)length / MLT.profile().fps();
+            }
+            return 0.0;
+        }
+        case LowestValueRole:
+        {
+            QmlKeyframesParameter* param = m_metadata->keyframes()->parameter(m_metadataIndex[index.row()]);
+            Mlt::Animation animation = m_filter->getAnimation(param->property());
+            double min = std::numeric_limits<double>::max();
+            if (animation.is_valid()) {
+                for (int i = 0; i < animation.key_count(); i++) {
+                    int frame = animation.key_get_frame(i);
+                    if (frame >= 0) {
+                        double value = m_filter->getDouble(param->property(), frame);
+                        if (value < min) min = value;
+                    }
+                }
+            }
+            if (min == std::numeric_limits<double>::max()) min = 0;
+            return min;
+        }
+        case HighestValueRole:
+        {
+            QmlKeyframesParameter* param = m_metadata->keyframes()->parameter(m_metadataIndex[index.row()]);
+            Mlt::Animation animation = m_filter->getAnimation(param->property());
+            double max = std::numeric_limits<double>::lowest();
+            if (animation.is_valid()) {
+                for (int i = 0; i < animation.key_count(); i++) {
+                    int frame = animation.key_get_frame(i);
+                    if (frame >= 0) {
+                        double value = m_filter->getDouble(param->property(), frame);
+                        if (value > max) max = value;
+                    }
+                }
+            }
+            if (max == std::numeric_limits<double>::lowest()) max = 0;
+            return max;
+        }
         default:
             break;
         }
@@ -158,6 +219,8 @@ QHash<int, QByteArray> KeyframesModel::roleNames() const
     roles[IsCurveRole]      = "isCurve";
     roles[MinimumValueRole] = "minimum";
     roles[MaximumValueRole] = "maximum";
+    roles[LowestValueRole] = "lowest";
+    roles[HighestValueRole] = "highest";
     roles[FrameNumberRole]  = "frame";
     roles[KeyframeTypeRole] = "interpolation";
     roles[NumericValueRole] = "value";
@@ -174,15 +237,13 @@ void KeyframesModel::load(QmlFilter* filter, QmlMetadata* meta)
     m_metadataIndex.clear();
     m_filter = filter;
     m_metadata = meta;
-    if (m_filter && m_metadata)
+    if (m_filter && m_metadata && m_filter->animateIn() <= 0 && m_filter->animateOut() <= 0)
     for (int i = 0; i < m_metadata->keyframes()->parameterCount(); i++) {
-        if (!m_metadata->keyframes()->parameter(i)->isSimple() || (m_filter->animateIn() <= 0 && m_filter->animateOut() <= 0)) {
-            if (m_filter->keyframeCount(m_metadata->keyframes()->parameter(i)->property()) > 0) {
-                m_propertyNames << m_metadata->keyframes()->parameter(i)->property();
-                m_keyframeCounts << keyframeCount(m_propertyNames.count() - 1);
-                m_metadataIndex << i;
-//            LOG_DEBUG() << m_propertyNames.last() << m_filter->get(m_propertyNames.last()) << keyframeCount(i);
-            }
+        if (m_filter->keyframeCount(m_metadata->keyframes()->parameter(i)->property()) > 0) {
+            m_propertyNames << m_metadata->keyframes()->parameter(i)->property();
+            m_keyframeCounts << keyframeCount(m_propertyNames.count() - 1);
+            m_metadataIndex << i;
+//        LOG_DEBUG() << m_propertyNames.last() << m_filter->get(m_propertyNames.last()) << keyframeCount(i);
         }
     }
     endResetModel();
@@ -195,7 +256,9 @@ bool KeyframesModel::remove(int parameterIndex, int keyframeIndex)
     if (m_filter && parameterIndex < m_propertyNames.count()) {
         QString name = m_propertyNames[parameterIndex];
         Mlt::Animation animation = m_filter->getAnimation(name);
-        if (animation.is_valid()) {
+        // Do not allow the user to delete the last keyframe.
+        // Keyframes should be disabled in the filter panel instead
+        if (animation.is_valid() && animation.key_count() > 1) {
             int frame_num = animation.key_get_frame(keyframeIndex);
             error = animation.remove(frame_num);
             if (!error) {
@@ -218,6 +281,7 @@ bool KeyframesModel::remove(int parameterIndex, int keyframeIndex)
                     modelIndex = index(keyframeIndex, 0, index(parameterIndex));
                     emit dataChanged(modelIndex, modelIndex, QVector<int>() << MinimumFrameRole);
                 }
+                emit dataChanged(index(parameterIndex), index(parameterIndex), QVector<int>() << LowestValueRole << HighestValueRole);
                 emit m_filter->changed();
             }
         }
@@ -296,6 +360,7 @@ bool KeyframesModel::setInterpolation(int parameterIndex, int keyframeIndex, Int
                 emit dataChanged(modelIndex, modelIndex, QVector<int>() << KeyframeTypeRole << NameRole);
                 error = false;
                 emit m_filter->changed();
+                emit m_filter->propertyChanged(name.toUtf8().constData());
             }
         }
     }
@@ -304,31 +369,56 @@ bool KeyframesModel::setInterpolation(int parameterIndex, int keyframeIndex, Int
     return error;
 }
 
-bool KeyframesModel::setPosition(int parameterIndex, int keyframeIndex, int position)
+void KeyframesModel::setKeyframePosition(int parameterIndex, int keyframeIndex, int position)
 {
-    bool error = true;
-    if (m_filter && parameterIndex < m_propertyNames.count()) {
-        QString name = m_propertyNames[parameterIndex];
-        Mlt::Animation animation = m_filter->getAnimation(name);
-        if (animation.is_valid()) {
-            if (!animation.key_set_frame(keyframeIndex, position)) {
-//                LOG_DEBUG() << "keyframe index" << keyframeIndex << "position" << position;
-                foreach (name, m_metadata->keyframes()->parameter(m_metadataIndex[parameterIndex])->gangedProperties()) {
-                    Mlt::Animation animation = m_filter->getAnimation(name);
-                    if (animation.is_valid())
-                        animation.key_set_frame(keyframeIndex, position);
-                }
-                QModelIndex modelIndex = index(keyframeIndex, 0, index(parameterIndex));
-                emit dataChanged(modelIndex, modelIndex, QVector<int>() << FrameNumberRole << NameRole);
-                updateNeighborsMinMax(parameterIndex, keyframeIndex);
-                error = false;
-                emit m_filter->changed();
-            }
-        }
+    if (!m_filter) {
+        LOG_ERROR() << "Invalid Filter" << parameterIndex;
+        return;
     }
-    if (error)
-        LOG_ERROR() << "failed to set keyframe" << "at parameter index" << parameterIndex << "keyframeIndex" << keyframeIndex << "to position" << position;
-    return error;
+
+    if (parameterIndex >= m_propertyNames.count()) {
+        LOG_ERROR() << "Invalid parameter index" << parameterIndex;
+        return;
+    }
+
+    QString name = m_propertyNames[parameterIndex];
+    Mlt::Animation animation = m_filter->getAnimation(name);
+    if (!animation.is_valid()) {
+        LOG_ERROR() << "Invalid animation" << parameterIndex;
+        return;
+    }
+
+    if (keyframeIndex >= animation.key_count()) {
+        LOG_ERROR() << "Invalid key index" << parameterIndex << keyframeIndex;
+        return;
+    }
+
+    if (position < 0) {
+        LOG_ERROR() << "Invalid key position" << parameterIndex << keyframeIndex << position;
+        return;
+    }
+
+    int prevPosition = animation.key_get_frame(keyframeIndex);
+    if (position == prevPosition) {
+        LOG_ERROR() << "Position did not change" << parameterIndex << keyframeIndex << position;
+        return;
+    }
+
+    if (animation.key_set_frame(keyframeIndex, position)) {
+        LOG_ERROR() << "Failed to set position" << parameterIndex << keyframeIndex << position;
+        return;
+    }
+
+    foreach (name, m_metadata->keyframes()->parameter(m_metadataIndex[parameterIndex])->gangedProperties()) {
+        Mlt::Animation animation = m_filter->getAnimation(name);
+        if (animation.is_valid())
+            animation.key_set_frame(keyframeIndex, position);
+    }
+    QModelIndex modelIndex = index(keyframeIndex, 0, index(parameterIndex));
+    emit dataChanged(modelIndex, modelIndex, QVector<int>() << FrameNumberRole << NameRole);
+    updateNeighborsMinMax(parameterIndex, keyframeIndex);
+    emit m_filter->changed();
+    emit m_filter->propertyChanged(name.toUtf8().constData());
 }
 
 void KeyframesModel::addKeyframe(int parameterIndex, double value, int position, KeyframesModel::InterpolationType type)
@@ -383,18 +473,102 @@ void KeyframesModel::addKeyframe(int parameterIndex, int position)
     }
 }
 
-void KeyframesModel::setKeyframe(int parameterIndex, double value, int position, KeyframesModel::InterpolationType type)
+void KeyframesModel::setKeyframeValue(int parameterIndex, int keyframeIndex, double value)
 {
-    if (m_filter && parameterIndex < m_propertyNames.count()) {
-        QString name = m_propertyNames[parameterIndex];
-        m_filter->filter().anim_set(name.toUtf8().constData(), value, position, m_filter->duration(), mlt_keyframe_type(type));
-        foreach (name, m_metadata->keyframes()->parameter(m_metadataIndex[parameterIndex])->gangedProperties())
-            m_filter->filter().anim_set(name.toUtf8().constData(), value, position, m_filter->duration(), mlt_keyframe_type(type));
-        emit m_filter->changed();
-        QModelIndex modelIndex = index(keyframeIndex(parameterIndex, position), 0, index(parameterIndex));
-        emit dataChanged(modelIndex, modelIndex, QVector<int>() << NumericValueRole << NameRole);
-        updateNeighborsMinMax(parameterIndex, modelIndex.row());
+    if (!m_filter) {
+        LOG_ERROR() << "Invalid Filter" << parameterIndex;
+        return;
     }
+
+    if (parameterIndex >= m_propertyNames.count()) {
+        LOG_ERROR() << "Invalid parameter index" << parameterIndex;
+        return;
+    }
+
+    QString name = m_propertyNames[parameterIndex];
+    Mlt::Animation animation = m_filter->getAnimation(name);
+    if (!animation.is_valid()) {
+        LOG_ERROR() << "Invalid animation" << parameterIndex;
+        return;
+    }
+
+    if (keyframeIndex >= animation.key_count()) {
+        LOG_ERROR() << "Invalid key index" << parameterIndex << keyframeIndex;
+        return;
+    }
+
+    int position = animation.key_get_frame(keyframeIndex);
+    if (position < 0) {
+        LOG_ERROR() << "Invalid position" << parameterIndex << keyframeIndex;
+        return;
+    }
+
+    mlt_keyframe_type type = animation.key_get_type(keyframeIndex);
+    m_filter->service().anim_set(name.toUtf8().constData(), value, position, m_filter->duration(), type);
+    foreach (name, m_metadata->keyframes()->parameter(m_metadataIndex[parameterIndex])->gangedProperties())
+        m_filter->service().anim_set(name.toUtf8().constData(), value, position, m_filter->duration(), type);
+    emit m_filter->changed();
+    emit m_filter->propertyChanged(name.toUtf8().constData());
+    QModelIndex modelIndex = index(keyframeIndex, 0, index(parameterIndex));
+    emit dataChanged(modelIndex, modelIndex, QVector<int>() << NumericValueRole << NameRole);
+    emit dataChanged(index(parameterIndex), index(parameterIndex), QVector<int>() << LowestValueRole << HighestValueRole);
+}
+
+void KeyframesModel::setKeyframeValuePosition(int parameterIndex, int keyframeIndex, double value, int position)
+{
+    if (!m_filter) {
+        LOG_ERROR() << "Invalid Filter" << parameterIndex;
+        return;
+    }
+
+    if (parameterIndex >= m_propertyNames.count()) {
+        LOG_ERROR() << "Invalid parameter index" << parameterIndex;
+        return;
+    }
+
+    QString name = m_propertyNames[parameterIndex];
+    Mlt::Animation animation = m_filter->getAnimation(name);
+    if (!animation.is_valid()) {
+        LOG_ERROR() << "Invalid animation" << parameterIndex;
+        return;
+    }
+
+    if (keyframeIndex >= animation.key_count()) {
+        LOG_ERROR() << "Invalid key index" << parameterIndex << keyframeIndex;
+        return;
+    }
+
+    if (position < 0) {
+        LOG_ERROR() << "Invalid key position" << parameterIndex << keyframeIndex << position;
+        return;
+    }
+
+    QVector<int> roles;
+    int prevPosition = animation.key_get_frame(keyframeIndex);
+    if (position != prevPosition) {
+        if (animation.key_set_frame(keyframeIndex, position)) {
+            LOG_ERROR() << "Failed to set position" << parameterIndex << keyframeIndex << position;
+            return;
+        }
+        foreach (name, m_metadata->keyframes()->parameter(m_metadataIndex[parameterIndex])->gangedProperties()) {
+            Mlt::Animation animation = m_filter->getAnimation(name);
+            if (animation.is_valid())
+                animation.key_set_frame(keyframeIndex, position);
+        }
+        roles << FrameNumberRole;
+        updateNeighborsMinMax(parameterIndex, keyframeIndex);
+    }
+
+    mlt_keyframe_type type = animation.key_get_type(keyframeIndex);
+    m_filter->service().anim_set(name.toUtf8().constData(), value, position, m_filter->duration(), type);
+    foreach (name, m_metadata->keyframes()->parameter(m_metadataIndex[parameterIndex])->gangedProperties())
+        m_filter->service().anim_set(name.toUtf8().constData(), value, position, m_filter->duration(), type);
+    emit m_filter->changed();
+    emit m_filter->propertyChanged(name.toUtf8().constData());
+    roles << NumericValueRole << NameRole;
+    QModelIndex modelIndex = index(keyframeIndex, 0, index(parameterIndex));
+    emit dataChanged(modelIndex, modelIndex, roles);
+    emit dataChanged(index(parameterIndex), index(parameterIndex), QVector<int>() << LowestValueRole << HighestValueRole);
 }
 
 bool KeyframesModel::isKeyframe(int parameterIndex, int position)
@@ -407,20 +581,105 @@ bool KeyframesModel::isKeyframe(int parameterIndex, int position)
     return false;
 }
 
+bool KeyframesModel::advancedKeyframesInUse()
+{
+    if (m_filter && m_metadata && m_filter->animateIn() <= 0 && m_filter->animateOut() <= 0)
+    for (int i = 0; i < m_metadata->keyframes()->parameterCount(); i++) {
+        if (m_filter->keyframeCount(m_metadata->keyframes()->parameter(i)->property()) > 0) {
+            return true;
+        }
+    }
+    return false;
+}
+
+void KeyframesModel::removeAdvancedKeyframes()
+{
+    if (m_filter && m_metadata && m_filter->animateIn() <= 0 && m_filter->animateOut() <= 0) {
+        for (int i = 0; i < m_metadata->keyframes()->parameterCount(); i++) {
+            QString name = m_metadata->keyframes()->parameter(i)->property();
+            if (m_filter->keyframeCount(name) > 0) {
+                m_filter->set(name, m_filter->get(name, 0));
+                for (auto& key : m_metadata->keyframes()->parameter(i)->gangedProperties()) {
+                    m_filter->set(key, m_filter->get(key, 0));
+                }
+            }
+        }
+        reload();
+    }
+}
+
+bool KeyframesModel::simpleKeyframesInUse()
+{
+    return m_filter && m_metadata && (m_filter->animateIn() > 0 || m_filter->animateOut() > 0);
+}
+
+void KeyframesModel::removeSimpleKeyframes()
+{
+    if (simpleKeyframesInUse()) {
+        for (int i = 0; i < m_metadata->keyframes()->parameterCount(); i++) {
+            QString name = m_metadata->keyframes()->parameter(i)->property();
+            auto parameter = m_metadata->keyframes()->parameter(i);
+            if (parameter->gangedProperties().count() > 0) {
+                // Do not attempt to detect all matching keyframes for ganged properties.
+                // Always convert to advanced.
+                continue;
+            }
+            bool clearKeyframes = true;
+            // Find out if all keyframe values are the same. If they are all the same,
+            // then clear keyframes and set the parameter to a single value.
+            if (parameter->isRectangle()) {
+                auto firstValue = m_filter->getRect(name, 0);
+                Mlt::Animation anim = m_filter->getAnimation(name);
+                if (anim.is_valid()) {
+                    for (int k = 1; k < anim.key_count(); k++) {
+                        auto value = m_filter->getRect(name, anim.key_get_frame(k));
+                        if (value != firstValue) {
+                            clearKeyframes = false;
+                            break;
+                        }
+                    }
+                }
+                if (clearKeyframes) {
+                    m_filter->set(name, firstValue);
+                }
+            } else {
+                double firstValue = m_filter->getDouble(name, 0);
+                Mlt::Animation anim = m_filter->getAnimation(name);
+                if (anim.is_valid()) {
+                    for (int k = 1; k < anim.key_count(); k++) {
+                        auto value = m_filter->getDouble(name, anim.key_get_frame(k));
+                        if (value != firstValue) {
+                            clearKeyframes = false;
+                            break;
+                        }
+                    }
+                }
+                if (clearKeyframes) {
+                    m_filter->set(name, firstValue);
+                    m_filter->blockSignals(true);
+                    for (auto& key : parameter->gangedProperties()) {
+                        m_filter->set(key, m_filter->getDouble(key, 0));
+                    }
+                    m_filter->blockSignals(false);
+                }
+            }
+        }
+        m_filter->clearAnimateInOut();
+    }
+}
+
 void KeyframesModel::reload()
 {
     beginResetModel();
     m_propertyNames.clear();
     m_keyframeCounts.clear();
     m_metadataIndex.clear();
-    if (m_filter)
+    if (m_filter && m_metadata && m_filter->animateIn() <= 0 && m_filter->animateOut() <= 0)
     for (int i = 0; i < m_metadata->keyframes()->parameterCount(); i++) {
-        if (!m_metadata->keyframes()->parameter(i)->isSimple() || (m_filter->animateIn() <= 0 && m_filter->animateOut() <= 0)) {
-            if (m_filter->keyframeCount(m_metadata->keyframes()->parameter(i)->property()) > 0) {
-                m_propertyNames << m_metadata->keyframes()->parameter(i)->property();
-                m_keyframeCounts << keyframeCount(m_propertyNames.count() - 1);
-                m_metadataIndex << i;
-            }
+        if (m_filter->keyframeCount(m_metadata->keyframes()->parameter(i)->property()) > 0) {
+            m_propertyNames << m_metadata->keyframes()->parameter(i)->property();
+            m_keyframeCounts << keyframeCount(m_propertyNames.count() - 1);
+            m_metadataIndex << i;
         }
     }
     endResetModel();
@@ -428,78 +687,76 @@ void KeyframesModel::reload()
 
 void KeyframesModel::onFilterChanged(const QString& property)
 {
-//    LOG_DEBUG() << property;
+    bool isKeyframeProperty = false;
+    for (int p = 0; p < m_metadata->keyframes()->parameterCount() && isKeyframeProperty == false; p++) {
+        if (m_metadata->keyframes()->parameter(p)->property() == property) {
+            isKeyframeProperty = true;
+            break;
+        }
+        QString name;
+        foreach (name, m_metadata->keyframes()->parameter(p)->gangedProperties()) {
+            if (name == property) {
+                isKeyframeProperty = true;
+                break;
+            }
+        }
+    }
+    if (!isKeyframeProperty) {
+        // Does not affect this model.
+        return;
+    }
+
     int i = m_propertyNames.indexOf(property);
-    if (i > -1) {
-        int count = m_keyframeCounts[i];
-        m_keyframeCounts[i] = keyframeCount(i);
-        if (count > 0) {
-            beginRemoveRows(index(i), 0, count - 1);
-            endRemoveRows();
-        }
-        if (m_keyframeCounts[i] > 0) {
-//            LOG_DEBUG() << property << m_filter->get(property) << m_keyframeCounts[i];
-            beginInsertRows(index(i), 0, m_keyframeCounts[i] - 1);
-            endInsertRows();
-        } else {
-            reload();
-        }
-    } else {
+    if (i < 0) {
+        // First keyframe added. Reset model to add this parameter.
+        reload();
+        return;
+    }
+
+    int prevCount = m_keyframeCounts[i];
+    m_keyframeCounts[i] = keyframeCount(i);
+    if (m_keyframeCounts[i] == 0) {
+        // All keyframes removed. Reset model to remove this parameter.
         reload();
     }
-}
-
-void KeyframesModel::onFilterInChanged(int delta)
-{
-    for (int parameterIndex = 0; parameterIndex < m_propertyNames.count(); parameterIndex++) {
-        int count = m_keyframeCounts[parameterIndex];
-        if (count > 0) {
-            Mlt::Animation animation = m_filter->getAnimation(m_propertyNames[parameterIndex]);
-            if (animation.is_valid()) {
-                for (int keyframeIndex = 0; keyframeIndex < count;) {
-                    int newFrame = animation.key_get_frame(keyframeIndex) - delta;
-                    if (animation.is_key(newFrame)) {
-                        beginRemoveRows(index(parameterIndex), keyframeIndex, keyframeIndex);
-                        animation.remove(animation.key_get_frame(keyframeIndex));
-                        animation.interpolate();
-                        m_keyframeCounts[parameterIndex] -= 1;
-                        endRemoveRows();
-                        --count;
-                    } else {
-                        animation.key_set_frame(keyframeIndex, newFrame);
-                        ++keyframeIndex;
-                    }
-                }
-                QModelIndex parentIndex = index(parameterIndex);
-                emit dataChanged(index(0, 0, parentIndex), index(count - 1, 0, parentIndex), QVector<int>() << FrameNumberRole);
-            }
+    else if (prevCount != m_keyframeCounts[i]) {
+        // Keyframe count changed. Remove all old and insert all new.
+        if (prevCount > 0) {
+            beginRemoveRows(index(i), 0, prevCount - 1);
+            endRemoveRows();
         }
+        beginInsertRows(index(i), 0, m_keyframeCounts[i] - 1);
+        endInsertRows();
+        emit dataChanged(index(i), index(i), QVector<int>() << LowestValueRole << HighestValueRole);
+    }
+    else {
+        // Keyframe count is unchanged. A value must have changed.
+        emit dataChanged(index(i), index(i), QVector<int>() << LowestValueRole << HighestValueRole);
+        emit dataChanged(index(0, 0, index(i)), index(m_keyframeCounts[i] - 1, 0, index(i)), QVector<int>() << NumericValueRole);
     }
 }
 
-void KeyframesModel::onFilterOutChanged(int delta)
+void KeyframesModel::onFilterInChanged(int /*delta*/)
 {
-    Q_UNUSED(delta)
-    for (int parameterIndex = 0; parameterIndex < m_propertyNames.count(); parameterIndex++) {
-        int count = m_keyframeCounts[parameterIndex];
-        if (count > 0) {
-            Mlt::Animation animation = m_filter->getAnimation(m_propertyNames[parameterIndex]);
-            if (animation.is_valid()) {
-                for (int keyframeIndex = 0; keyframeIndex < count;) {
-                    int frame = animation.key_get_frame(keyframeIndex);
-                    if (frame < 0) {
-                        beginRemoveRows(index(parameterIndex), keyframeIndex, keyframeIndex);
-                        animation.remove(animation.key_get_frame(keyframeIndex));
-                        animation.interpolate();
-                        m_keyframeCounts[parameterIndex] -= 1;
-                        endRemoveRows();
-                        --count;
-                    } else {
-                        ++keyframeIndex;
-                    }
-                }
-            }
-        }
+    QTimer::singleShot(0, this, SLOT(reload()));
+}
+
+
+void KeyframesModel::trimFilterIn(int in)
+{
+    Mlt::Service& service = m_filter->service();
+    if (service.is_valid() && service.type() == mlt_service_filter_type) {
+        Mlt::Filter filter = service;
+        MLT.adjustFilter(&filter, filter.get_in(), filter.get_out(), in - filter.get_in(), 0);
+    }
+}
+
+void KeyframesModel::trimFilterOut(int out)
+{
+    Mlt::Service& service = m_filter->service();
+    if (service.is_valid() && service.type() == mlt_service_filter_type) {
+        Mlt::Filter filter = service;
+        MLT.adjustFilter(&filter, filter.get_in(), filter.get_out(), 0, filter.get_out() - out);
     }
 }
 
