@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020 Meltytech, LLC
+ * Copyright (c) 2020-2022 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 
 static const char* kProxySubfolder = "proxies";
 static const char* kProxyVideoExtension = ".mp4";
+static const char* kGoProProxyVideoExtension = ".LRV";
 static const char* kProxyPendingVideoExtension = ".pending.mp4";
 static const char* kProxyImageExtension = ".jpg";
 static const char* kProxyPendingImageExtension = ".pending.jpg";
@@ -98,10 +99,12 @@ void ProxyManager::generateVideoProxy(Mlt::Producer& producer, bool fullRange, S
     args << "-i" << resource;
     args << "-max_muxing_queue_size" << "9999";
     // transcode all streams except data, subtitles, and attachments
-    if (producer.get_int("video_index") < producer.get_int("audio_index"))
+    auto audioIndex = producer.property_exists(kDefaultAudioIndexProperty)? producer.get_int(kDefaultAudioIndexProperty) : producer.get_int("audio_index");
+    if (producer.get_int("video_index") < audioIndex) {
         args << "-map" << "0:V?" << "-map" << "0:a?";
-    else
+    } else {
         args << "-map" << "0:a?" << "-map" << "0:V?";
+    }
     args << "-map_metadata" << "0" << "-ignore_unknown";
     args << "-vf";
 
@@ -173,7 +176,7 @@ void ProxyManager::generateVideoProxy(Mlt::Producer& producer, bool fullRange, S
             args << "-rc" << "1";
             args << "-qp_i" << "32" << "-qp_p" << "32";
         } else if (hwCodecs.contains("hevc_vaapi")) {
-            args << "-init_hw_device" << "vaapi=vaapi0:,connection_type=x11" << "-filter_hw_device" << "vaapi0";
+            args << "-init_hw_device" << "vaapi=vaapi0:" << "-filter_hw_device" << "vaapi0";
             args << "-codec:v" << "hevc_vaapi";
             args << "-qp" << "37";
         } else if (hwCodecs.contains("h264_nvenc")) {
@@ -181,7 +184,7 @@ void ProxyManager::generateVideoProxy(Mlt::Producer& producer, bool fullRange, S
             args << "-rc" << "constqp";
             args << "-vglobal_quality" << "37";
         } else if (hwCodecs.contains("h264_vaapi")) {
-            args << "-init_hw_device" << "vaapi=vaapi0:,connection_type=x11" << "-filter_hw_device" << "vaapi0";
+            args << "-init_hw_device" << "vaapi=vaapi0:" << "-filter_hw_device" << "vaapi0";
             args << "-codec:v" << "h264_vaapi";
             args << "-qp" << "30";
         } else if (hwCodecs.contains("hevc_videotoolbox")) {
@@ -212,7 +215,7 @@ void ProxyManager::generateVideoProxy(Mlt::Producer& producer, bool fullRange, S
     if (replace) {
         job->setPostJobAction(new ProxyReplacePostJobAction(resource, fileName, hash));
     } else {
-        job->setPostJobAction(new ProxyFinalizePostJobAction(fileName));
+        job->setPostJobAction(new ProxyFinalizePostJobAction(resource, fileName));
     }
     JOBS.add(job);
 }
@@ -236,7 +239,7 @@ void ProxyManager::generateImageProxy(Mlt::Producer& producer, bool replace)
     if (replace) {
         job->setPostJobAction(new ProxyReplacePostJobAction(resource, fileName, hash));
     } else {
-        job->setPostJobAction(new ProxyFinalizePostJobAction(fileName));
+        job->setPostJobAction(new ProxyFinalizePostJobAction(resource, fileName));
     }
     JOBS.add(job);
 }
@@ -272,10 +275,7 @@ static void processProperties(QXmlStreamWriter& newXml, QVector<MltProperty>& pr
             if (p.first == "resource") {
                 // Convert to relative
                 if (!root.isEmpty() && newResource.startsWith(root)) {
-                    if (root.endsWith('/'))
-                        newResource = newResource.mid(root.size());
-                    else
-                        newResource = newResource.mid(root.size() + 1);
+                    newResource = newResource.mid(root.size());
                 }
                 if (service == "timewarp") {
                     newProperties << MltProperty(p.first, QString("%1:%2").arg(speed).arg(newResource));
@@ -302,7 +302,7 @@ static void processProperties(QXmlStreamWriter& newXml, QVector<MltProperty>& pr
     properties.clear();
 }
 
-bool ProxyManager::filterXML(QString& xmlString, const QString& root)
+bool ProxyManager::filterXML(QString& xmlString, QString root)
 {
     QString output;
     QXmlStreamReader xml(xmlString);
@@ -310,6 +310,12 @@ bool ProxyManager::filterXML(QString& xmlString, const QString& root)
     bool isPropertyElement = false;
     QVector<MltProperty> properties;
 
+    // This prevents processProperties() from mis-matching a resource path that begins with root
+    // when it is converting to relative paths.
+    if (!root.isEmpty() && root.endsWith('/')) {
+        root.append('/');
+    }
+
     newXml.setAutoFormatting(true);
     newXml.setAutoFormattingIndent(2);
 
@@ -369,9 +375,7 @@ bool ProxyManager::filterXML(QString& xmlString, const QString& root)
     }
 
     // Useful for debugging
-//        tempFile.open();
-//        LOG_DEBUG() << tempFile.readAll().constData();
-//        tempFile.close();
+//    LOG_DEBUG() << output;
 
     if (!xml.hasError()) {
         xmlString = output;
@@ -387,6 +391,9 @@ bool ProxyManager::fileExists(Mlt::Producer& producer)
     QString service = QString::fromLatin1(producer.get("mlt_service"));
     QString fileName;
     if (service.startsWith("avformat")) {
+        if (QFile::exists(GoProProxyFilePath(producer.get("resource")))) {
+            return true;
+        }
         fileName = Util::getHash(producer) + kProxyVideoExtension;
     } else if (isValidImage(producer)) {
         fileName = Util::getHash(producer) + kProxyImageExtension;
@@ -462,7 +469,15 @@ bool ProxyManager::generateIfNotExists(Mlt::Producer& producer, bool replace)
             QDir projectDir(MLT.projectFolder());
             QString fileName;
             if (service.startsWith("avformat")) {
-                fileName = Util::getHash(producer) + kProxyVideoExtension;
+                auto gopro = GoProProxyFilePath(producer.get("resource"));
+                if (QFile::exists(gopro)) {
+                    producer.set(kIsProxyProperty, 1);
+                    producer.set(kOriginalResourceProperty, producer.get("resource"));
+                    producer.set("resource", gopro.toUtf8().constData());
+                    return true;
+                } else {
+                    fileName = Util::getHash(producer) + kProxyVideoExtension;
+                }
             } else if (isValidImage(producer)) {
                 fileName = Util::getHash(producer) + kProxyImageExtension;
             } else {
@@ -556,6 +571,14 @@ public:
     int on_end_filter(Mlt::Filter*) { return 0; }
     int on_start_transition(Mlt::Transition*) { return 0; }
     int on_end_transition(Mlt::Transition*) { return 0; }
+    int on_start_chain(Mlt::Chain* chain) {
+        if (!chain->parent().get_int(kIsProxyProperty))
+            m_producers << Mlt::Producer(chain);
+        return 0;
+    }
+    int on_end_chain(Mlt::Chain*) { return 0; }
+    int on_start_link(Mlt::Link*) { return 0; }
+    int on_end_link(Mlt::Link*) { return 0; }
 };
 
 void ProxyManager::generateIfNotExistsAll(Mlt::Producer& producer)
@@ -587,3 +610,13 @@ bool ProxyManager::removePending()
     //TODO if any pending remove, let user know and offer to regenerate?
     return foundAny;
 }
+
+QString ProxyManager::GoProProxyFilePath(const QString& resource)
+{
+    auto fi = QFileInfo(resource);
+    auto base = fi.baseName();
+    base = "GL" + base.mid(2);
+    auto result = fi.absoluteDir().filePath(base + kGoProProxyVideoExtension);
+    LOG_DEBUG() << result;
+    return result;
+}
