@@ -35,7 +35,7 @@ class TimelineTransition : public QQuickPaintedItem
 public:
     TimelineTransition()
     {
-        setAntialiasing(QPainter::Antialiasing);
+        setAntialiasing(true);
         connect(this, SIGNAL(propertyChanged()), this, SLOT(update()));
     }
 
@@ -79,7 +79,7 @@ class TimelineTriangle : public QQuickPaintedItem
 public:
     TimelineTriangle()
     {
-        setAntialiasing(QPainter::Antialiasing);
+        setAntialiasing(true);
     }
     void paint(QPainter *painter)
     {
@@ -133,7 +133,7 @@ public:
         for (; i < width(); ++i)
         {
             int idx = inPoint + int(i * indicesPrPixel);
-            if (idx + 1 >= data.length())
+            if ((idx < 0) || (idx + 2 >= data.length()))
                 break;
             qreal level = qMax(data.at(idx).toReal(), data.at(idx + 1).toReal()) / 256;
             path.lineTo(i, height() - level * height());
@@ -159,12 +159,74 @@ private:
     bool m_isActive {true};
 };
 
+class MarkerStart : public QQuickPaintedItem
+{
+    Q_OBJECT
+    Q_PROPERTY(QColor fillColor MEMBER m_color NOTIFY propertyChanged)
+
+public:
+    MarkerStart()
+    {
+        setAntialiasing(true);
+        connect(this, SIGNAL(propertyChanged()), this, SLOT(update()));
+    }
+
+    void paint(QPainter *painter)
+    {
+        QPainterPath path;
+        path.moveTo(0, 0);
+        path.lineTo(0, 10);
+        path.lineTo(7, 17);
+        path.lineTo(7, 0);
+        path.lineTo(0, 0);
+        painter->fillPath(path, m_color);
+    }
+
+signals:
+    void propertyChanged();
+
+private:
+    QColor m_color;
+};
+
+class MarkerEnd : public QQuickPaintedItem
+{
+    Q_OBJECT
+    Q_PROPERTY(QColor fillColor MEMBER m_color NOTIFY propertyChanged)
+
+public:
+    MarkerEnd()
+    {
+        setAntialiasing(true);
+        connect(this, SIGNAL(propertyChanged()), this, SLOT(update()));
+    }
+
+    void paint(QPainter *painter)
+    {
+        QPainterPath path;
+        path.moveTo(0, 17);
+        path.lineTo(7, 10);
+        path.lineTo(7, 0);
+        path.lineTo(0, 0);
+        painter->fillPath(path, m_color);
+    }
+
+signals:
+    void propertyChanged();
+
+private:
+    QColor m_color;
+};
+
+
 void registerTimelineItems()
 {
     qmlRegisterType<TimelineTransition>("Shotcut.Controls", 1, 0, "TimelineTransition");
     qmlRegisterType<TimelinePlayhead>("Shotcut.Controls", 1, 0, "TimelinePlayhead");
     qmlRegisterType<TimelineTriangle>("Shotcut.Controls", 1, 0, "TimelineTriangle");
     qmlRegisterType<TimelineWaveform>("Shotcut.Controls", 1, 0, "TimelineWaveform");
+    qmlRegisterType<MarkerStart>("Shotcut.Controls", 1, 0, "MarkerStart");
+    qmlRegisterType<MarkerEnd>("Shotcut.Controls", 1, 0, "MarkerEnd");
 }
 
 #include "timelineitems.moc"
