@@ -16,6 +16,7 @@
  */
 
 #include "settings.h"
+#include <QColor>
 #include <QLocale>
 #include <QStandardPaths>
 #include <QFile>
@@ -133,6 +134,16 @@ void ShotcutSettings::setTheme(const QString& s)
     settings.setValue("theme", s);
 }
 
+QString ShotcutSettings::jobPriority() const
+{
+    return settings.value("jobPriority", "low").toString();
+}
+
+void ShotcutSettings::setJobPriority(const QString& s)
+{
+    settings.setValue("jobPriority", s);
+}
+
 bool ShotcutSettings::showTitleBars() const
 {
     return settings.value("titleBars", true).toBool();
@@ -225,6 +236,16 @@ void ShotcutSettings::setViewMode(const QString& viewMode)
     emit viewModeChanged();
 }
 
+QString ShotcutSettings::exportFrameSuffix() const
+{
+    return settings.value("exportFrameSuffix", ".png").toString();
+}
+
+void ShotcutSettings::setExportFrameSuffix(const QString& exportFrameSuffix)
+{
+    settings.setValue("exportFrameSuffix", exportFrameSuffix);
+}
+
 QString ShotcutSettings::encodePath() const
 {
     return settings.value("encode/path", QStandardPaths::standardLocations(QStandardPaths::MoviesLocation)).toString();
@@ -278,6 +299,16 @@ void ShotcutSettings::setEncodeAdvanced(bool b)
     settings.setValue("encode/advanced", b);
 }
 
+bool ShotcutSettings::convertAdvanced() const
+{
+    return settings.value("convertAdvanced", false).toBool();
+}
+
+void ShotcutSettings::setConvertAdvanced(bool b)
+{
+    settings.setValue("convertAdvanced", b);
+}
+
 bool ShotcutSettings::showConvertClipDialog() const
 {
     return settings.value("showConvertClipDialog", true).toBool();
@@ -557,6 +588,17 @@ void ShotcutSettings::setTimelineRippleAllTracks(bool b)
     emit timelineRippleAllTracksChanged();
 }
 
+bool ShotcutSettings::timelineRippleMarkers() const
+{
+    return settings.value("timeline/rippleMarkers", false).toBool();
+}
+
+void ShotcutSettings::setTimelineRippleMarkers(bool b)
+{
+    settings.setValue("timeline/rippleMarkers", b);
+    emit timelineRippleMarkersChanged();
+}
+
 bool ShotcutSettings::timelineSnap() const
 {
     return settings.value("timeline/snap", true).toBool();
@@ -686,6 +728,42 @@ void ShotcutSettings::setLoudnessScopeShowMeter(const QString& meter, bool b)
     settings.setValue("scope/loudness/" + meter, b);
 }
 
+void ShotcutSettings::setMarkerColor(const QColor& color)
+{
+    settings.setValue("markers/color", color.name());
+}
+
+QColor ShotcutSettings::markerColor() const
+{
+    return QColor(settings.value("markers/color", "green").toString());
+}
+
+void ShotcutSettings::setMarkersShowColumn(const QString& column, bool b)
+{
+    settings.setValue("markers/columns/" + column, b);
+}
+
+bool ShotcutSettings::markersShowColumn(const QString& column) const
+{
+    return settings.value("markers/columns/" + column, true).toBool();
+}
+
+void ShotcutSettings::setMarkerSort(int column, Qt::SortOrder order)
+{
+    settings.setValue("markers/sortColumn", column);
+    settings.setValue("markers/sortOrder", order);
+}
+
+int ShotcutSettings::getMarkerSortColumn()
+{
+    return settings.value("markers/sortColumn", -1).toInt();
+}
+
+Qt::SortOrder ShotcutSettings::getMarkerSortOrder()
+{
+    return (Qt::SortOrder)settings.value("markers/sortOrder", Qt::AscendingOrder).toInt();
+}
+
 int ShotcutSettings::drawMethod() const
 {
 #ifdef Q_OS_WIN
