@@ -31,6 +31,7 @@ class ShotcutSettings : public QObject
     Q_PROPERTY(bool timelineShowThumbnails READ timelineShowThumbnails WRITE setTimelineShowThumbnails NOTIFY timelineShowThumbnailsChanged)
     Q_PROPERTY(bool timelineRipple READ timelineRipple WRITE setTimelineRipple NOTIFY timelineRippleChanged)
     Q_PROPERTY(bool timelineRippleAllTracks READ timelineRippleAllTracks WRITE setTimelineRippleAllTracks NOTIFY timelineRippleAllTracksChanged)
+    Q_PROPERTY(bool timelineRippleMarkers READ timelineRippleMarkers WRITE setTimelineRippleMarkers NOTIFY timelineRippleMarkersChanged)
     Q_PROPERTY(bool timelineSnap READ timelineSnap WRITE setTimelineSnap NOTIFY timelineSnapChanged)
     Q_PROPERTY(bool timelineCenterPlayhead READ timelineCenterPlayhead WRITE setTimelineCenterPlayhead NOTIFY timelineCenterPlayheadChanged)
     Q_PROPERTY(bool timelineScrollZoom READ timelineScrollZoom WRITE setTimelineScrollZoom NOTIFY timelineScrollZoomChanged)
@@ -67,6 +68,8 @@ public:
     void setRecent(const QStringList&);
     QString theme() const;
     void setTheme(const QString&);
+    QString jobPriority() const;
+    void setJobPriority(const QString&);
     bool showTitleBars() const;
     void setShowTitleBars(bool);
     bool showToolBar() const;
@@ -85,6 +88,10 @@ public:
     void setWindowStateDefault(const QByteArray&);
     QString viewMode() const;
     void setViewMode(const QString& viewMode);
+    QString exportFrameSuffix() const;
+    void setExportFrameSuffix(const QString& suffix);
+    bool convertAdvanced() const;
+    void setConvertAdvanced(bool);
 
     // encode
     QString encodePath() const;
@@ -156,6 +163,8 @@ public:
     void setTimelineRipple(bool);
     bool timelineRippleAllTracks() const;
     void setTimelineRippleAllTracks(bool);
+    bool timelineRippleMarkers() const;
+    void setTimelineRippleMarkers(bool);
     bool timelineSnap() const;
     void setTimelineSnap(bool);
     bool timelineCenterPlayhead() const;
@@ -185,6 +194,15 @@ public:
     bool loudnessScopeShowMeter(const QString& meter) const;
     void setLoudnessScopeShowMeter(const QString& meter, bool b);
 
+    // Markers
+    void setMarkerColor(const QColor& color);
+    QColor markerColor() const;
+    void setMarkersShowColumn(const QString& column, bool b);
+    bool markersShowColumn(const QString& column) const;
+    void setMarkerSort(int column, Qt::SortOrder order);
+    int getMarkerSortColumn();
+    Qt::SortOrder getMarkerSortOrder();
+
     // general continued
     int drawMethod() const;
     void setDrawMethod(int);
@@ -235,6 +253,7 @@ signals:
     void timelineShowThumbnailsChanged();
     void timelineRippleChanged();
     void timelineRippleAllTracksChanged();
+    void timelineRippleMarkersChanged();
     void timelineSnapChanged();
     void timelineCenterPlayheadChanged();
     void timelineScrollZoomChanged();
