@@ -33,6 +33,7 @@
 #include <QCryptographicHash>
 #include <QtGlobal>
 
+#include <MltChain.h>
 #include <MltProducer.h>
 #include <Logger.h>
 #include "shotcut_mlt_properties.h"
@@ -49,12 +50,15 @@ static const unsigned int kLowMemoryThresholdPercent = 10U;
 static const unsigned int kLowMemoryThresholdKB = 256U * 1024U;
 #endif
 
-QString Util::baseName(const QString &filePath)
+QString Util::baseName(const QString &filePath, bool trimQuery)
 {
     QString s = filePath;
     // Only if absolute path and not a URI.
     if (s.startsWith('/') || s.midRef(1, 2) == ":/" || s.midRef(1, 2) == ":\\")
         s = QFileInfo(s).fileName();
+    if (trimQuery) {
+        return removeQueryString(s);
+    }
     return s;
 }
 
@@ -85,7 +89,7 @@ void Util::setColorsToHighlight(QWidget* widget, QPalette::ColorRole role)
 
 void Util::showInFolder(const QString& path)
 {
-    QFileInfo info(path);
+    QFileInfo info(removeQueryString(path));
 #if defined(Q_OS_WIN)
     QStringList args;
     if (!info.isDir())
@@ -143,7 +147,7 @@ QString Util::producerTitle(const Mlt::Producer& producer)
         return QObject::tr("Transition");
     if (p.get(kTrackNameProperty))
         return QObject::tr("Track: %1").arg(QString::fromUtf8(p.get(kTrackNameProperty)));
-    if (tractor_type == p.type())
+    if (mlt_service_tractor_type == p.type())
         return QObject::tr("Output");
     if (p.get(kShotcutCaptionProperty))
         return QString::fromUtf8(p.get(kShotcutCaptionProperty));
@@ -160,13 +164,13 @@ QString Util::removeFileScheme(QUrl& url)
 
 static inline bool isValidGoProFirstFilePrefix(const QFileInfo& info)
 {
-    QStringList list {"GOPR", "GH01", "GS01", "GX01"};
+    QStringList list {"GOPR", "GH01", "GL01", "GM01", "GS01", "GX01"};
     return list.contains(info.baseName().left(4).toUpper());
 }
 
 static inline bool isValidGoProPrefix(const QFileInfo& info)
 {
-    QStringList list {"GP", "GH", "GS", "GX"};
+    QStringList list {"GP", "GH", "GL", "GM", "GS", "GX"};
     return list.contains(info.baseName().left(2).toUpper());
 }
 
@@ -343,6 +347,7 @@ void Util::applyCustomProperties(Mlt::Producer& destination, Mlt::Producer& sour
     p.clear("speed");
     p.clear("warp_speed");
     p.clear("warp_pitch");
+    p.clear("rotate");
     p.clear(kAspectRatioNumerator);
     p.clear(kAspectRatioDenominator);
     p.clear(kCommentProperty);
@@ -353,7 +358,7 @@ void Util::applyCustomProperties(Mlt::Producer& destination, Mlt::Producer& sour
     if (!p.get_int(kIsProxyProperty))
         p.clear(kOriginalResourceProperty);
     destination.pass_list(source, "mlt_service, audio_index, video_index, force_progressive, force_tff,"
-                       "force_aspect_ratio, video_delay, color_range, warp_speed, warp_pitch,"
+                       "force_aspect_ratio, video_delay, color_range, warp_speed, warp_pitch, rotate,"
                        kAspectRatioNumerator ","
                        kAspectRatioDenominator ","
                        kCommentProperty ","
@@ -370,7 +375,7 @@ void Util::applyCustomProperties(Mlt::Producer& destination, Mlt::Producer& sour
     QString resource = ProxyManager::resource(destination);
     if (!qstrcmp("timewarp", source.get("mlt_service"))) {
         auto speed = qAbs(source.get_double("warp_speed"));
-        auto caption = QString("%1 (%2x)").arg(Util::baseName(resource)).arg(speed);
+        auto caption = QString("%1 (%2x)").arg(Util::baseName(resource, true)).arg(speed);
         destination.set(kShotcutCaptionProperty, caption.toUtf8().constData());
 
         resource = destination.get("_shotcut:resource");
@@ -381,7 +386,7 @@ void Util::applyCustomProperties(Mlt::Producer& destination, Mlt::Producer& sour
         int length = qRound(destination.get_length() * speedRatio);
         destination.set("length", destination.frames_to_time(length, mlt_time_clock));
     } else {
-        auto caption = Util::baseName(resource);
+        auto caption = Util::baseName(resource, true);
         destination.set(kShotcutCaptionProperty, caption.toUtf8().constData());
 
         p.clear("warp_resource");
@@ -394,7 +399,7 @@ void Util::applyCustomProperties(Mlt::Producer& destination, Mlt::Producer& sour
 QString Util::getFileHash(const QString& path)
 {
     // This routine is intentionally copied from Kdenlive.
-    QFile file(path);
+    QFile file(removeQueryString(path));
     if (file.open(QIODevice::ReadOnly)) {
         QByteArray fileData;
          // 1 MB = 1 second per 450 files (or faster)
@@ -479,6 +484,19 @@ bool Util::isMemoryLow()
             }
         }
     }
+    return false;
+#elif defined(__FreeBSD__)
+    QProcess p;
+    p.start("sysctl -n hw.usermem");
+    p.waitForFinished();
+    auto lines = p.readAllStandardOutput();
+    p.close();
+    bool ok = false;
+    auto availableKB = lines.toUInt(&ok);
+    if (ok) {
+	    return availableKB < kLowMemoryThresholdKB;
+    }
+
     return false;
 #elif defined(Q_OS_LINUX)
     unsigned int availableKB = UINT_MAX;
@@ -503,3 +521,54 @@ bool Util::isMemoryLow()
     return availableKB < kLowMemoryThresholdKB;
 #endif
 }
+
+QString Util::removeQueryString(const QString& s)
+{
+    auto i = s.lastIndexOf("\\?");
+    if (i < 0) {
+        i = s.lastIndexOf("%5C?");
+    }
+    if (i > 0 ) {
+        return s.left(i);
+    }
+    return s;
+}
+
+int Util::greatestCommonDivisor(int m, int n)
+{
+    int gcd, remainder;
+    while (n) {
+        remainder = m % n;
+        m = n;
+        n = remainder;
+    }
+    gcd = m;
+    return gcd;
+}
+
+void Util::normalizeFrameRate(double fps, int& numerator, int& denominator)
+{
+    // Convert some common non-integer frame rates to fractions.
+    if (qRound(fps * 1000000.0) == 23976024) {
+        numerator = 24000;
+        denominator = 1001;
+    } else if (qRound(fps * 100000.0) == 2997003) {
+        numerator = 30000;
+        denominator = 1001;
+    } else if (qRound(fps * 1000000.0) == 47952048) {
+        numerator = 48000;
+         denominator = 1001;
+    } else if (qRound(fps * 100000.0) == 5994006) {
+        numerator = 60000;
+        denominator = 1001;
+    } else {
+        // Workaround storing QDoubleSpinBox::value() loses precision.
+        numerator = qRound(fps * 1000000.0);
+        denominator = 1000000;
+    }
+}
+
+QString Util::textColor(const QColor& color)
+{
+    return (color.value() < 150)? "white" : "black";
+}
