@@ -36,6 +36,8 @@
 
 #include <QtWidgets>
 
+static const auto kHandleSeconds = 15.0;
+
 static bool ProducerIsTimewarp( Mlt::Producer* producer )
 {
     return QString::fromUtf8(producer->get("mlt_service")) == "timewarp";
@@ -47,7 +49,13 @@ static QString GetFilenameFromProducer(Mlt::Producer* producer, bool useOriginal
     if (useOriginal && producer->get(kOriginalResourceProperty)) {
         resource = QString::fromUtf8(producer->get(kOriginalResourceProperty));
     } else if (ProducerIsTimewarp(producer)) {
-        resource = QString::fromUtf8(producer->get("warp_resource"));
+        resource = QString::fromUtf8(producer->get("resource"));
+        auto i = resource.indexOf(':');
+        if (producer->get_int(kIsProxyProperty) && i > 0) {
+            resource = resource.mid(i + 1);
+        } else {
+            resource = QString::fromUtf8(producer->get("warp_resource"));
+        }
     } else {
         resource = QString::fromUtf8(producer->get("resource"));
     }
@@ -68,6 +76,23 @@ static double GetSpeedFromProducer( Mlt::Producer* producer )
     return speed;
 }
 
+static QString updateCaption(Mlt::Producer* producer)
+{
+    double warpSpeed = GetSpeedFromProducer(producer);
+    QString resource = GetFilenameFromProducer(producer);
+    QString name = Util::baseName(resource, true);
+    QString caption = producer->get(kShotcutCaptionProperty);
+    if (caption.isEmpty() || caption.startsWith(name)) {
+        // compute the caption
+        if (warpSpeed != 1.0)
+            caption = QString("%1 (%2x)").arg(name).arg(warpSpeed);
+        else
+            caption = name;
+        producer->set(kShotcutCaptionProperty, caption.toUtf8().constData());
+    }
+    return caption;
+}
+
 DecodeTask::DecodeTask(AvformatProducerWidget* widget)
     : QObject(0)
     , QRunnable()
@@ -90,16 +115,16 @@ AvformatProducerWidget::AvformatProducerWidget(QWidget *parent)
     , ui(new Ui::AvformatProducerWidget)
     , m_defaultDuration(-1)
     , m_recalcDuration(true)
-    , m_userDefinedCaption(false)
 {
     ui->setupUi(this);
     ui->timelineDurationText->setFixedWidth(ui->durationSpinBox->width());
     ui->filenameLabel->setFrame(true);
     Util::setColorsToHighlight(ui->filenameLabel, QPalette::Base);
+    connect(ui->applySpeedButton, SIGNAL(clicked()), SLOT(on_speedSpinBox_editingFinished()));
     if (Settings.playerGPU())
         connect(MLT.videoWidget(), SIGNAL(frameDisplayed(const SharedFrame&)), this, SLOT(onFrameDisplayed(const SharedFrame&)));
     else
-        connect(this, SIGNAL(producerChanged(Mlt::Producer*)), SLOT(onProducerChanged()));
+        connect(this, SIGNAL(producerChanged(Mlt::Producer*)), SLOT(onProducerChanged(Mlt::Producer*)));
 }
 
 AvformatProducerWidget::~AvformatProducerWidget()
@@ -112,7 +137,7 @@ Mlt::Producer* AvformatProducerWidget::newProducer(Mlt::Profile& profile)
     Mlt::Producer* p = 0;
     if ( ui->speedSpinBox->value() == 1.0 )
     {
-        p = new Mlt::Producer(profile, GetFilenameFromProducer(producer(), false).toUtf8().constData());
+        p = new Mlt::Chain(profile, GetFilenameFromProducer(producer(), false).toUtf8().constData());
     }
     else
     {
@@ -164,6 +189,27 @@ void AvformatProducerWidget::rename()
     ui->filenameLabel->selectAll();
 }
 
+void AvformatProducerWidget::offerConvert(QString message, bool set709Convert, bool setSubClip)
+{
+    m_producer->set(kShotcutSkipConvertProperty, true);
+    LongUiTask::cancel();
+    MLT.pause();
+    TranscodeDialog dialog(message.append(tr(" Do you want to convert it to an edit-friendly format?\n\n"
+                              "If yes, choose a format below and then click OK to choose a file name. "
+                              "After choosing a file name, a job is created. "
+                              "When it is done, it automatically replaces clips, or you can double-click the job to open it.\n")),
+                           ui->scanComboBox->currentIndex(), this);
+    dialog.setWindowModality(QmlApplication::dialogModality());
+    if (!setSubClip) {
+        dialog.showCheckBox();
+    }
+    dialog.set709Convert(set709Convert);
+    dialog.showSubClipCheckBox();
+    LOG_DEBUG() << "in" << m_producer->get_in() << "out" << m_producer->get_out() << "length" << m_producer->get_length() - 1;
+    dialog.setSubClipChecked(setSubClip && (m_producer->get_in() > 0 || m_producer->get_out() < m_producer->get_length() - 1));
+    convert(dialog);
+}
+
 void AvformatProducerWidget::keyPressEvent(QKeyEvent* event)
 {
     if (ui->speedSpinBox->hasFocus() &&
@@ -186,9 +232,11 @@ void AvformatProducerWidget::onFrameDisplayed(const SharedFrame&)
         disconnect(MLT.videoWidget(), SIGNAL(frameDisplayed(const SharedFrame&)), this, 0);
 }
 
-void AvformatProducerWidget::onProducerChanged()
+void AvformatProducerWidget::onProducerChanged(Mlt::Producer* producer)
 {
-    QThreadPool::globalInstance()->start(new DecodeTask(this), 10);
+    if( producer->get_producer() == m_producer->get_producer() ) {
+        QThreadPool::globalInstance()->start(new DecodeTask(this), 10);
+    }
 }
 
 void AvformatProducerWidget::reopen(Mlt::Producer* p)
@@ -242,7 +290,7 @@ void AvformatProducerWidget::reopen(Mlt::Producer* p)
         return;
     }
     MLT.stop();
-    emit producerReopened();
+    emit producerReopened(false);
     emit producerChanged(p);
     MLT.seek(position);
     MLT.play(speed);
@@ -253,7 +301,7 @@ void AvformatProducerWidget::recreateProducer()
 {
     Mlt::Producer* p = newProducer(MLT.profile());
     p->pass_list(*m_producer, "audio_index, video_index, force_aspect_ratio,"
-                 "video_delay, force_progressive, force_tff, set.force_full_luma, color_range, warp_pitch,"
+                 "video_delay, force_progressive, force_tff, set.force_full_luma, color_range, warp_pitch, rotate,"
                  kAspectRatioNumerator ","
                  kAspectRatioDenominator ","
                  kShotcutHashProperty ","
@@ -265,8 +313,20 @@ void AvformatProducerWidget::recreateProducer()
                  kOriginalResourceProperty ","
                  kDisableProxyProperty ","
                  kIsProxyProperty);
+    updateCaption(p);
     Mlt::Controller::copyFilters(*m_producer, *p);
     if (m_producer->get(kMultitrackItemProperty)) {
+        int length = ui->durationSpinBox->value();
+        int in = m_producer->get_in();
+        int out = m_producer->get_out();
+        double oldSpeed = GetSpeedFromProducer(producer());
+        double newSpeed = ui->speedSpinBox->value();
+        double speedRatio = oldSpeed / newSpeed;
+        length = qRound(length * speedRatio);
+        in = qMin(qRound(in * speedRatio), length - 1);
+        out = qMin(qRound(out * speedRatio), length - 1);
+        p->set("length", p->frames_to_time(length, mlt_time_clock));
+        p->set_in_and_out(in, out);
         emit producerChanged(p);
         delete p;
     } else {
@@ -285,24 +345,9 @@ void AvformatProducerWidget::onFrameDecoded()
 
     double warpSpeed = GetSpeedFromProducer(producer());
     QString resource = GetFilenameFromProducer(producer());
-    QString name = Util::baseName(resource);
-    QString caption = m_producer->get(kShotcutCaptionProperty);
-    if (caption.isEmpty() || caption.startsWith(name)) {
-        // compute the caption
-        if (warpSpeed != 1.0)
-            caption = QString("%1 (%2x)").arg(name).arg(warpSpeed);
-        else
-            caption = name;
-        m_producer->set(kShotcutCaptionProperty, caption.toUtf8().constData());
-        ui->filenameLabel->setText(ui->filenameLabel->fontMetrics().elidedText(caption, Qt::ElideLeft, width() - 30));
-        m_userDefinedCaption = false;
-    } else {
-        ui->filenameLabel->setText(ui->filenameLabel->fontMetrics().elidedText(caption, Qt::ElideLeft, width() - 30));
-        auto computedCaption = name;
-        if (warpSpeed != 1.0)
-            computedCaption = QString("%1 (%2x)").arg(name).arg(warpSpeed);
-        m_userDefinedCaption = caption != computedCaption;
-    }
+    QString caption = updateCaption(m_producer.data());
+    ui->filenameLabel->setText(caption);
+    ui->filenameLabel->setCursorPosition(caption.length());
     ui->filenameLabel->setToolTip(resource);
     ui->notesTextEdit->setPlainText(QString::fromUtf8(m_producer->get(kCommentProperty)));
     ui->durationSpinBox->setValue(m_producer->get_length());
@@ -362,6 +407,11 @@ void AvformatProducerWidget::onFrameDecoded()
                     color_range = 1;
                     ui->rangeComboBox->setEnabled(false);
                 }
+                key = QString("meta.media.%1.codec.rotate").arg(i);
+                int rotation = m_producer->property_exists("rotate") ?
+                    m_producer->get_int("rotate") :
+                    m_producer->get_int(key.toLatin1().constData());
+                ui->rotationComboBox->setCurrentIndex(rotation / 90);
                 ui->videoTableWidget->setItem(3, 1, new QTableWidgetItem(pix_fmt));
                 key = QString("meta.media.%1.codec.colorspace").arg(i);
                 int colorspace = m_producer->get_int(key.toLatin1().constData());
@@ -390,6 +440,7 @@ void AvformatProducerWidget::onFrameDecoded()
                     case 1: trcString = "ITU-R BT.709"; break;
                     case 6: trcString = "ITU-R BT.601"; break;
                     case 7: trcString = "SMPTE ST240"; break;
+                    case 11: trcString = "IEC 61966-2-4"; break;
                     case 14: trcString = "ITU-R BT.2020"; break;
                     case 15: trcString = "ITU-R BT.2020"; break;
                     case 16: trcString = "SMPTE ST2084 (PQ)"; break;
@@ -426,7 +477,7 @@ void AvformatProducerWidget::onFrameDecoded()
                 key = QString("meta.media.%1.codec.long_name").arg(i);
                 QString codec(m_producer->get(key.toLatin1().constData()));
                 ui->audioTableWidget->setItem(0, 1, new QTableWidgetItem(codec));
-                const char* layout = mlt_channel_layout_name(mlt_channel_layout_default(channels));
+                const char* layout = mlt_audio_channel_layout_name(mlt_audio_channel_layout_default(channels));
                 QString channelsStr = QString("%1 (%2)").arg(channels).arg(layout);
                 ui->audioTableWidget->setItem(1, 1, new QTableWidgetItem(channelsStr));
                 ui->audioTableWidget->setItem(2, 1, new QTableWidgetItem(sampleRate));
@@ -555,54 +606,21 @@ void AvformatProducerWidget::onFrameDecoded()
             }
         }
     }
-    ui->syncSlider->setValue(qRound(m_producer->get_double("video_delay") * 1000.0));
+    setSyncVisibility();
 
     if (Settings.showConvertClipDialog() && !m_producer->get_int(kShotcutSkipConvertProperty)) {
-        if (ui->videoTableWidget->item(5, 1)->data(Qt::UserRole).toInt() > 7) {
-            // Transfer characteristics > SMPTE240M Probably need conversion
+        auto transferItem = ui->videoTableWidget->item(5, 1);
+        if (transferItem && transferItem->data(Qt::UserRole).toInt() > 7 && transferItem->data(Qt::UserRole).toInt() != 11 ) {
+            // Transfer characteristics > SMPTE240M Probably need conversion except IEC61966-2-4 is OK
             QString trcString = ui->videoTableWidget->item(5, 1)->text();
-            m_producer->set(kShotcutSkipConvertProperty, true);
-            LongUiTask::cancel();
-            MLT.pause();
-            LOG_INFO() << resource << "Probable HDR" << ui->videoTableWidget->item(5, 1)->text();
-            TranscodeDialog dialog(tr("This file uses color transfer characteristics %1, which may result in incorrect colors or brightness in Shotcut. "
-                                      "Do you want to convert it to an edit-friendly format?\n\n"
-                                      "If yes, choose a format below and then click OK to choose a file name. "
-                                      "After choosing a file name, a job is created. "
-                                      "When it is done, double-click the job to open it.\n").arg(trcString),
-                                      ui->scanComboBox->currentIndex(), this);
-            dialog.set709Convert(true);
-            dialog.setWindowModality(QmlApplication::dialogModality());
-            dialog.showCheckBox();
-            convert(dialog);
+            LOG_INFO() << resource << "Probable HDR" << trcString;
+            offerConvert(tr("This file uses color transfer characteristics %1, which may result in incorrect colors or brightness in Shotcut.").arg(trcString), true);
         } else if (isVariableFrameRate) {
-            m_producer->set(kShotcutSkipConvertProperty, true);
-            LongUiTask::cancel();
-            MLT.pause();
             LOG_INFO() << resource << "is variable frame rate";
-            TranscodeDialog dialog(tr("This file is variable frame rate, which is not reliable for editing. "
-                                      "Do you want to convert it to an edit-friendly format?\n\n"
-                                      "If yes, choose a format below and then click OK to choose a file name. "
-                                      "After choosing a file name, a job is created. "
-                                      "When it is done, double-click the job to open it.\n"),
-                                   ui->scanComboBox->currentIndex(), this);
-            dialog.setWindowModality(QmlApplication::dialogModality());
-            dialog.showCheckBox();
-            convert(dialog);
+            offerConvert(tr("This file is variable frame rate, which is not reliable for editing."));
         } else if (QFile::exists(resource) && !MLT.isSeekable(m_producer.data())) {
-            m_producer->set(kShotcutSkipConvertProperty, true);
-            LongUiTask::cancel();
-            MLT.pause();
             LOG_INFO() << resource << "is not seekable";
-            TranscodeDialog dialog(tr("This file does not support seeking and cannot be used for editing. "
-                                      "Do you want to convert it to an edit-friendly format?\n\n"
-                                      "If yes, choose a format below and then click OK to choose a file name. "
-                                      "After choosing a file name, a job is created. "
-                                      "When it is done, double-click the job to open it.\n"),
-                                   ui->scanComboBox->currentIndex(), this);
-            dialog.setWindowModality(QmlApplication::dialogModality());
-            dialog.showCheckBox();
-            convert(dialog);
+            offerConvert(tr("This file does not support seeking and cannot be used for editing."));
         }
     }
 }
@@ -744,6 +762,7 @@ void AvformatProducerWidget::on_menuButton_clicked()
     if (GetFilenameFromProducer(producer()).toLower().endsWith(".mp4")) {
         menu.addAction(ui->actionSetEquirectangular);
     }
+    menu.addAction(ui->actionFFmpegVideoQuality);
     menu.exec(ui->menuButton->mapToGlobal(QPoint(0, 0)));
 }
 
@@ -794,9 +813,27 @@ void AvformatProducerWidget::on_actionFFmpegConvert_triggered()
                            ui->scanComboBox->currentIndex(), this);
     dialog.setWindowModality(QmlApplication::dialogModality());
     dialog.set709Convert(ui->videoTableWidget->item(5, 1)->data(Qt::UserRole).toInt() > 7);
+    dialog.showSubClipCheckBox();
     convert(dialog);
 }
 
+static QString getNextFile(const QString& filePath)
+{
+    QFileInfo info(filePath);
+    QString basename = info.completeBaseName();
+    QString extension = info.suffix();
+    if (extension.isEmpty()) {
+        extension = basename;
+        basename = QString();
+    }
+    for (unsigned i = 1; i < std::numeric_limits<unsigned>::max(); i++) {
+        QString filename = QString::fromLatin1("%1%2.%3").arg(basename).arg(i).arg(extension);
+        if (!info.dir().exists(filename))
+            return info.dir().filePath(filename);
+    }
+    return filePath;
+}
+
 void AvformatProducerWidget::convert(TranscodeDialog& dialog)
 {
     int result = dialog.exec();
@@ -808,15 +845,44 @@ void AvformatProducerWidget::convert(TranscodeDialog& dialog)
         QString path = Settings.savePath();
         QStringList args;
         QString nameFilter;
+        int in = -1;
 
         args << "-loglevel" << "verbose";
         args << "-i" << resource;
         args << "-max_muxing_queue_size" << "9999";
+
+        if (dialog.isSubClip()) {
+            if (Settings.proxyEnabled()) {
+                m_producer->Mlt::Properties::clear(kOriginalResourceProperty);
+            }
+
+            // set trim options
+            if (m_producer->get(kFilterInProperty)) {
+                in = m_producer->get_int(kFilterInProperty);
+                int ss = qMax(0, in - qRound(m_producer->get_fps() * kHandleSeconds));
+                auto s = QString::fromLatin1(m_producer->frames_to_time(ss, mlt_time_clock));
+                args << "-ss" << s.replace(',', '.');
+                in -= ss;
+            } else {
+                args << "-ss" << QString::fromLatin1(m_producer->get_time("in", mlt_time_clock)).replace(',', '.').replace(',', '.');
+            }
+            if (m_producer->get(kFilterOutProperty)) {
+                int out = m_producer->get_int(kFilterOutProperty);
+                int to = qMin(m_producer->get_playtime() - 1, out + qRound(m_producer->get_fps() * kHandleSeconds));
+                auto s = QString::fromLatin1(m_producer->frames_to_time(to, mlt_time_clock));
+                args << "-to" << s.replace(',', '.');
+            } else {
+                args << "-to" << QString::fromLatin1(m_producer->get_time("out", mlt_time_clock)).replace(',', '.');
+            }
+        }
+
         // transcode all streams except data, subtitles, and attachments
-        if (m_producer->get_int("video_index") < m_producer->get_int("audio_index"))
+        auto audioIndex = m_producer->property_exists(kDefaultAudioIndexProperty)? m_producer->get_int(kDefaultAudioIndexProperty) : m_producer->get_int("audio_index");
+        if (m_producer->get_int("video_index") < audioIndex) {
             args << "-map" << "0:V?" << "-map" << "0:a?";
-        else
+        } else {
             args << "-map" << "0:a?" << "-map" << "0:V?";
+        }
         args << "-map_metadata" << "0" << "-ignore_unknown";
 
         // Set video filters
@@ -837,7 +903,13 @@ void AvformatProducerWidget::convert(TranscodeDialog& dialog)
         }
         filterString = filterString + QString("scale=flags=accurate_rnd+full_chroma_inp+full_chroma_int:in_range=%1:out_range=%2").arg(range).arg(range);
         if (dialog.fpsOverride()) {
-            QString minterpFilter = QString(",minterpolate='mi_mode=%1:mc_mode=aobmc:me_mode=bidir:vsbmc=1:fps=%2'").arg(dialog.frc()).arg(dialog.fps());
+            auto fps = QString("%1").arg(dialog.fps(), 0, 'f', 6);
+            int numerator, denominator;
+            Util::normalizeFrameRate(dialog.fps(), numerator, denominator);
+            if (denominator == 1001) {
+                fps = QString("%1/%2").arg(numerator).arg(denominator);
+            }
+            QString minterpFilter = QString(",minterpolate='mi_mode=%1:mc_mode=aobmc:me_mode=bidir:vsbmc=1:fps=%2'").arg(dialog.frc()).arg(fps);
             filterString = filterString + minterpFilter;
         }
         args << filterString;
@@ -879,7 +951,11 @@ void AvformatProducerWidget::convert(TranscodeDialog& dialog)
             args << "-colorspace" << "bt709" << "-color_primaries" << "bt709" << "-color_trc" << "bt709";
         }
         QFileInfo fi(resource);
-        path = path.arg(fi.completeBaseName()).arg(tr("Converted"));
+        QString suffix = dialog.isSubClip()? tr("Sub-clip") + ' ' : tr("Converted");
+        path = path.arg(fi.completeBaseName(), suffix);
+        if (dialog.isSubClip()) {
+            path = getNextFile(path);
+        }
         QString filename = QFileDialog::getSaveFileName(this, dialog.windowTitle(), path, nameFilter,
             nullptr, Util::getFileDialogOptions());
         if (!filename.isEmpty()) {
@@ -900,7 +976,26 @@ void AvformatProducerWidget::convert(TranscodeDialog& dialog)
 
             FfmpegJob* job = new FfmpegJob(filename, args, false);
             job->setLabel(tr("Convert %1").arg(Util::baseName(filename)));
-            job->setPostJobAction(new ConvertReplacePostJobAction(resource, filename, Util::getHash(*m_producer)));
+            if (dialog.isSubClip()) {
+                if (m_producer->get(kMultitrackItemProperty)) {
+                    QString s = QString::fromLatin1(m_producer->get(kMultitrackItemProperty));
+                    QVector<QStringRef> parts = s.splitRef(':');
+                    if (parts.length() == 2) {
+                        int clipIndex = parts[0].toInt();
+                        int trackIndex = parts[1].toInt();
+                        QUuid uuid = MAIN.timelineClipUuid(trackIndex, clipIndex);
+                        if (!uuid.isNull()) {
+                            job->setPostJobAction(new ReplaceOnePostJobAction(resource, filename, QString(), uuid.toByteArray(), in));
+                            JOBS.add(job);
+                        }
+                    }
+                } else {
+                    job->setPostJobAction(new OpenPostJobAction(resource, filename, QString()));
+                    JOBS.add(job);
+                }
+                return;
+            }
+            job->setPostJobAction(new ReplaceAllPostJobAction(resource, filename, Util::getHash(*m_producer)));
             JOBS.add(job);
         }
     }
@@ -919,15 +1014,14 @@ bool AvformatProducerWidget::revertToOriginalResource()
                 int trackIndex = parts[1].toInt();
                 QUuid uuid = MAIN.timelineClipUuid(trackIndex, clipIndex);
                 if (!uuid.isNull()) {
-                    Mlt::Producer producer(MLT.profile(), resource.toUtf8().constData());
-                    if (producer.is_valid()) {
-                        if (!qstrcmp(producer.get("mlt_service"), "avformat")) {
-                            producer.set("mlt_service", "avformat-novalidate");
-                            producer.set("mute_on_pause", 0);
-                        }
-                        MLT.lockCreationTime(&producer);
-                        producer.set_in_and_out(m_producer->get_int(kOriginalInProperty), m_producer->get_int(kOriginalOutProperty));
-                        MAIN.replaceInTimeline(uuid, producer);
+                    Mlt::Producer newProducer(MLT.profile(), resource.toUtf8().constData());
+                    if (newProducer.is_valid()) {
+                        Mlt::Producer* producer = MLT.setupNewProducer(&newProducer);
+                        producer->set(kIsProxyProperty, 1);
+                        producer->set(kOriginalResourceProperty, resource.toUtf8().constData());
+                        producer->set_in_and_out(m_producer->get_int(kOriginalInProperty), m_producer->get_int(kOriginalOutProperty));
+                        MAIN.replaceInTimeline(uuid, *producer);
+                        delete producer;
                         return true;
                     }
                 }
@@ -940,6 +1034,15 @@ bool AvformatProducerWidget::revertToOriginalResource()
     return false;
 }
 
+void AvformatProducerWidget::setSyncVisibility()
+{
+    bool visible = ui->tabWidget->isTabEnabled(0) && ui->tabWidget->isTabEnabled(1) &&
+            m_producer->get_int("video_index") != -1;
+    ui->syncSlider->setVisible(visible);
+    ui->syncLabel->setVisible(visible);
+    ui->syncSpinBox->setVisible(visible);
+}
+
 void AvformatProducerWidget::on_reverseButton_clicked()
 {
     if (revertToOriginalResource())
@@ -978,10 +1081,11 @@ void AvformatProducerWidget::on_reverseButton_clicked()
         ffmpegArgs << "-loglevel" << "verbose";
         ffmpegArgs << "-i" << resource;
         ffmpegArgs << "-max_muxing_queue_size" << "9999";
+
         // set trim options
         if (m_producer->get(kFilterInProperty)) {
             in = m_producer->get_int(kFilterInProperty);
-            int ss = qMax(0, in - qRound(m_producer->get_fps() * 15.0));
+            int ss = qMax(0, in - qRound(m_producer->get_fps() * kHandleSeconds));
             auto s = QString::fromLatin1(m_producer->frames_to_time(ss, mlt_time_clock));
             ffmpegArgs << "-ss" << s.replace(',', '.');
         } else {
@@ -989,13 +1093,14 @@ void AvformatProducerWidget::on_reverseButton_clicked()
         }
         if (m_producer->get(kFilterOutProperty)) {
             int out = m_producer->get_int(kFilterOutProperty);
-            int to = qMin(m_producer->get_playtime() - 1, out + qRound(m_producer->get_fps() * 15.0));
+            int to = qMin(m_producer->get_playtime() - 1, out + qRound(m_producer->get_fps() * kHandleSeconds));
             in = to - out - 1;
             auto s = QString::fromLatin1(m_producer->frames_to_time(to, mlt_time_clock));
             ffmpegArgs << "-to" << s.replace(',', '.');
         } else {
             ffmpegArgs << "-to" << QString::fromLatin1(m_producer->get_time("out", mlt_time_clock)).replace(',', '.');
         }
+
         // transcode all streams except data, subtitles, and attachments
         ffmpegArgs << "-map" << "0:V?" << "-map" << "0:a?" << "-map_metadata" << "0" << "-ignore_unknown";
         if (ui->rangeComboBox->currentIndex())
@@ -1017,11 +1122,12 @@ void AvformatProducerWidget::on_reverseButton_clicked()
         if (m_producer->get_int("video_index") == -1)
             meltArgs << "vn=1" << "video_off=1";
 
+        ffmpegArgs << "-f" << "mov" << "-codec:a" << "alac";
+
         switch (dialog.format()) {
         case 0:
             path.append("/%1 - %2.mp4");
             nameFilter = tr("MP4 (*.mp4);;All Files (*)");
-            ffmpegArgs << "-f" << "mov" << "-codec:a" << "alac";
             if (ui->scanComboBox->currentIndex()) { // progressive
                 ffmpegArgs << "-codec:v" << "dnxhd" << "-profile:v" << "dnxhr_hq" << "-pix_fmt" << "yuv422p";
             } else { // interlaced
@@ -1032,7 +1138,6 @@ void AvformatProducerWidget::on_reverseButton_clicked()
             meltArgs << "vpreset=medium" << "g=1" << "crf=11";
             break;
         case 1:
-            ffmpegArgs << "-f" << "mov" << "-codec:a" << "alac";
             meltArgs << "acodec=alac";
             if (ui->scanComboBox->currentIndex()) { // progressive
                 ffmpegArgs << "-codec:v" << "dnxhd" << "-profile:v" << "dnxhr_hq" << "-pix_fmt" << "yuv422p";
@@ -1046,9 +1151,7 @@ void AvformatProducerWidget::on_reverseButton_clicked()
             nameFilter = tr("MOV (*.mov);;All Files (*)");
             break;
         case 2:
-            ffmpegSuffix = "mkv";
-            ffmpegArgs << "-f" << "matroska" << "-codec:a" << "pcm_s32le" << "-codec:v" << "utvideo";
-            ffmpegArgs << "-pix_fmt" << "yuv422p";
+            ffmpegArgs << "-codec:v" << "utvideo" << "-pix_fmt" << "yuv422p";
             if (!ui->scanComboBox->currentIndex()) { // interlaced
                 meltArgs << "field_order=" + QString::fromLatin1(ui->fieldOrderComboBox->currentIndex()? "tt" : "bb");
             }
@@ -1108,13 +1211,13 @@ void AvformatProducerWidget::on_reverseButton_clicked()
                     int trackIndex = parts[1].toInt();
                     QUuid uuid = MAIN.timelineClipUuid(trackIndex, clipIndex);
                     if (!uuid.isNull()) {
-                        meltJob->setPostJobAction(new ReverseReplacePostJobAction(resource, filename, tmpFileName, uuid.toByteArray(), in));
+                        meltJob->setPostJobAction(new ReplaceOnePostJobAction(resource, filename, tmpFileName, uuid.toByteArray(), in));
                         JOBS.add(meltJob);
                         return;
                     }
                 }
             }
-            meltJob->setPostJobAction(new ReverseOpenPostJobAction(resource, filename, tmpFileName));
+            meltJob->setPostJobAction(new OpenPostJobAction(resource, filename, tmpFileName));
             JOBS.add(meltJob);
         }
     }
@@ -1164,7 +1267,8 @@ void AvformatProducerWidget::on_actionExtractSubclip_triggered()
         else
             ffmpegArgs << "-to" << QString::fromLatin1(m_producer->get_time("out", mlt_time_clock)).replace(',', '.');
         ffmpegArgs << "-avoid_negative_ts" << "make_zero"
-                   << "-map" << "0" << "-map_metadata" << "0"
+                   << "-map" << "0:V?" << "-map" << "0:a?" << "-map" << "0:s?"
+                   << "-map_metadata" << "0"
                    << "-codec" << "copy" << "-y" << filename;
 
         // Run the ffmpeg job.
@@ -1198,7 +1302,7 @@ void AvformatProducerWidget::on_filenameLabel_editingFinished()
         if (caption.isEmpty()) {
             double warpSpeed = GetSpeedFromProducer(producer());
             QString resource = GetFilenameFromProducer(producer());
-            QString caption = Util::baseName(resource);
+            QString caption = Util::baseName(resource, true);
             if(warpSpeed != 1.0)
                 caption = QString("%1 (%2x)").arg(caption).arg(warpSpeed);
             m_producer->set(kShotcutCaptionProperty, caption.toUtf8().constData());
@@ -1224,12 +1328,10 @@ void AvformatProducerWidget::on_actionDisableProxy_triggered(bool checked)
         if (producer()->get_int(kIsProxyProperty) && producer()->get(kOriginalResourceProperty)) {
             Mlt::Producer original(MLT.profile(), producer()->get(kOriginalResourceProperty));
             if (original.is_valid()) {
-                if (!qstrcmp(original.get("mlt_service"), "avformat")) {
-                    original.set("mlt_service", "avformat-novalidate");
-                    original.set("mute_on_pause", 0);
-                }
-                original.set(kDisableProxyProperty, 1);
-                MAIN.replaceAllByHash(Util::getHash(original), original, true);
+                Mlt::Producer* producer = MLT.setupNewProducer(&original);
+                producer->set(kDisableProxyProperty, 1);
+                MAIN.replaceAllByHash(Util::getHash(original), *producer, true);
+                delete producer;
             }
         }
     } else {
@@ -1267,11 +1369,9 @@ void AvformatProducerWidget::on_actionDeleteProxy_triggered()
     if (producer()->get_int(kIsProxyProperty) && producer()->get(kOriginalResourceProperty)) {
         Mlt::Producer original(MLT.profile(), producer()->get(kOriginalResourceProperty));
         if (original.is_valid()) {
-            if (!qstrcmp(original.get("mlt_service"), "avformat")) {
-                original.set("mlt_service", "avformat-novalidate");
-                original.set("mute_on_pause", 0);
-            }
-            MAIN.replaceAllByHash(hash, original, true);
+            Mlt::Producer* producer = MLT.setupNewProducer(&original);
+            MAIN.replaceAllByHash(hash, *producer, true);
+            delete producer;
         }
     }
 }
@@ -1340,3 +1440,54 @@ void AvformatProducerWidget::on_actionSetEquirectangular_triggered()
         }
     }
 }
+
+void AvformatProducerWidget::on_actionFFmpegVideoQuality_triggered()
+{
+    QString caption = tr("Choose the Other Video");
+    QFileInfo info(GetFilenameFromProducer(producer()));
+    QString directory = QString("%1/%2 - ERP.%3").arg(info.path(), info.completeBaseName(), info.suffix());
+    QString filePath = QFileDialog::getOpenFileName(&MAIN, caption, directory, QString(),
+        nullptr, Util::getFileDialogOptions());
+    if (!filePath.isEmpty()) {
+        QString resource = GetFilenameFromProducer(producer());
+        QDir dir = QmlApplication::dataDir();
+        dir.cd("vmaf");
+        QStringList args;
+        args << "-hide_banner";
+        args << "-i" << resource;
+        args << "-i" << filePath;
+        args << "-filter_complex";
+        int width = m_producer->get_int("meta.media.width");
+        int height = m_producer->get_int("meta.media.height");
+        double fps = m_producer->get_double("meta.media.frame_rate_num");
+        int frameRateNum, frameRateDen;
+        if (m_producer->get_double("meta.media.frame_rate_den") > 0) {
+            fps /= m_producer->get_double("meta.media.frame_rate_den");
+        }
+        if (m_producer->get("force_fps")) {
+            fps = m_producer->get_double("fps");
+        }
+        Util::normalizeFrameRate(fps, frameRateNum, frameRateDen);
+        auto colorRange = (ui->rangeComboBox->currentIndex() == 1)? "full" : "limited";
+
+        args << QString("[0:v]fps=%4/%5,setpts=PTS-STARTPTS[reference];[1:v]scale=%1:%2:out_range=%6:flags=bicubic,fps=%4/%5,setpts=PTS-STARTPTS[distorted];[distorted][reference]libvmaf=log_fmt=csv:log_path=/dev/stderr:psnr=true:ssim=true:shortest=true:n_threads=%7:model_path=%3")
+                .arg(width).arg(height)
+                .arg((width < 3840 && height < 2160)? dir.filePath("vmaf_v0.6.1.json") : dir.filePath("vmaf_4k_v0.6.1.json"))
+                .arg(frameRateNum).arg(frameRateDen)
+                .arg(colorRange)
+                .arg(qRound(QThread::idealThreadCount() / 2.));
+        args << "-f" << "null" << "pipe:";
+        FfmpegJob* job = new FfmpegJob(resource, args);
+        job->setLabel(tr("Measure %1").arg(Util::baseName(filePath)));
+        JOBS.add(job);
+    }
+}
+
+void AvformatProducerWidget::on_rotationComboBox_activated(int index)
+{
+    if (m_producer) {
+        MLT.stop();
+        m_producer->set("rotate", index * 90);
+        recreateProducer();
+    }
+}
