@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2020 Meltytech, LLC
+ * Copyright (c) 2012-2021 Meltytech, LLC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,10 +43,11 @@ public:
 public slots:
     void updateDuration();
     void rename();
+    void offerConvert(QString message, bool set709Convert = false, bool setSubClip = false);
 
 signals:
     void producerChanged(Mlt::Producer*);
-    void producerReopened();
+    void producerReopened(bool play);
     void modified();
 
 protected:
@@ -55,7 +56,7 @@ protected:
 private slots:
     void onFrameDisplayed(const SharedFrame&);
 
-    void onProducerChanged();
+    void onProducerChanged(Mlt::Producer*);
 
     void onFrameDecoded();
 
@@ -119,17 +120,21 @@ private slots:
 
     void on_actionSetEquirectangular_triggered();
 
+    void on_actionFFmpegVideoQuality_triggered();
+
+    void on_rotationComboBox_activated(int index);
+
 private:
     Ui::AvformatProducerWidget *ui;
     int m_defaultDuration;
     bool m_recalcDuration;
     bool m_askToConvert;
-    bool m_userDefinedCaption;
 
     void reopen(Mlt::Producer* p);
     void recreateProducer();
     void convert(TranscodeDialog& dialog);
     bool revertToOriginalResource();
+    void setSyncVisibility();
 };
 
 
